@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     17/09/98
-// RCS-ID:      $Id: stattext.h,v 1.16 2006/07/25 14:31:55 ABX Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -12,7 +12,7 @@
 #ifndef _WX_STATTEXT_H_
 #define _WX_STATTEXT_H_
 
-class WXDLLEXPORT wxStaticText: public wxStaticTextBase
+class WXDLLIMPEXP_CORE wxStaticText: public wxStaticTextBase
 {
     DECLARE_DYNAMIC_CLASS(wxStaticText)
 
@@ -52,6 +52,10 @@ public:
     virtual WXWidget GetLabelWidget() const
         { return m_labelWidget; }
 
+    virtual void DoSetLabel(const wxString& str);
+    virtual wxString DoGetLabel() const;
+
+    virtual wxSize DoGetBestSize() const;
 protected:
     WXWidget              m_labelWidget;
 };
