@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: button.h,v 1.30 2006/02/08 21:46:18 VZ Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -16,10 +16,10 @@
 // Pushbutton
 // ----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxButton : public wxButtonBase
+class WXDLLIMPEXP_CORE wxButton : public wxButtonBase
 {
 public:
-    wxButton() { }
+    wxButton() { m_imageData = NULL; }
     wxButton(wxWindow *parent,
              wxWindowID id,
              const wxString& label = wxEmptyString,
@@ -29,6 +29,8 @@ public:
              const wxValidator& validator = wxDefaultValidator,
              const wxString& name = wxButtonNameStr)
     {
+        m_imageData = NULL;
+
         Create(parent, id, label, pos, size, style, validator, name);
     }
 
@@ -43,20 +45,24 @@ public:
 
     virtual ~wxButton();
 
-    virtual void SetDefault();
+    virtual wxWindow *SetDefault();
+
+    // overridden base class methods
+    virtual void SetLabel(const wxString& label);
+    virtual bool SetBackgroundColour(const wxColour &colour);
+    virtual bool SetForegroundColour(const wxColour &colour);
 
     // implementation from now on
     virtual void Command(wxCommandEvent& event);
     virtual WXLRESULT MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam);
     virtual bool MSWCommand(WXUINT param, WXWORD id);
 
-    // coloured buttons support
-    virtual bool SetBackgroundColour(const wxColour &colour);
-    virtual bool SetForegroundColour(const wxColour &colour);
-
     virtual bool MSWOnDraw(WXDRAWITEMSTRUCT *item);
     virtual WXDWORD MSWGetStyle(long style, WXDWORD *exstyle) const;
 
+    // returns true if the platform should explicitly apply a theme border
+    virtual bool CanApplyThemeBorder() const { return false; }
+
 private:
     void MakeOwnerDrawn();
 
@@ -74,9 +80,21 @@ protected:
     // usually overridden base class virtuals
     virtual wxSize DoGetBestSize() const;
 
+    virtual bool DoGetAuthNeeded() const;
+    virtual void DoSetAuthNeeded(bool show);
+    virtual wxBitmap DoGetBitmap(State which) const;
+    virtual void DoSetBitmap(const wxBitmap& bitmap, State which);
+    virtual wxSize DoGetBitmapMargins() const;
+    virtual void DoSetBitmapMargins(wxCoord x, wxCoord y);
+    virtual void DoSetBitmapPosition(wxDirection dir);
+
+    class wxButtonImageData *m_imageData;
+
+    // true if the UAC symbol is shown
+    bool m_authNeeded;
+
 private:
     DECLARE_DYNAMIC_CLASS_NO_COPY(wxButton)
 };
 
-#endif
-    // _WX_BUTTON_H_
+#endif // _WX_BUTTON_H_
