@@ -2,7 +2,7 @@
 // Name:        dnd.h
 // Purpose:     declaration of wxDropTarget, wxDropSource classes
 // Author:      Julian Smart
-// RCS-ID:      $Id: dnd.h,v 1.8 2006/09/05 20:46:54 VZ Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Vadim Zeitlin, Robert Roebling, Julian Smart
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -23,35 +23,35 @@
 // classes
 //-------------------------------------------------------------------------
 
-class WXDLLEXPORT wxWindow;
+class WXDLLIMPEXP_FWD_CORE wxWindow;
 
-class WXDLLEXPORT wxDropTarget;
-class WXDLLEXPORT wxTextDropTarget;
-class WXDLLEXPORT wxFileDropTarget;
-class WXDLLEXPORT wxPrivateDropTarget;
+class WXDLLIMPEXP_FWD_CORE wxDropTarget;
+class WXDLLIMPEXP_FWD_CORE wxTextDropTarget;
+class WXDLLIMPEXP_FWD_CORE wxFileDropTarget;
+class WXDLLIMPEXP_FWD_CORE wxPrivateDropTarget;
 
-class WXDLLEXPORT wxDropSource;
+class WXDLLIMPEXP_FWD_CORE wxDropSource;
 
 //-------------------------------------------------------------------------
 // wxDropTarget
 //-------------------------------------------------------------------------
 
-class WXDLLEXPORT wxDropTarget: public wxObject
+class WXDLLIMPEXP_CORE wxDropTarget: public wxObject
 {
 public:
-    
+
     wxDropTarget();
     virtual ~wxDropTarget();
-    
+
     virtual void OnEnter() { }
     virtual void OnLeave() { }
     virtual bool OnDrop( long x, long y, const void *data, size_t size ) = 0;
-    
-    // Override these to indicate what kind of data you support: 
-    
+
+    // Override these to indicate what kind of data you support:
+
     virtual size_t GetFormatCount() const = 0;
     virtual wxDataFormat GetFormat(size_t n) const = 0;
-    
+
     // implementation
 };
 
@@ -59,16 +59,16 @@ public:
 // wxTextDropTarget
 //-------------------------------------------------------------------------
 
-class WXDLLEXPORT wxTextDropTarget: public wxDropTarget
+class WXDLLIMPEXP_CORE wxTextDropTarget: public wxDropTarget
 {
 public:
-    
+
     wxTextDropTarget() {};
     virtual bool OnDrop( long x, long y, const void *data, size_t size );
     virtual bool OnDropText( long x, long y, const char *psz );
-    
+
 protected:
-    
+
     virtual size_t GetFormatCount() const;
     virtual wxDataFormat GetFormat(size_t n) const;
 };
@@ -77,29 +77,29 @@ protected:
 // wxPrivateDropTarget
 //-------------------------------------------------------------------------
 
-class WXDLLEXPORT wxPrivateDropTarget: public wxDropTarget
+class WXDLLIMPEXP_CORE wxPrivateDropTarget: public wxDropTarget
 {
 public:
-    
+
     wxPrivateDropTarget();
-    
+
     // you have to override OnDrop to get at the data
-    
+
     // the string ID identifies the format of clipboard or DnD data. a word
     // processor would e.g. add a wxTextDataObject and a wxPrivateDataObject
     // to the clipboard - the latter with the Id "WXWORD_FORMAT".
-    
+
     void SetId( const wxString& id )
     { m_id = id; }
-    
+
     wxString GetId()
     { return m_id; }
-    
+
 private:
-    
+
     virtual size_t GetFormatCount() const;
     virtual wxDataFormat GetFormat(size_t n) const;
-    
+
     wxString   m_id;
 };
 
@@ -107,18 +107,18 @@ private:
 // A drop target which accepts files (dragged from File Manager or Explorer)
 // ----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxFileDropTarget: public wxDropTarget
+class WXDLLIMPEXP_CORE wxFileDropTarget: public wxDropTarget
 {
 public:
-    
+
     wxFileDropTarget() {};
-    
+
     virtual bool OnDrop( long x, long y, const void *data, size_t size );
-    virtual bool OnDropFiles( long x, long y, 
+    virtual bool OnDropFiles( long x, long y,
         size_t nFiles, const char * const aszFiles[] );
-    
+
 protected:
-    
+
     virtual size_t GetFormatCount() const;
     virtual wxDataFormat GetFormat(size_t n) const;
 };
@@ -136,29 +136,29 @@ enum wxDragResult
         wxDragCancel    // the operation was cancelled by user (not an error)
 };
 
-class WXDLLEXPORT wxDropSource: public wxObject
+class WXDLLIMPEXP_CORE wxDropSource: public wxObject
 {
 public:
-    
+
     wxDropSource( wxWindow *win );
     wxDropSource( wxDataObject &data, wxWindow *win );
-    
+
     virtual ~wxDropSource(void);
-    
+
     void SetData( wxDataObject &data  );
     wxDragResult DoDragDrop(int flags = wxDrag_CopyOnly);
-    
+
     virtual bool GiveFeedback( wxDragResult WXUNUSED(effect), bool WXUNUSED(bScrolling) ) { return TRUE; };
-    
+
     // implementation
 #if 0
     void RegisterWindow(void);
     void UnregisterWindow(void);
-    
+
     wxWindow      *m_window;
     wxDragResult   m_retValue;
     wxDataObject  *m_data;
-    
+
     wxCursor      m_defaultCursor;
     wxCursor      m_goaheadCursor;
 #endif
@@ -168,6 +168,6 @@ public:
 
 // wxUSE_DRAG_AND_DROP
 
-#endif  
+#endif
 //_WX_DND_H_
 
