@@ -3,9 +3,9 @@
 // Purpose:     Grid control wxWidgets sample
 // Author:      Michael Bedward
 // Modified by:
-// RCS-ID:      $Id: griddemo.h,v 1.8 2005/06/02 12:04:11 JS Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Michael Bedward, Julian Smart
-// Licence:     wxWindows license
+// Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 
@@ -28,7 +28,6 @@ class GridFrame : public wxFrame
     wxTextCtrl     *logWin;
     wxLogTextCtrl  *logger;
 #endif // wxUSE_LOG
-    wxString       logBuf;
 
     void SetDefaults();
 
@@ -37,8 +36,12 @@ class GridFrame : public wxFrame
     void ToggleEditing( wxCommandEvent& );
     void ToggleRowSizing( wxCommandEvent& );
     void ToggleColSizing( wxCommandEvent& );
+    void ToggleColMoving( wxCommandEvent& );
     void ToggleGridSizing( wxCommandEvent& );
     void ToggleGridDragCell ( wxCommandEvent& );
+    void SetNativeColHeader ( wxCommandEvent& );
+    void SetCustomColHeader( wxCommandEvent& );
+    void SetDefaultColHeader( wxCommandEvent& );
     void ToggleGridLines( wxCommandEvent& );
     void AutoSizeCols( wxCommandEvent& );
     void CellOverflow( wxCommandEvent& );
@@ -63,6 +66,7 @@ class GridFrame : public wxFrame
     void SelectCells( wxCommandEvent& );
     void SelectRows( wxCommandEvent& );
     void SelectCols( wxCommandEvent& );
+    void SelectRowsOrCols( wxCommandEvent& );
 
     void DeselectCell(wxCommandEvent& event);
     void DeselectCol(wxCommandEvent& event);
@@ -73,6 +77,15 @@ class GridFrame : public wxFrame
     void SelectRow(wxCommandEvent& event);
     void SelectAll(wxCommandEvent& event);
     void OnAddToSelectToggle(wxCommandEvent& event);
+    void OnShowSelection(wxCommandEvent& event);
+
+    void AutoSizeRow(wxCommandEvent& event);
+    void AutoSizeCol(wxCommandEvent& event);
+    void AutoSizeRowLabel(wxCommandEvent& event);
+    void AutoSizeColLabel(wxCommandEvent& event);
+    void AutoSizeLabelsCol(wxCommandEvent& event);
+    void AutoSizeLabelsRow(wxCommandEvent& event);
+    void AutoSizeTable(wxCommandEvent& event);
 
     void OnLabelLeftClick( wxGridEvent& );
     void OnCellLeftClick( wxGridEvent& );
@@ -80,6 +93,7 @@ class GridFrame : public wxFrame
     void OnColSize( wxGridSizeEvent& );
     void OnSelectCell( wxGridEvent& );
     void OnRangeSelected( wxGridRangeSelectEvent& );
+    void OnCellValueChanging( wxGridEvent& );
     void OnCellValueChanged( wxGridEvent& );
     void OnCellBeginDrag( wxGridEvent& );
 
@@ -97,7 +111,7 @@ public:
     void About( wxCommandEvent& );
     void OnVTable( wxCommandEvent& );
     void OnBugsTable( wxCommandEvent& );
-    void OnSmallGrid( wxCommandEvent& );
+    void OnTabularTable( wxCommandEvent& );
 
     enum
     {
@@ -106,6 +120,7 @@ public:
         ID_TOGGLEEDIT,
         ID_TOGGLEROWSIZING,
         ID_TOGGLECOLSIZING,
+        ID_TOGGLECOLMOVING,
         ID_TOGGLEGRIDSIZING,
         ID_TOGGLEGRIDDRAGCELL,
         ID_TOGGLEGRIDLINES,
@@ -121,6 +136,9 @@ public:
         ID_COLLABELALIGN,
         ID_COLLABELHORIZALIGN,
         ID_COLLABELVERTALIGN,
+        ID_COLDEFAULTHEADER,
+        ID_COLNATIVEHEADER,
+        ID_COLCUSTOMHEADER,
         ID_GRIDLINECOLOUR,
         ID_INSERTROW,
         ID_INSERTCOL,
@@ -131,12 +149,14 @@ public:
         ID_SELCELLS,
         ID_SELROWS,
         ID_SELCOLS,
+        ID_SELROWSORCOLS,
         ID_SET_CELL_FG_COLOUR,
         ID_SET_CELL_BG_COLOUR,
         ID_VTABLE,
         ID_BUGS_TABLE,
-        ID_SMALL_GRID,
+        ID_TABULAR_TABLE,
         ID_SELECT_UNSELECT,
+        ID_SHOW_SELECTION,
         ID_SELECT_ALL,
         ID_SELECT_ROW,
         ID_SELECT_COL,
@@ -145,6 +165,13 @@ public:
         ID_DESELECT_ROW,
         ID_DESELECT_COL,
         ID_DESELECT_CELL,
+        ID_SIZE_ROW,
+        ID_SIZE_COL,
+        ID_SIZE_ROW_LABEL,
+        ID_SIZE_COL_LABEL,
+        ID_SIZE_LABELS_COL,
+        ID_SIZE_LABELS_ROW,
+        ID_SIZE_GRID,
 
         ID_SET_HIGHLIGHT_WIDTH,
         ID_SET_RO_HIGHLIGHT_WIDTH,
@@ -231,7 +258,7 @@ private:
 class BugsGridTable : public wxGridTableBase
 {
 public:
-    BugsGridTable(){};
+    BugsGridTable() { }
 
     virtual int GetNumberRows();
     virtual int GetNumberCols();
