@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Jurgen Doornik
 // Created:     25/01/99
-// RCS-ID:      $Id: ipcsetup.h,v 1.3 2006/01/26 19:05:50 ABX Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -15,14 +15,19 @@
 #include "wx/ipc.h"
 
 // the default service name
-#define IPC_SERVICE _T("4242")
-//#define IPC_SERVICE _T("/tmp/wxsrv424")
+#define IPC_SERVICE "4242"
+//#define IPC_SERVICE wxT("/tmp/wxsrv424")
 
 // the hostname
-#define IPC_HOST _T("localhost")
+#define IPC_HOST "localhost"
 
 // the IPC topic
-#define IPC_TOPIC _T("IPC TEST")
+#define IPC_TOPIC "IPC TEST"
 
 // the name of the item we're being advised about
-#define IPC_ADVISE_NAME _T("Item")
+#define IPC_ADVISE_NAME "Item"
+
+
+// the values used by tests/benchmarks/ipcclient.cpp
+#define IPC_BENCHMARK_TOPIC "wxIPC BENCH"
+#define IPC_BENCHMARK_ITEM "Benchmark"
