@@ -4,35 +4,32 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: treetest.cpp,v 1.98 2005/04/06 22:32:24 VZ Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
-// Licence:     wxWindows license
+// Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#ifdef __GNUG__
-  #pragma implementation
-  #pragma interface
-#endif
-
 // For compilers that support precompilation, includes "wx/wx.h".
 #include "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-  #pragma hdrstop
+    #pragma hdrstop
 #endif
 
 #ifndef WX_PRECOMP
-  #include "wx/wx.h"
-  #include "wx/log.h"
+    #include "wx/wx.h"
+    #include "wx/log.h"
 #endif
 
 #include "wx/colordlg.h"
 #include "wx/numdlg.h"
 
+#include "wx/artprov.h"
 #include "wx/image.h"
 #include "wx/imaglist.h"
 #include "wx/treectrl.h"
 #include "wx/math.h"
+#include "wx/renderer.h"
 
 #ifdef __WIN32__
     // this is not supported by native control
@@ -47,10 +44,21 @@
 #include "icon4.xpm"
 #include "icon5.xpm"
 
-#if defined(__WXGTK__) || defined(__WXX11__) || defined(__WXMOTIF__) || defined(__WXMAC__) || defined(__WXMGL__)
-#include "mondrian.xpm"
+#include "state1.xpm"
+#include "state2.xpm"
+#include "state3.xpm"
+#include "state4.xpm"
+#include "state5.xpm"
+
+#include "unchecked.xpm"
+#include "checked.xpm"
+
+#ifndef __WXMSW__
+    #include "../sample.xpm"
 #endif
 
+static const int NUM_CHILDREN_PER_LEVEL = 5;
+static const int NUM_LEVELS = 2;
 
 // verify that the item is ok and insult the user if it is not
 #define CHECK_ITEM( item ) if ( !item.IsOk() ) {                                 \
@@ -69,6 +77,7 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
 
     MENU_LINK(Quit)
     MENU_LINK(About)
+    MENU_LINK(ClearLog)
 
     MENU_LINK(TogButtons)
     MENU_LINK(TogTwist)
@@ -82,12 +91,14 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     MENU_LINK(SetBgColour)
     MENU_LINK(ResetStyle)
 
+    MENU_LINK(Highlight)
     MENU_LINK(Dump)
 #ifndef NO_MULTIPLE_SELECTION
     MENU_LINK(DumpSelected)
     MENU_LINK(Select)
     MENU_LINK(Unselect)
     MENU_LINK(ToggleSel)
+    MENU_LINK(SelectChildren)
 #endif // NO_MULTIPLE_SELECTION
     MENU_LINK(Rename)
     MENU_LINK(Count)
@@ -101,10 +112,14 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     MENU_LINK(DeleteAll)
     MENU_LINK(Recreate)
     MENU_LINK(ToggleImages)
+    MENU_LINK(ToggleStates)
+    MENU_LINK(ToggleAlternateImages)
+    MENU_LINK(ToggleAlternateStates)
     MENU_LINK(ToggleButtons)
     MENU_LINK(SetImageSize)
     MENU_LINK(CollapseAndReset)
     MENU_LINK(EnsureVisible)
+    MENU_LINK(SetFocus)
     MENU_LINK(AddItem)
     MENU_LINK(InsertItem)
     MENU_LINK(IncIndent)
@@ -112,7 +127,22 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
     MENU_LINK(IncSpacing)
     MENU_LINK(DecSpacing)
     MENU_LINK(ToggleIcon)
+    MENU_LINK(ToggleState)
     MENU_LINK(SelectRoot)
+    MENU_LINK(SetFocusedRoot)
+    MENU_LINK(ClearFocused)
+
+    MENU_LINK(ShowFirstVisible)
+#ifdef wxHAS_LAST_VISIBLE
+    MENU_LINK(ShowLastVisible)
+#endif // wxHAS_LAST_VISIBLE
+    MENU_LINK(ShowNextVisible)
+    MENU_LINK(ShowPrevVisible)
+    MENU_LINK(ShowParent)
+    MENU_LINK(ShowPrevSibling)
+    MENU_LINK(ShowNextSibling)
+    MENU_LINK(ScrollTo)
+    MENU_LINK(SelectLast)
 #undef MENU_LINK
 
 END_EVENT_TABLE()
@@ -141,9 +171,13 @@ BEGIN_EVENT_TABLE(MyTreeCtrl, wxTreeCtrl)
     EVT_TREE_SEL_CHANGING(TreeTest_Ctrl, MyTreeCtrl::OnSelChanging)
     EVT_TREE_KEY_DOWN(TreeTest_Ctrl, MyTreeCtrl::OnTreeKeyDown)
     EVT_TREE_ITEM_ACTIVATED(TreeTest_Ctrl, MyTreeCtrl::OnItemActivated)
+    EVT_TREE_STATE_IMAGE_CLICK(TreeTest_Ctrl, MyTreeCtrl::OnItemStateClick)
 
-    // so many differents ways to handle right mouse button clicks...
+    // so many different ways to handle right mouse button clicks...
     EVT_CONTEXT_MENU(MyTreeCtrl::OnContextMenu)
+    // EVT_TREE_ITEM_MENU is the preferred event for creating context menus
+    // on a tree control, because it includes the point of the click or item,
+    // meaning that no additional placement calculations are required.
     EVT_TREE_ITEM_MENU(TreeTest_Ctrl, MyTreeCtrl::OnItemMenu)
     EVT_TREE_ITEM_RIGHT_CLICK(TreeTest_Ctrl, MyTreeCtrl::OnItemRClick)
 
@@ -156,6 +190,9 @@ IMPLEMENT_APP(MyApp)
 
 bool MyApp::OnInit()
 {
+    if ( !wxApp::OnInit() )
+        return false;
+
     // Create the main frame window
     MyFrame *frame = new MyFrame(wxT("wxTreeCtrl Test"), 50, 50, 450, 600);
 
@@ -181,7 +218,7 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     SetBackgroundColour(wxColour(255, 255, 255));
 
     // Give it an icon
-    SetIcon(wxICON(mondrian));
+    SetIcon(wxICON(sample));
 
 #if wxUSE_MENUS
     // Make a menubar
@@ -190,6 +227,8 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
            *tree_menu = new wxMenu,
            *item_menu = new wxMenu;
 
+    file_menu->Append(TreeTest_ClearLog, wxT("&Clear log\tCtrl-L"));
+    file_menu->AppendSeparator();
     file_menu->Append(TreeTest_About, wxT("&About..."));
     file_menu->AppendSeparator();
     file_menu->Append(TreeTest_Quit, wxT("E&xit\tAlt-X"));
@@ -205,9 +244,12 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     style_menu->AppendCheckItem(TreeTest_TogFullHighlight, wxT("Toggle &full row highlight"));
     style_menu->AppendCheckItem(TreeTest_TogEdit, wxT("Toggle &edit mode"));
 #ifndef NO_MULTIPLE_SELECTION
-    style_menu->AppendCheckItem(TreeTest_ToggleSel, wxT("Toggle &selection mode"));
+    style_menu->AppendCheckItem(TreeTest_ToggleSel, wxT("Toggle &selection mode\tCtrl-S"));
 #endif // NO_MULTIPLE_SELECTION
     style_menu->AppendCheckItem(TreeTest_ToggleImages, wxT("Toggle show ima&ges"));
+    style_menu->AppendCheckItem(TreeTest_ToggleStates, wxT("Toggle show st&ates"));
+    style_menu->AppendCheckItem(TreeTest_ToggleAlternateImages, wxT("Toggle alternate images"));
+    style_menu->AppendCheckItem(TreeTest_ToggleAlternateStates, wxT("Toggle alternate state images"));
     style_menu->Append(TreeTest_SetImageSize, wxT("Set image si&ze..."));
     style_menu->AppendSeparator();
     style_menu->Append(TreeTest_SetFgColour, wxT("Set &foreground colour..."));
@@ -224,6 +266,10 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     tree_menu->Append(TreeTest_DeleteChildren, wxT("Delete &children"));
     tree_menu->Append(TreeTest_DeleteAll, wxT("Delete &all items"));
     tree_menu->Append(TreeTest_SelectRoot, wxT("Select root item"));
+    tree_menu->AppendSeparator();
+    tree_menu->Append(TreeTest_SetFocusedRoot, wxT("Set focus to root item"));
+    tree_menu->Append(TreeTest_ClearFocused, wxT("Reset focus"));
+
     tree_menu->AppendSeparator();
     tree_menu->Append(TreeTest_Count, wxT("Count children of current item"));
     tree_menu->Append(TreeTest_CountRec, wxT("Recursively count children of current item"));
@@ -232,6 +278,7 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     tree_menu->Append(TreeTest_SortRev, wxT("Sort in reversed order"));
     tree_menu->AppendSeparator();
     tree_menu->Append(TreeTest_EnsureVisible, wxT("Make the last item &visible"));
+    tree_menu->Append(TreeTest_SetFocus, wxT("Set &focus to the tree"));
     tree_menu->AppendSeparator();
     tree_menu->Append(TreeTest_IncIndent, wxT("Add 5 points to indentation\tAlt-I"));
     tree_menu->Append(TreeTest_DecIndent, wxT("Reduce indentation by 5 points\tAlt-R"));
@@ -247,12 +294,30 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     item_menu->Append(TreeTest_ClearBold, wxT("Make item &not bold"));
     item_menu->AppendSeparator();
     item_menu->Append(TreeTest_ToggleIcon, wxT("Toggle the item's &icon"));
+    item_menu->Append(TreeTest_ToggleState, wxT("Toggle the item's &state"));
+    item_menu->AppendSeparator();
+    item_menu->Append(TreeTest_ShowFirstVisible, wxT("Show &first visible"));
+#ifdef wxHAS_LAST_VISIBLE
+    item_menu->Append(TreeTest_ShowLastVisible, wxT("Show &last visible"));
+#endif // wxHAS_LAST_VISIBLE
+    item_menu->Append(TreeTest_ShowNextVisible, wxT("Show &next visible"));
+    item_menu->Append(TreeTest_ShowPrevVisible, wxT("Show &previous visible"));
+    item_menu->AppendSeparator();
+    item_menu->Append(TreeTest_ShowParent, "Show pa&rent");
+    item_menu->Append(TreeTest_ShowPrevSibling, "Show &previous sibling");
+    item_menu->Append(TreeTest_ShowNextSibling, "Show &next sibling");
+    item_menu->AppendSeparator();
+    item_menu->Append(TreeTest_ScrollTo, "Scroll &to item",
+                      "Scroll to the last by one top level child");
+    item_menu->Append(TreeTest_SelectLast, "Select &last item",
+                      "Select the last item");
 
 #ifndef NO_MULTIPLE_SELECTION
     item_menu->AppendSeparator();
     item_menu->Append(TreeTest_DumpSelected, wxT("Dump selected items\tAlt-D"));
     item_menu->Append(TreeTest_Select, wxT("Select current item\tAlt-S"));
     item_menu->Append(TreeTest_Unselect, wxT("Unselect everything\tAlt-U"));
+    item_menu->Append(TreeTest_SelectChildren, wxT("Select all children\tCtrl-A"));
 #endif // NO_MULTIPLE_SELECTION
 
     wxMenuBar *menu_bar = new wxMenuBar;
@@ -263,9 +328,11 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     SetMenuBar(menu_bar);
 #endif // wxUSE_MENUS
 
+    m_panel = new wxPanel(this);
+
 #if wxUSE_LOG
     // create the controls
-    m_textCtrl = new wxTextCtrl(this, wxID_ANY, wxT(""),
+    m_textCtrl = new wxTextCtrl(m_panel, wxID_ANY, wxT(""),
                                 wxDefaultPosition, wxDefaultSize,
                                 wxTE_MULTILINE | wxSUNKEN_BORDER);
 #endif // wxUSE_LOG
@@ -273,6 +340,9 @@ MyFrame::MyFrame(const wxString& title, int x, int y, int w, int h)
     CreateTreeWithDefStyle();
 
     menu_bar->Check(TreeTest_ToggleImages, true);
+    menu_bar->Check(TreeTest_ToggleStates, true);
+    menu_bar->Check(TreeTest_ToggleAlternateImages, false);
+    menu_bar->Check(TreeTest_ToggleAlternateStates, false);
 
 #if wxUSE_STATUSBAR
     // create a status bar
@@ -324,9 +394,12 @@ void MyFrame::CreateTreeWithDefStyle()
 
 void MyFrame::CreateTree(long style)
 {
-    m_treeCtrl = new MyTreeCtrl(this, TreeTest_Ctrl,
+    m_treeCtrl = new MyTreeCtrl(m_panel, TreeTest_Ctrl,
                                 wxDefaultPosition, wxDefaultSize,
                                 style);
+
+    GetMenuBar()->Enable(TreeTest_SelectRoot, !(style & wxTR_HIDE_ROOT));
+
     Resize();
 }
 
@@ -352,14 +425,20 @@ void MyFrame::OnIdle(wxIdleEvent& event)
     if ( m_treeCtrl )
     {
         wxTreeItemId idRoot = m_treeCtrl->GetRootItem();
+        wxString status;
+        if (idRoot.IsOk())
+        {
+            wxTreeItemId idLast = m_treeCtrl->GetLastChild(idRoot);
+            status = wxString::Format(
+                wxT("Root/last item is %svisible/%svisible"),
+                m_treeCtrl->IsVisible(idRoot) ? wxT("") : wxT("not "),
+                idLast.IsOk() && m_treeCtrl->IsVisible(idLast)
+                    ? wxT("") : wxT("not "));
+        }
+        else
+            status = wxT("No root item");
 
-        SetStatusText(wxString::Format
-                      (
-                        _T("Root/last item is %svisible/%svisible"),
-                        m_treeCtrl->IsVisible(idRoot) ? _T("") : _T("not "),
-                        m_treeCtrl->IsVisible(m_treeCtrl->GetLastChild(idRoot))
-                            ? _T("") : _T("not ")
-                      ), 1);
+        SetStatusText(status, 1);
     }
 #endif // wxUSE_STATUSBAR
 
@@ -384,12 +463,11 @@ void MyFrame::Resize()
 {
     wxSize size = GetClientSize();
     m_treeCtrl->SetSize(0, 0, size.x, size.y
-#if !wxUSE_LOG
-                                            );
-#else
+#if wxUSE_LOG
                                             *2/3);
-    m_textCtrl->SetSize(0, 2*size.y/3, size.x, size.y/3);
+    m_textCtrl->SetSize(0, 2*size.y/3, size.x, size.y/3
 #endif
+    );
 }
 
 void MyFrame::OnQuit(wxCommandEvent& WXUNUSED(event))
@@ -405,9 +483,14 @@ void MyFrame::OnAbout(wxCommandEvent& WXUNUSED(event))
                  wxOK | wxICON_INFORMATION, this);
 }
 
+void MyFrame::OnClearLog(wxCommandEvent& WXUNUSED(event))
+{
+    m_textCtrl->Clear();
+}
+
 void MyFrame::OnRename(wxCommandEvent& WXUNUSED(event))
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
@@ -428,7 +511,7 @@ void MyFrame::OnRename(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnCount(wxCommandEvent& WXUNUSED(event))
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
@@ -439,7 +522,7 @@ void MyFrame::OnCount(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnCountRec(wxCommandEvent& WXUNUSED(event))
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
@@ -450,16 +533,36 @@ void MyFrame::OnCountRec(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::DoSort(bool reverse)
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
     m_treeCtrl->DoSortChildren(item, reverse);
 }
 
+void MyFrame::OnHighlight(wxCommandEvent& WXUNUSED(event))
+{
+    wxTreeItemId id = m_treeCtrl->GetFocusedItem();
+
+    CHECK_ITEM( id );
+
+    wxRect r;
+    if ( !m_treeCtrl->GetBoundingRect(id, r, true /* text, not full row */) )
+    {
+        wxLogMessage(wxT("Failed to get bounding item rect"));
+        return;
+    }
+
+    wxClientDC dc(m_treeCtrl);
+    dc.SetBrush(*wxRED);
+    dc.SetPen(*wxTRANSPARENT_PEN);
+    dc.DrawRectangle(r);
+    m_treeCtrl->Update();
+}
+
 void MyFrame::OnDump(wxCommandEvent& WXUNUSED(event))
 {
-    wxTreeItemId root = m_treeCtrl->GetSelection();
+    wxTreeItemId root = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( root );
 
@@ -478,7 +581,7 @@ void MyFrame::OnDumpSelected(wxCommandEvent& WXUNUSED(event))
     wxArrayTreeItemIds array;
 
     size_t count = m_treeCtrl->GetSelections(array);
-    wxLogMessage(wxT("%u items selected"), count);
+    wxLogMessage(wxT("%u items selected"), unsigned(count));
 
     for ( size_t n = 0; n < count; n++ )
     {
@@ -488,12 +591,24 @@ void MyFrame::OnDumpSelected(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnSelect(wxCommandEvent& WXUNUSED(event))
 {
-    m_treeCtrl->SelectItem(m_treeCtrl->GetSelection());
+    m_treeCtrl->SelectItem(m_treeCtrl->GetFocusedItem());
 }
 
 void MyFrame::OnSelectRoot(wxCommandEvent& WXUNUSED(event))
 {
-    m_treeCtrl->SelectItem(m_treeCtrl->GetRootItem());
+    if ( !m_treeCtrl->HasFlag(wxTR_HIDE_ROOT) )
+        m_treeCtrl->SelectItem(m_treeCtrl->GetRootItem());
+}
+
+void MyFrame::OnSetFocusedRoot(wxCommandEvent& WXUNUSED(event))
+{
+    if ( !m_treeCtrl->HasFlag(wxTR_HIDE_ROOT) )
+        m_treeCtrl->SetFocusedItem(m_treeCtrl->GetRootItem());
+}
+
+void MyFrame::OnClearFocused(wxCommandEvent& WXUNUSED(event))
+{
+    m_treeCtrl->ClearFocusedItem();
 }
 
 void MyFrame::OnUnselect(wxCommandEvent& WXUNUSED(event))
@@ -501,11 +616,20 @@ void MyFrame::OnUnselect(wxCommandEvent& WXUNUSED(event))
     m_treeCtrl->UnselectAll();
 }
 
+void MyFrame::OnSelectChildren(wxCommandEvent& WXUNUSED(event))
+{
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
+    if ( !item.IsOk() )
+        item = m_treeCtrl->GetRootItem();
+
+    m_treeCtrl->SelectChildren(item);
+}
+
 #endif // NO_MULTIPLE_SELECTION
 
 void MyFrame::DoSetBold(bool bold)
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
@@ -514,7 +638,7 @@ void MyFrame::DoSetBold(bool bold)
 
 void MyFrame::OnDelete(wxCommandEvent& WXUNUSED(event))
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
@@ -523,7 +647,7 @@ void MyFrame::OnDelete(wxCommandEvent& WXUNUSED(event))
 
 void MyFrame::OnDeleteChildren(wxCommandEvent& WXUNUSED(event))
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
@@ -538,7 +662,7 @@ void MyFrame::OnDeleteAll(wxCommandEvent& WXUNUSED(event))
 void MyFrame::OnRecreate(wxCommandEvent& event)
 {
     OnDeleteAll(event);
-    m_treeCtrl->AddTestItemsToTree(5, 2);
+    m_treeCtrl->AddTestItemsToTree(NUM_CHILDREN_PER_LEVEL, NUM_LEVELS);
 }
 
 void MyFrame::OnSetImageSize(wxCommandEvent& WXUNUSED(event))
@@ -568,6 +692,41 @@ void MyFrame::OnToggleImages(wxCommandEvent& WXUNUSED(event))
     }
 }
 
+void MyFrame::OnToggleStates(wxCommandEvent& WXUNUSED(event))
+{
+    if ( wxGetApp().ShowStates() )
+    {
+        m_treeCtrl->CreateStateImageList(true);
+        wxGetApp().SetShowStates(false);
+    }
+    else
+    {
+        m_treeCtrl->CreateStateImageList(false);
+        wxGetApp().SetShowStates(true);
+    }
+}
+
+void MyFrame::OnToggleAlternateImages(wxCommandEvent& WXUNUSED(event))
+{
+    bool alternateImages = m_treeCtrl->AlternateImages();
+
+    m_treeCtrl->SetAlternateImages(!alternateImages);
+    m_treeCtrl->CreateImageList(0);
+}
+
+void MyFrame::OnToggleAlternateStates(wxCommandEvent& WXUNUSED(event))
+{
+    bool alternateStates = m_treeCtrl->AlternateStates();
+
+    m_treeCtrl->SetAlternateStates(!alternateStates);
+    m_treeCtrl->CreateStateImageList();
+
+    // normal states < alternate states
+    // so we must reset broken states
+    if ( alternateStates )
+        m_treeCtrl->ResetBrokenStateImages();
+}
+
 void MyFrame::OnToggleButtons(wxCommandEvent& WXUNUSED(event))
 {
 #if USE_GENERIC_TREECTRL || !defined(__WXMSW__)
@@ -594,6 +753,11 @@ void MyFrame::OnEnsureVisible(wxCommandEvent& WXUNUSED(event))
     m_treeCtrl->DoEnsureVisible();
 }
 
+void MyFrame::OnSetFocus(wxCommandEvent& WXUNUSED(event))
+{
+    m_treeCtrl->SetFocus();
+}
+
 void MyFrame::OnInsertItem(wxCommandEvent& WXUNUSED(event))
 {
     int image = wxGetApp().ShowImages() ? MyTreeCtrl::TreeCtrlIcon_File : -1;
@@ -630,25 +794,111 @@ void MyFrame::OnIncSpacing(wxCommandEvent& WXUNUSED(event))
 {
     unsigned int indent = m_treeCtrl->GetSpacing();
     if (indent < 100)
+    {
         m_treeCtrl->SetSpacing( indent+5 );
+        m_treeCtrl->Refresh();
+    }
 }
 
 void MyFrame::OnDecSpacing(wxCommandEvent& WXUNUSED(event))
 {
     unsigned int indent = m_treeCtrl->GetSpacing();
     if (indent > 10)
+    {
         m_treeCtrl->SetSpacing( indent-5 );
+        m_treeCtrl->Refresh();
+    } 
 }
 
 void MyFrame::OnToggleIcon(wxCommandEvent& WXUNUSED(event))
 {
-    wxTreeItemId item = m_treeCtrl->GetSelection();
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
 
     CHECK_ITEM( item );
 
     m_treeCtrl->DoToggleIcon(item);
 }
 
+void MyFrame::OnToggleState(wxCommandEvent& WXUNUSED(event))
+{
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
+
+    CHECK_ITEM( item );
+
+    m_treeCtrl->DoToggleState(item);
+}
+
+void MyFrame::DoShowFirstOrLast(TreeFunc0_t pfn, const wxString& label)
+{
+    const wxTreeItemId item = (m_treeCtrl->*pfn)();
+
+    if ( !item.IsOk() )
+    {
+        wxLogMessage("There is no %s item", label);
+    }
+    else
+    {
+        wxLogMessage("The %s item is \"%s\"",
+                     label, m_treeCtrl->GetItemText(item));
+    }
+}
+
+void MyFrame::DoShowRelativeItem(TreeFunc1_t pfn, const wxString& label)
+{
+    wxTreeItemId item = m_treeCtrl->GetFocusedItem();
+
+    CHECK_ITEM( item );
+
+    if ((pfn == (TreeFunc1_t) &wxTreeCtrl::GetPrevVisible
+         || pfn == (TreeFunc1_t) &wxTreeCtrl::GetNextVisible)
+        && !m_treeCtrl->IsVisible(item))
+    {
+        wxLogMessage("The selected item must be visible.");
+        return;
+    }
+
+    wxTreeItemId new_item = (m_treeCtrl->*pfn)(item);
+
+    if ( !new_item.IsOk() )
+    {
+        wxLogMessage("There is no %s item", label);
+    }
+    else
+    {
+        wxLogMessage("The %s item is \"%s\"",
+                     label, m_treeCtrl->GetItemText(new_item));
+    }
+}
+
+void MyFrame::OnScrollTo(wxCommandEvent& WXUNUSED(event))
+{
+    // scroll to the last but one top level child
+    wxTreeItemId item = m_treeCtrl->GetPrevSibling(
+                            m_treeCtrl->GetLastChild(
+                                m_treeCtrl->GetRootItem()));
+    CHECK_ITEM( item );
+
+    m_treeCtrl->ScrollTo(item);
+}
+
+void MyFrame::OnSelectLast(wxCommandEvent& WXUNUSED(event))
+{
+    // select the very last item of the tree
+    wxTreeItemId item = m_treeCtrl->GetRootItem();
+    for ( ;; )
+    {
+        wxTreeItemId itemChild = m_treeCtrl->GetLastChild(item);
+        if ( !itemChild.IsOk() )
+            break;
+
+        item = itemChild;
+    }
+
+    CHECK_ITEM( item );
+
+    m_treeCtrl->SelectItem(item);
+}
+
 void MyFrame::OnSetFgColour(wxCommandEvent& WXUNUSED(event))
 {
     wxColour col = wxGetColourFromUser(this, m_treeCtrl->GetForegroundColour());
@@ -673,14 +923,17 @@ IMPLEMENT_DYNAMIC_CLASS(MyTreeCtrl, wxTreeCtrl)
 MyTreeCtrl::MyTreeCtrl(wxWindow *parent, const wxWindowID id,
                        const wxPoint& pos, const wxSize& size,
                        long style)
-          : wxTreeCtrl(parent, id, pos, size, style)
+          : wxTreeCtrl(parent, id, pos, size, style),
+            m_alternateImages(false),
+            m_alternateStates(false)
 {
     m_reverseSort = false;
 
     CreateImageList();
+    CreateStateImageList();
 
     // Add some items to the tree
-    AddTestItemsToTree(5, 2);
+    AddTestItemsToTree(NUM_CHILDREN_PER_LEVEL, NUM_LEVELS);
 }
 
 void MyTreeCtrl::CreateImageList(int size)
@@ -701,15 +954,29 @@ void MyTreeCtrl::CreateImageList(int size)
     // should correspond to TreeCtrlIcon_xxx enum
     wxBusyCursor wait;
     wxIcon icons[5];
-    icons[0] = wxIcon(icon1_xpm);
-    icons[1] = wxIcon(icon2_xpm);
-    icons[2] = wxIcon(icon3_xpm);
-    icons[3] = wxIcon(icon4_xpm);
-    icons[4] = wxIcon(icon5_xpm);
 
-    int sizeOrig = icons[0].GetWidth();
+    if (m_alternateImages)
+    {
+        icons[TreeCtrlIcon_File] = wxIcon(icon1_xpm);
+        icons[TreeCtrlIcon_FileSelected] = wxIcon(icon2_xpm);
+        icons[TreeCtrlIcon_Folder] = wxIcon(icon3_xpm);
+        icons[TreeCtrlIcon_FolderSelected] = wxIcon(icon4_xpm);
+        icons[TreeCtrlIcon_FolderOpened] = wxIcon(icon5_xpm);
+    }
+    else
+    {
+        wxSize iconSize(size, size);
+
+        icons[TreeCtrlIcon_File] =
+        icons[TreeCtrlIcon_FileSelected] = wxArtProvider::GetIcon(wxART_NORMAL_FILE, wxART_LIST, iconSize);
+        icons[TreeCtrlIcon_Folder] =
+        icons[TreeCtrlIcon_FolderSelected] =
+        icons[TreeCtrlIcon_FolderOpened] = wxArtProvider::GetIcon(wxART_FOLDER, wxART_LIST, iconSize);
+    }
+
     for ( size_t i = 0; i < WXSIZEOF(icons); i++ )
     {
+        int sizeOrig = icons[0].GetWidth();
         if ( size == sizeOrig )
         {
             images->Add(icons[i]);
@@ -723,6 +990,54 @@ void MyTreeCtrl::CreateImageList(int size)
     AssignImageList(images);
 }
 
+void MyTreeCtrl::CreateStateImageList(bool del)
+{
+    if ( del )
+    {
+        SetStateImageList(NULL);
+        return;
+    }
+
+    wxImageList *states;
+    wxBusyCursor wait;
+
+    if (m_alternateStates)
+    {
+        wxIcon icons[5];
+        icons[0] = wxIcon(state1_xpm);  // yellow
+        icons[1] = wxIcon(state2_xpm);  // green
+        icons[2] = wxIcon(state3_xpm);  // red
+        icons[3] = wxIcon(state4_xpm);  // blue
+        icons[4] = wxIcon(state5_xpm);  // black
+
+        int width  = icons[0].GetWidth(),
+            height = icons[0].GetHeight();
+
+        // Make an state image list containing small icons
+        states = new wxImageList(width, height, true);
+
+        for ( size_t i = 0; i < WXSIZEOF(icons); i++ )
+            states->Add(icons[i]);
+    }
+    else
+    {
+        wxIcon icons[2];
+        icons[0] = wxIcon(unchecked_xpm);
+        icons[1] = wxIcon(checked_xpm);
+        
+        int width  = icons[0].GetWidth(),
+            height = icons[0].GetHeight();
+            
+        // Make an state image list containing small icons
+        states = new wxImageList(width, height, true);
+
+        for ( size_t i = 0; i < WXSIZEOF(icons); i++ )
+            states->Add(icons[i]);
+    }
+
+    AssignStateImageList(states);
+}
+
 #if USE_GENERIC_TREECTRL || !defined(__WXMSW__)
 void MyTreeCtrl::CreateButtonsImageList(int size)
 {
@@ -738,12 +1053,25 @@ void MyTreeCtrl::CreateButtonsImageList(int size)
     // should correspond to TreeCtrlIcon_xxx enum
     wxBusyCursor wait;
     wxIcon icons[4];
-    icons[0] = wxIcon(icon3_xpm);   // closed
-    icons[1] = wxIcon(icon3_xpm);   // closed, selected
-    icons[2] = wxIcon(icon5_xpm);   // open
-    icons[3] = wxIcon(icon5_xpm);   // open, selected
 
-    for ( size_t i = 0; i < WXSIZEOF(icons); i++ )
+    if (m_alternateImages)
+    {
+        icons[0] = wxIcon(icon3_xpm);   // closed
+        icons[1] = wxIcon(icon3_xpm);   // closed, selected
+        icons[2] = wxIcon(icon5_xpm);   // open
+        icons[3] = wxIcon(icon5_xpm);   // open, selected
+    }
+    else
+    {
+        wxSize iconSize(size, size);
+
+        icons[0] =                                                                 // closed
+        icons[1] = wxArtProvider::GetIcon(wxART_FOLDER, wxART_LIST, iconSize);     // closed, selected
+        icons[2] =                                                                 // open
+        icons[3] = wxArtProvider::GetIcon(wxART_FOLDER_OPEN, wxART_LIST, iconSize);// open, selected
+    }
+
+   for ( size_t i = 0; i < WXSIZEOF(icons); i++ )
     {
         int sizeOrig = icons[i].GetWidth();
         if ( size == sizeOrig )
@@ -791,9 +1119,9 @@ void MyTreeCtrl::AddItemsRecursively(const wxTreeItemId& idParent,
         {
             // at depth 1 elements won't have any more children
             if ( hasChildren )
-                str.Printf(wxT("%s child %d"), wxT("Folder"), n + 1);
+                str.Printf(wxT("%s child %u"), wxT("Folder"), unsigned(n + 1));
             else
-                str.Printf(wxT("%s child %d.%d"), wxT("File"), folder, n + 1);
+                str.Printf(wxT("%s child %u.%u"), wxT("File"), unsigned(folder), unsigned(n + 1));
 
             // here we pass to AppendItem() normal and selected item images (we
             // suppose that selected image follows the normal one in the enum)
@@ -810,6 +1138,9 @@ void MyTreeCtrl::AddItemsRecursively(const wxTreeItemId& idParent,
             wxTreeItemId id = AppendItem(idParent, str, image, imageSel,
                                          new MyTreeItemData(str));
 
+            if ( wxGetApp().ShowStates() )
+                SetItemState(id, 0);
+
             // and now we also set the expanded one (only for the folders)
             if ( hasChildren && wxGetApp().ShowImages() )
             {
@@ -836,7 +1167,7 @@ void MyTreeCtrl::AddTestItemsToTree(size_t numChildren,
     wxTreeItemId rootId = AddRoot(wxT("Root"),
                                   image, image,
                                   new MyTreeItemData(wxT("Root item")));
-    if ( image != -1 )
+    if ( !HasFlag(wxTR_HIDE_ROOT) && image != -1 )
     {
         SetItemImage(rootId, TreeCtrlIcon_FolderOpened, wxTreeItemIcon_Expanded);
     }
@@ -844,16 +1175,21 @@ void MyTreeCtrl::AddTestItemsToTree(size_t numChildren,
     AddItemsRecursively(rootId, numChildren, depth, 0);
 
     // set some colours/fonts for testing
-    SetItemFont(rootId, *wxITALIC_FONT);
+    if ( !HasFlag(wxTR_HIDE_ROOT) )
+        SetItemFont(rootId, *wxITALIC_FONT);
 
     wxTreeItemIdValue cookie;
     wxTreeItemId id = GetFirstChild(rootId, cookie);
     SetItemTextColour(id, *wxBLUE);
 
     id = GetNextChild(rootId, cookie);
-    id = GetNextChild(rootId, cookie);
-    SetItemTextColour(id, *wxRED);
-    SetItemBackgroundColour(id, *wxLIGHT_GREY);
+    if ( id )
+        id = GetNextChild(rootId, cookie);
+    if ( id )
+    {
+        SetItemTextColour(id, *wxRED);
+        SetItemBackgroundColour(id, *wxLIGHT_GREY);
+    }
 }
 
 void MyTreeCtrl::GetItemsRecursively(const wxTreeItemId& idParent,
@@ -880,18 +1216,78 @@ void MyTreeCtrl::GetItemsRecursively(const wxTreeItemId& idParent,
 
 void MyTreeCtrl::DoToggleIcon(const wxTreeItemId& item)
 {
-    int image = GetItemImage(item) == TreeCtrlIcon_Folder ? TreeCtrlIcon_File
-                                                          : TreeCtrlIcon_Folder;
+    int image = GetItemImage(item) == TreeCtrlIcon_Folder
+                    ? TreeCtrlIcon_File
+                    : TreeCtrlIcon_Folder;
+    SetItemImage(item, image, wxTreeItemIcon_Normal);
 
-    SetItemImage(item, image);
+    image = GetItemImage(item, wxTreeItemIcon_Selected) == TreeCtrlIcon_FolderSelected
+                    ? TreeCtrlIcon_FileSelected
+                    : TreeCtrlIcon_FolderSelected;
+    SetItemImage(item, image, wxTreeItemIcon_Selected);
 }
 
+void MyTreeCtrl::DoToggleState(const wxTreeItemId& item)
+{
+    if ( m_alternateStates )
+    {
+        // sets random state unlike current
+        int state = GetItemState(item);
+        int nState;
+
+        srand (time(NULL));
+        do {
+            nState = rand() % GetStateImageList()->GetImageCount();
+        } while (nState == state);
+
+        SetItemState(item, nState);
+    }
+    else
+    {
+        // we have only 2 checkbox states, so next state will be reversed
+        SetItemState(item, wxTREE_ITEMSTATE_NEXT);
+    }
+}
+
+void MyTreeCtrl::DoResetBrokenStateImages(const wxTreeItemId& idParent,
+                                          wxTreeItemIdValue cookie, int state)
+{
+    wxTreeItemId id;
+
+    if ( !cookie )
+        id = GetFirstChild(idParent, cookie);
+    else
+        id = GetNextChild(idParent, cookie);
+
+    if ( !id.IsOk() )
+        return;
+
+    int curState = GetItemState(id);
+    if ( curState != wxTREE_ITEMSTATE_NONE && curState > state )
+        SetItemState(id, state);
+
+    if (ItemHasChildren(id))
+        DoResetBrokenStateImages(id, 0, state);
+
+    DoResetBrokenStateImages(idParent, cookie, state);
+}
+
+void MyTreeCtrl::LogEvent(const wxChar *name, const wxTreeEvent& event)
+{
+    wxTreeItemId item = event.GetItem();
+    wxString text;
+    if ( item.IsOk() )
+        text << wxT('"') << GetItemText(item).c_str() << wxT('"');
+    else
+        text = wxT("invalid item");
+    wxLogMessage(wxT("%s(%s)"), name, text.c_str());
+}
 
 // avoid repetition
 #define TREE_EVENT_HANDLER(name)                                 \
 void MyTreeCtrl::name(wxTreeEvent& event)                        \
 {                                                                \
-    wxLogMessage(wxT(#name));                                    \
+    LogEvent(wxT(#name), event);                                  \
     SetLastItem(wxTreeItemId());                                 \
     event.Skip();                                                \
 }
@@ -933,8 +1329,6 @@ void LogKeyEvent(const wxChar *name, const wxKeyEvent& event)
             case WXK_MENU: key = wxT("MENU"); break;
             case WXK_PAUSE: key = wxT("PAUSE"); break;
             case WXK_CAPITAL: key = wxT("CAPITAL"); break;
-            case WXK_PRIOR: key = wxT("PRIOR"); break;
-            case WXK_NEXT: key = wxT("NEXT"); break;
             case WXK_END: key = wxT("END"); break;
             case WXK_HOME: key = wxT("HOME"); break;
             case WXK_LEFT: key = wxT("LEFT"); break;
@@ -1003,7 +1397,6 @@ void LogKeyEvent(const wxChar *name, const wxKeyEvent& event)
             case WXK_NUMPAD_UP: key = wxT("NUMPAD_UP"); break;
             case WXK_NUMPAD_RIGHT: key = wxT("NUMPAD_RIGHT"); break;
             case WXK_NUMPAD_DOWN: key = wxT("NUMPAD_DOWN"); break;
-            case WXK_NUMPAD_PRIOR: key = wxT("NUMPAD_PRIOR"); break;
             case WXK_NUMPAD_PAGEUP: key = wxT("NUMPAD_PAGEUP"); break;
             case WXK_NUMPAD_PAGEDOWN: key = wxT("NUMPAD_PAGEDOWN"); break;
             case WXK_NUMPAD_END: key = wxT("NUMPAD_END"); break;
@@ -1052,8 +1445,12 @@ void MyTreeCtrl::OnBeginDrag(wxTreeEvent& event)
     {
         m_draggedItem = event.GetItem();
 
-        wxLogMessage(wxT("OnBeginDrag: started dragging %s"),
-                     GetItemText(m_draggedItem).c_str());
+        wxPoint clientpt = event.GetPoint();
+        wxPoint screenpt = ClientToScreen(clientpt);
+
+        wxLogMessage(wxT("OnBeginDrag: started dragging %s at screen coords (%i,%i)"),
+                     GetItemText(m_draggedItem).c_str(),
+                     screenpt.x, screenpt.y);
 
         event.Allow();
     }
@@ -1095,7 +1492,10 @@ void MyTreeCtrl::OnEndDrag(wxTreeEvent& event)
     // Finally, we only copy one item here but we might copy the entire tree if
     // we were dragging a folder.
     int image = wxGetApp().ShowImages() ? TreeCtrlIcon_File : -1;
-    AppendItem(itemDst, text, image);
+    wxTreeItemId id = AppendItem(itemDst, text, image);
+
+    if ( wxGetApp().ShowStates() )
+        SetItemState(id, GetItemState(itemSrc));
 }
 
 void MyTreeCtrl::OnBeginLabelEdit(wxTreeEvent& event)
@@ -1113,7 +1513,7 @@ void MyTreeCtrl::OnBeginLabelEdit(wxTreeEvent& event)
     else if ( itemId == GetRootItem() )
     {
         // test that it is possible to change the text of the item being edited
-        SetItemText(itemId, _T("Editing root item"));
+        SetItemText(itemId, wxT("Editing root item"));
     }
 }
 
@@ -1158,48 +1558,38 @@ void MyTreeCtrl::OnItemActivated(wxTreeEvent& event)
     wxLogMessage(wxT("OnItemActivated"));
 }
 
+void MyTreeCtrl::OnItemStateClick(wxTreeEvent& event)
+{
+    // toggle item state
+    wxTreeItemId itemId = event.GetItem();
+    DoToggleState(itemId);
+
+    wxLogMessage(wxT("Item \"%s\" state changed to %d"),
+                 GetItemText(itemId), GetItemState(itemId));
+}
+
 void MyTreeCtrl::OnItemMenu(wxTreeEvent& event)
 {
     wxTreeItemId itemId = event.GetItem();
     MyTreeItemData *item = itemId.IsOk() ? (MyTreeItemData *)GetItemData(itemId)
                                          : NULL;
+    wxPoint clientpt = event.GetPoint();
+    wxPoint screenpt = ClientToScreen(clientpt);
 
-    wxLogMessage(wxT("OnItemMenu for item \"%s\""), item ? item->GetDesc()
-                                                         : _T(""));
+    wxLogMessage(wxT("OnItemMenu for item \"%s\" at screen coords (%i, %i)"),
+                 item ? item->GetDesc() : wxT(""), screenpt.x, screenpt.y);
 
+    ShowMenu(itemId, clientpt);
     event.Skip();
 }
 
 void MyTreeCtrl::OnContextMenu(wxContextMenuEvent& event)
 {
     wxPoint pt = event.GetPosition();
-    wxTreeItemId item;
-    if ( !HasFlag(wxTR_MULTIPLE) )
-        item = GetSelection();
+
     wxLogMessage(wxT("OnContextMenu at screen coords (%i, %i)"), pt.x, pt.y);
 
-    // check if event was generated by keyboard (MSW-specific?)
-    if ( pt.x == -1 && pt.y == -1 ) //(this is how MSW indicates it)
-    {
-        // attempt to guess where to show the menu
-        if ( item.IsOk() )
-        {
-            // if an item was clicked, show menu to the right of it
-            wxRect rect;
-            GetBoundingRect(item, rect, true /* only the label */);
-            pt = wxPoint(rect.GetRight(), rect.GetTop());
-        }
-        else
-        {
-            pt = wxPoint(0, 0);
-        }
-    }
-    else // event was generated by mouse, use supplied coords
-    {
-        pt = ScreenToClient(pt);
-    }
-
-    ShowMenu(item, pt);
+    event.Skip();
 }
 
 void MyTreeCtrl::ShowMenu(wxTreeItemId id, const wxPoint& pt)
@@ -1217,6 +1607,8 @@ void MyTreeCtrl::ShowMenu(wxTreeItemId id, const wxPoint& pt)
 #if wxUSE_MENUS
     wxMenu menu(title);
     menu.Append(TreeTest_About, wxT("&About..."));
+    menu.AppendSeparator();
+    menu.Append(TreeTest_Highlight, wxT("&Highlight item"));
     menu.Append(TreeTest_Dump, wxT("&Dump"));
 
     PopupMenu(&menu, pt);
@@ -1230,7 +1622,7 @@ void MyTreeCtrl::OnItemRClick(wxTreeEvent& event)
                                          : NULL;
 
     wxLogMessage(wxT("Item \"%s\" right clicked"), item ? item->GetDesc()
-                                                        : _T(""));
+                                                        : wxT(""));
 
     event.Skip();
 }
@@ -1253,12 +1645,16 @@ void MyTreeCtrl::OnRMouseDClick(wxMouseEvent& event)
 {
     wxTreeItemId id = HitTest(event.GetPosition());
     if ( !id )
+    {
         wxLogMessage(wxT("No item under mouse"));
+    }
     else
     {
         MyTreeItemData *item = (MyTreeItemData *)GetItemData(id);
         if ( item )
+        {
             wxLogMessage(wxT("Item '%s' under mouse"), item->GetDesc());
+        }
     }
 
     event.Skip();
@@ -1277,6 +1673,6 @@ void MyTreeItemData::ShowInfo(wxTreeCtrl *tree)
                  Bool2String(tree->IsSelected(GetId())),
                  Bool2String(tree->IsExpanded(GetId())),
                  Bool2String(tree->IsBold(GetId())),
-                 tree->GetChildrenCount(GetId()),
-                 tree->GetChildrenCount(GetId(), false));
+                 unsigned(tree->GetChildrenCount(GetId())),
+                 unsigned(tree->GetChildrenCount(GetId(), false)));
 }
