@@ -4,9 +4,9 @@
 // Purpose:     Part of the widgets sample showing wxSlider
 // Author:      Vadim Zeitlin
 // Created:     16.04.01
-// Id:          $Id: slider.cpp,v 1.25 2005/04/10 18:07:08 VZ Exp $
+// Id:          $Id$
 // Copyright:   (c) 2001 Vadim Zeitlin
-// License:     wxWindows license
+// Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 // ============================================================================
@@ -60,12 +60,16 @@ enum
     SliderPage_Clear,
     SliderPage_SetValue,
     SliderPage_SetMinAndMax,
+    SliderPage_SetLineSize,
+    SliderPage_SetPageSize,
     SliderPage_SetTickFreq,
     SliderPage_SetThumbLen,
     SliderPage_CurValueText,
     SliderPage_ValueText,
     SliderPage_MinText,
     SliderPage_MaxText,
+    SliderPage_LineSizeText,
+    SliderPage_PageSizeText,
     SliderPage_TickFreqText,
     SliderPage_ThumbLenText,
     SliderPage_RadioSides,
@@ -76,6 +80,7 @@ enum
 // sides radiobox values
 enum
 {
+    SliderTicks_None,
     SliderTicks_Top,
     SliderTicks_Bottom,
     SliderTicks_Left,
@@ -89,10 +94,14 @@ enum
 class SliderWidgetsPage : public WidgetsPage
 {
 public:
-    SliderWidgetsPage(wxBookCtrl *book, wxImageList *imaglist);
+    SliderWidgetsPage(WidgetsBookCtrl *book, wxImageList *imaglist);
     virtual ~SliderWidgetsPage(){};
 
     virtual wxControl *GetWidget() const { return m_slider; }
+    virtual void RecreateWidget() { CreateSlider(); }
+
+    // lazy creation of the content
+    virtual void CreateContent();
 
 protected:
     // event handlers
@@ -100,6 +109,8 @@ protected:
     void OnButtonClear(wxCommandEvent& event);
     void OnButtonSetValue(wxCommandEvent& event);
     void OnButtonSetMinAndMax(wxCommandEvent& event);
+    void OnButtonSetLineSize(wxCommandEvent& event);
+    void OnButtonSetPageSize(wxCommandEvent& event);
     void OnButtonSetTickFreq(wxCommandEvent& event);
     void OnButtonSetThumbLen(wxCommandEvent& event);
 
@@ -109,6 +120,8 @@ protected:
 
     void OnUpdateUIValueButton(wxUpdateUIEvent& event);
     void OnUpdateUIMinMaxButton(wxUpdateUIEvent& event);
+    void OnUpdateUILineSize(wxUpdateUIEvent& event);
+    void OnUpdateUIPageSize(wxUpdateUIEvent& event);
     void OnUpdateUITickFreq(wxUpdateUIEvent& event);
     void OnUpdateUIThumbLen(wxUpdateUIEvent& event);
     void OnUpdateUIRadioSides(wxUpdateUIEvent& event);
@@ -124,6 +137,12 @@ protected:
     // (re)create the slider
     void CreateSlider();
 
+    // set the line size from the text field value
+    void DoSetLineSize();
+
+    // set the page size from the text field value
+    void DoSetPageSize();
+
     // set the tick frequency from the text field value
     void DoSetTickFreq();
 
@@ -141,7 +160,8 @@ protected:
     // ------------
 
     // the check/radio boxes for styles
-    wxCheckBox *m_chkLabels,
+    wxCheckBox *m_chkMinMaxLabels,
+               *m_chkValueLabel,
                *m_chkInverse,
                *m_chkTicks,
                *m_chkBothSides;
@@ -156,6 +176,8 @@ protected:
     wxTextCtrl *m_textValue,
                *m_textMin,
                *m_textMax,
+               *m_textLineSize,
+               *m_textPageSize,
                *m_textTickFreq,
                *m_textThumbLen;
 
@@ -172,14 +194,17 @@ BEGIN_EVENT_TABLE(SliderWidgetsPage, WidgetsPage)
     EVT_BUTTON(SliderPage_Reset, SliderWidgetsPage::OnButtonReset)
     EVT_BUTTON(SliderPage_SetValue, SliderWidgetsPage::OnButtonSetValue)
     EVT_BUTTON(SliderPage_SetMinAndMax, SliderWidgetsPage::OnButtonSetMinAndMax)
+    EVT_BUTTON(SliderPage_SetLineSize, SliderWidgetsPage::OnButtonSetLineSize)
+    EVT_BUTTON(SliderPage_SetPageSize, SliderWidgetsPage::OnButtonSetPageSize)
     EVT_BUTTON(SliderPage_SetTickFreq, SliderWidgetsPage::OnButtonSetTickFreq)
     EVT_BUTTON(SliderPage_SetThumbLen, SliderWidgetsPage::OnButtonSetThumbLen)
 
     EVT_UPDATE_UI(SliderPage_SetValue, SliderWidgetsPage::OnUpdateUIValueButton)
     EVT_UPDATE_UI(SliderPage_SetMinAndMax, SliderWidgetsPage::OnUpdateUIMinMaxButton)
+    EVT_UPDATE_UI(SliderPage_SetLineSize, SliderWidgetsPage::OnUpdateUILineSize)
+    EVT_UPDATE_UI(SliderPage_SetPageSize, SliderWidgetsPage::OnUpdateUIPageSize)
     EVT_UPDATE_UI(SliderPage_SetTickFreq, SliderWidgetsPage::OnUpdateUITickFreq)
     EVT_UPDATE_UI(SliderPage_SetThumbLen, SliderWidgetsPage::OnUpdateUIThumbLen)
-    EVT_UPDATE_UI(SliderPage_TickFreqText, SliderWidgetsPage::OnUpdateUITickFreq)
     EVT_UPDATE_UI(SliderPage_RadioSides, SliderWidgetsPage::OnUpdateUIRadioSides)
     EVT_UPDATE_UI(SliderPage_BothSides, SliderWidgetsPage::OnUpdateUIBothSides)
 
@@ -197,66 +222,76 @@ END_EVENT_TABLE()
 // implementation
 // ============================================================================
 
-IMPLEMENT_WIDGETS_PAGE(SliderWidgetsPage, _T("Slider"));
+#if defined(__WXUNIVERSAL__)
+    #define FAMILY_CTRLS UNIVERSAL_CTRLS
+#else
+    #define FAMILY_CTRLS NATIVE_CTRLS
+#endif
 
-SliderWidgetsPage::SliderWidgetsPage(wxBookCtrl *book,
+IMPLEMENT_WIDGETS_PAGE(SliderWidgetsPage, wxT("Slider"), FAMILY_CTRLS );
+
+SliderWidgetsPage::SliderWidgetsPage(WidgetsBookCtrl *book,
                                      wxImageList *imaglist)
-                  : WidgetsPage(book)
+                  : WidgetsPage(book, imaglist, slider_xpm)
 {
-    imaglist->Add(wxBitmap(slider_xpm));
-
     // init everything
     m_min = 0;
     m_max = 100;
 
     m_chkInverse =
     m_chkTicks =
-    m_chkLabels =
+    m_chkMinMaxLabels =
+    m_chkValueLabel =
     m_chkBothSides = (wxCheckBox *)NULL;
 
     m_radioSides = (wxRadioBox *)NULL;
 
     m_slider = (wxSlider *)NULL;
     m_sizerSlider = (wxSizer *)NULL;
+}
 
+void SliderWidgetsPage::CreateContent()
+{
     wxSizer *sizerTop = new wxBoxSizer(wxHORIZONTAL);
 
     // left pane
-    wxStaticBox *box = new wxStaticBox(this, wxID_ANY, _T("&Set style"));
+    wxStaticBox *box = new wxStaticBox(this, wxID_ANY, wxT("&Set style"));
     wxSizer *sizerLeft = new wxStaticBoxSizer(box, wxVERTICAL);
 
-    m_chkInverse = CreateCheckBoxAndAddToSizer(sizerLeft, _T("&Inverse"));
-    m_chkTicks = CreateCheckBoxAndAddToSizer(sizerLeft, _T("Show &ticks"));
-    m_chkLabels = CreateCheckBoxAndAddToSizer(sizerLeft, _T("Show &labels"));
+    m_chkInverse = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("&Inverse"));
+    m_chkTicks = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("Show &ticks"));
+    m_chkMinMaxLabels = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("Show min/max &labels"));
+    m_chkValueLabel = CreateCheckBoxAndAddToSizer(sizerLeft, wxT("Show &value label"));
     static const wxString sides[] =
     {
-        _T("top"),
-        _T("bottom"),
-        _T("left"),
-        _T("right"),
+        wxT("default"),
+        wxT("top"),
+        wxT("bottom"),
+        wxT("left"),
+        wxT("right"),
     };
-    m_radioSides = new wxRadioBox(this, SliderPage_RadioSides, _T("&Ticks/Labels"),
+    m_radioSides = new wxRadioBox(this, SliderPage_RadioSides, wxT("&Label position"),
                                  wxDefaultPosition, wxDefaultSize,
                                  WXSIZEOF(sides), sides,
                                  1, wxRA_SPECIFY_COLS);
     sizerLeft->Add(m_radioSides, 0, wxGROW | wxALL, 5);
     m_chkBothSides = CreateCheckBoxAndAddToSizer
-                     (sizerLeft, _T("&Both sides"), SliderPage_BothSides);
+                     (sizerLeft, wxT("&Both sides"), SliderPage_BothSides);
 #if wxUSE_TOOLTIPS
-    m_chkBothSides->SetToolTip( _T("\"Both sides\" is only supported \nin Win95 and Universal") );
+    m_chkBothSides->SetToolTip( wxT("\"Both sides\" is only supported \nin Win95 and Universal") );
 #endif // wxUSE_TOOLTIPS
 
     sizerLeft->Add(5, 5, 0, wxGROW | wxALL, 5); // spacer
 
-    wxButton *btn = new wxButton(this, SliderPage_Reset, _T("&Reset"));
+    wxButton *btn = new wxButton(this, SliderPage_Reset, wxT("&Reset"));
     sizerLeft->Add(btn, 0, wxALIGN_CENTRE_HORIZONTAL | wxALL, 15);
 
     // middle pane
-    wxStaticBox *box2 = new wxStaticBox(this, wxID_ANY, _T("&Change slider value"));
+    wxStaticBox *box2 = new wxStaticBox(this, wxID_ANY, wxT("&Change slider value"));
     wxSizer *sizerMiddle = new wxStaticBoxSizer(box2, wxVERTICAL);
 
     wxTextCtrl *text;
-    wxSizer *sizerRow = CreateSizerWithTextAndLabel(_T("Current value"),
+    wxSizer *sizerRow = CreateSizerWithTextAndLabel(wxT("Current value"),
                                                     SliderPage_CurValueText,
                                                     &text);
     text->SetEditable(false);
@@ -264,35 +299,49 @@ SliderWidgetsPage::SliderWidgetsPage(wxBookCtrl *book,
     sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
 
     sizerRow = CreateSizerWithTextAndButton(SliderPage_SetValue,
-                                            _T("Set &value"),
+                                            wxT("Set &value"),
                                             SliderPage_ValueText,
                                             &m_textValue);
     sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
 
     sizerRow = CreateSizerWithTextAndButton(SliderPage_SetMinAndMax,
-                                            _T("&Min and max"),
+                                            wxT("&Min and max"),
                                             SliderPage_MinText,
                                             &m_textMin);
 
     m_textMax = new wxTextCtrl(this, SliderPage_MaxText, wxEmptyString);
     sizerRow->Add(m_textMax, 1, wxLEFT | wxALIGN_CENTRE_VERTICAL, 5);
 
-    m_textMin->SetValue( wxString::Format(_T("%d"), m_min) );
-    m_textMax->SetValue( wxString::Format(_T("%d"), m_max) );
+    m_textMin->SetValue( wxString::Format(wxT("%d"), m_min) );
+    m_textMax->SetValue( wxString::Format(wxT("%d"), m_max) );
+
+    sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
+
+    sizerRow = CreateSizerWithTextAndButton(SliderPage_SetLineSize,
+                                            wxT("Li&ne size"),
+                                            SliderPage_LineSizeText,
+                                            &m_textLineSize);
+
+    sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
+
+    sizerRow = CreateSizerWithTextAndButton(SliderPage_SetPageSize,
+                                            wxT("P&age size"),
+                                            SliderPage_PageSizeText,
+                                            &m_textPageSize);
 
     sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
 
     sizerRow = CreateSizerWithTextAndButton(SliderPage_SetTickFreq,
-                                            _T("Tick &frequency"),
+                                            wxT("Tick &frequency"),
                                             SliderPage_TickFreqText,
                                             &m_textTickFreq);
 
-    m_textTickFreq->SetValue(_T("10"));
+    m_textTickFreq->SetValue(wxT("10"));
 
     sizerMiddle->Add(sizerRow, 0, wxALL | wxGROW, 5);
 
     sizerRow = CreateSizerWithTextAndButton(SliderPage_SetThumbLen,
-                                            _T("Thumb &length"),
+                                            wxT("Thumb &length"),
                                             SliderPage_ThumbLenText,
                                             &m_textThumbLen);
 
@@ -306,6 +355,9 @@ SliderWidgetsPage::SliderWidgetsPage(wxBookCtrl *book,
     Reset();
     CreateSlider();
 
+    m_textLineSize->SetValue(wxString::Format(wxT("%d"), m_slider->GetLineSize()));
+    m_textPageSize->SetValue(wxString::Format(wxT("%d"), m_slider->GetPageSize()));
+
     // the 3 panes panes compose the window
     sizerTop->Add(sizerLeft, 0, wxGROW | (wxALL & ~wxLEFT), 10);
     sizerTop->Add(sizerMiddle, 0, wxGROW | wxALL, 10);
@@ -313,8 +365,6 @@ SliderWidgetsPage::SliderWidgetsPage(wxBookCtrl *book,
 
     // final initializations
     SetSizer(sizerTop);
-
-    sizerTop->Fit(this);
 }
 
 // ----------------------------------------------------------------------------
@@ -325,24 +375,30 @@ void SliderWidgetsPage::Reset()
 {
     m_chkInverse->SetValue(false);
     m_chkTicks->SetValue(true);
-    m_chkLabels->SetValue(true);
+    m_chkValueLabel->SetValue(true);
+    m_chkMinMaxLabels->SetValue(true);
     m_chkBothSides->SetValue(false);
 
-    m_radioSides->SetSelection(SliderTicks_Top);
+    m_radioSides->SetSelection(SliderTicks_None);
 }
 
 void SliderWidgetsPage::CreateSlider()
 {
-    int flags = 0;
+    int flags = ms_defaultFlags;
 
     if ( m_chkInverse->GetValue() )
     {
         flags |= wxSL_INVERSE;
     }
 
-    if ( m_chkLabels->GetValue() )
+    if ( m_chkMinMaxLabels->GetValue() )
     {
-        flags |= wxSL_LABELS;
+        flags |= wxSL_MIN_MAX_LABELS;
+    }
+
+    if ( m_chkValueLabel->GetValue() )
+    {
+        flags |= wxSL_VALUE_LABEL;
     }
 
     if ( m_chkTicks->GetValue() )
@@ -350,26 +406,32 @@ void SliderWidgetsPage::CreateSlider()
         flags |= wxSL_AUTOTICKS;
     }
 
+    // notice that the style names refer to the _ticks_ positions while we want
+    // to allow the user to select the label(s) positions and the labels are on
+    // the opposite side from the ticks, hence the apparent reversal below
     switch ( m_radioSides->GetSelection() )
     {
+        case SliderTicks_None:
+            break;
+
         case SliderTicks_Top:
-            flags |= wxSL_TOP;
-            break;
-
-        case SliderTicks_Left:
-            flags |= wxSL_LEFT;
-            break;
-
-        case SliderTicks_Bottom:
             flags |= wxSL_BOTTOM;
             break;
 
+        case SliderTicks_Left:
+            flags |= wxSL_RIGHT | wxSL_VERTICAL;
+            break;
+
+        case SliderTicks_Bottom:
+            flags |= wxSL_TOP;
+            break;
+
         case SliderTicks_Right:
-            flags |= wxSL_RIGHT;
+            flags |= wxSL_LEFT | wxSL_VERTICAL;
             break;
 
         default:
-            wxFAIL_MSG(_T("unexpected radiobox selection"));
+            wxFAIL_MSG(wxT("unexpected radiobox selection"));
             // fall through
     }
 
@@ -423,12 +485,48 @@ void SliderWidgetsPage::CreateSlider()
     m_sizerSlider->Layout();
 }
 
+void SliderWidgetsPage::DoSetLineSize()
+{
+    long lineSize;
+    if ( !m_textLineSize->GetValue().ToLong(&lineSize) )
+    {
+        wxLogWarning(wxT("Invalid slider line size"));
+
+        return;
+    }
+
+    m_slider->SetLineSize(lineSize);
+
+    if ( m_slider->GetLineSize() != lineSize )
+    {
+        wxLogWarning(wxT("Invalid line size in slider."));
+    }
+}
+
+void SliderWidgetsPage::DoSetPageSize()
+{
+    long pageSize;
+    if ( !m_textPageSize->GetValue().ToLong(&pageSize) )
+    {
+        wxLogWarning(wxT("Invalid slider page size"));
+
+        return;
+    }
+
+    m_slider->SetPageSize(pageSize);
+
+    if ( m_slider->GetPageSize() != pageSize )
+    {
+        wxLogWarning(wxT("Invalid page size in slider."));
+    }
+}
+
 void SliderWidgetsPage::DoSetTickFreq()
 {
     long freq;
     if ( !m_textTickFreq->GetValue().ToLong(&freq) )
     {
-        wxLogWarning(_T("Invalid slider tick frequency"));
+        wxLogWarning(wxT("Invalid slider tick frequency"));
 
         return;
     }
@@ -441,7 +539,7 @@ void SliderWidgetsPage::DoSetThumbLen()
     long len;
     if ( !m_textThumbLen->GetValue().ToLong(&len) )
     {
-        wxLogWarning(_T("Invalid slider thumb length"));
+        wxLogWarning(wxT("Invalid slider thumb length"));
 
         return;
     }
@@ -460,6 +558,16 @@ void SliderWidgetsPage::OnButtonReset(wxCommandEvent& WXUNUSED(event))
     CreateSlider();
 }
 
+void SliderWidgetsPage::OnButtonSetLineSize(wxCommandEvent& WXUNUSED(event))
+{
+    DoSetLineSize();
+}
+
+void SliderWidgetsPage::OnButtonSetPageSize(wxCommandEvent& WXUNUSED(event))
+{
+    DoSetPageSize();
+}
+
 void SliderWidgetsPage::OnButtonSetTickFreq(wxCommandEvent& WXUNUSED(event))
 {
     DoSetTickFreq();
@@ -478,7 +586,7 @@ void SliderWidgetsPage::OnButtonSetMinAndMax(wxCommandEvent& WXUNUSED(event))
          !m_textMax->GetValue().ToLong(&maxNew) ||
          minNew >= maxNew )
     {
-        wxLogWarning(_T("Invalid min/max values for the slider."));
+        wxLogWarning(wxT("Invalid min/max values for the slider."));
 
         return;
     }
@@ -491,7 +599,7 @@ void SliderWidgetsPage::OnButtonSetMinAndMax(wxCommandEvent& WXUNUSED(event))
     if ( m_slider->GetMin() != m_min ||
          m_slider->GetMax() != m_max )
     {
-        wxLogWarning(_T("Invalid range in slider."));
+        wxLogWarning(wxT("Invalid range in slider."));
     }
 }
 
@@ -500,7 +608,7 @@ void SliderWidgetsPage::OnButtonSetValue(wxCommandEvent& WXUNUSED(event))
     long val;
     if ( !m_textValue->GetValue().ToLong(&val) || !IsValidValue(val) )
     {
-        wxLogWarning(_T("Invalid slider value."));
+        wxLogWarning(wxT("Invalid slider value."));
 
         return;
     }
@@ -514,6 +622,20 @@ void SliderWidgetsPage::OnUpdateUIValueButton(wxUpdateUIEvent& event)
     event.Enable( m_textValue->GetValue().ToLong(&val) && IsValidValue(val) );
 }
 
+void SliderWidgetsPage::OnUpdateUILineSize(wxUpdateUIEvent& event)
+{
+    long lineSize;
+    event.Enable( m_textLineSize->GetValue().ToLong(&lineSize) &&
+            (lineSize > 0) && (lineSize <= m_max - m_min) );
+}
+
+void SliderWidgetsPage::OnUpdateUIPageSize(wxUpdateUIEvent& event)
+{
+    long pageSize;
+    event.Enable( m_textPageSize->GetValue().ToLong(&pageSize) &&
+            (pageSize > 0) && (pageSize <= m_max - m_min) );
+}
+
 void SliderWidgetsPage::OnUpdateUITickFreq(wxUpdateUIEvent& event)
 {
     long freq;
@@ -540,9 +662,10 @@ void SliderWidgetsPage::OnUpdateUIResetButton(wxUpdateUIEvent& event)
 {
     event.Enable( m_chkInverse->GetValue() ||
                   !m_chkTicks->GetValue() ||
-                  !m_chkLabels->GetValue() ||
+                  !m_chkValueLabel->GetValue() ||
+                  !m_chkMinMaxLabels->GetValue() ||
                   m_chkBothSides->GetValue() ||
-                  m_radioSides->GetSelection() != SliderTicks_Top );
+                  m_radioSides->GetSelection() != SliderTicks_None );
 }
 
 void SliderWidgetsPage::OnCheckOrRadioBox(wxCommandEvent& WXUNUSED(event))
@@ -552,21 +675,21 @@ void SliderWidgetsPage::OnCheckOrRadioBox(wxCommandEvent& WXUNUSED(event))
 
 void SliderWidgetsPage::OnUpdateUICurValueText(wxUpdateUIEvent& event)
 {
-    event.SetText( wxString::Format(_T("%d"), m_slider->GetValue()) );
+    event.SetText( wxString::Format(wxT("%d"), m_slider->GetValue()) );
 }
 
 void SliderWidgetsPage::OnUpdateUIRadioSides(wxUpdateUIEvent& event)
 {
-    event.Enable( m_chkLabels->GetValue() || m_chkTicks->GetValue() );
+    event.Enable( m_chkValueLabel->GetValue() || m_chkTicks->GetValue() );
 }
 
 void SliderWidgetsPage::OnUpdateUIBothSides(wxUpdateUIEvent& event)
 {
-#if defined(__WIN95__) || defined(__WXUNIVERSAL__)
+#if defined(__WXMSW__) || defined(__WXUNIVERSAL__)
     event.Enable( m_chkTicks->GetValue() );
 #else
     event.Enable( false );
-#endif // defined(__WIN95__) || defined(__WXUNIVERSAL__)
+#endif // defined(__WXMSW__) || defined(__WXUNIVERSAL__)
 }
 
 void SliderWidgetsPage::OnSlider(wxScrollEvent& event)
@@ -591,7 +714,7 @@ void SliderWidgetsPage::OnSlider(wxScrollEvent& event)
         wxT("wxEVT_SCROLL_PAGEDOWN"),
         wxT("wxEVT_SCROLL_THUMBTRACK"),
         wxT("wxEVT_SCROLL_THUMBRELEASE"),
-        wxT("wxEVT_SCROLL_ENDSCROLL")
+        wxT("wxEVT_SCROLL_CHANGED")
     };
 
     int index = eventType - wxEVT_SCROLL_TOP;
@@ -606,7 +729,7 @@ void SliderWidgetsPage::OnSlider(wxScrollEvent& event)
 
     static int s_numSliderEvents = 0;
 
-    wxLogMessage(wxT("Slider event #%d: %s (pos = %d, int value = %ld)"),
+    wxLogMessage(wxT("Slider event #%d: %s (pos = %d, int value = %d)"),
                  s_numSliderEvents++,
                  eventNames[index],
                  event.GetPosition(),
