@@ -1,38 +1,45 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        src/cocoa/bitmap.cpp
+// Name:        src/cocoa/bitmap.mm
 // Purpose:     wxBitmap
 // Author:      David Elliott
 // Modified by:
 // Created:     2003/07/19
-// RCS-ID:      $Id: bitmap.mm,v 1.22 2005/05/22 23:39:30 DE Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2003 David Elliott
-// Licence:     wxWidgets licence
+// Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
 #include "wx/wxprec.h"
+
+#include "wx/bitmap.h"
+
 #ifndef WX_PRECOMP
     #include "wx/log.h"
     #include "wx/utils.h"
     #include "wx/palette.h"
     #include "wx/icon.h"
     #include "wx/colour.h"
+    #include "wx/image.h"
 #endif //WX_PRECOMP
-#include "wx/bitmap.h"
-#include "wx/image.h"
+
 #include "wx/xpmdecod.h"
 #include "wx/rawbmp.h"
 
 #include "wx/cocoa/autorelease.h"
 #include "wx/cocoa/string.h"
+#include "wx/cocoa/ObjcRef.h"
 
 #import <AppKit/NSBitmapImageRep.h>
 #import <AppKit/NSGraphics.h>
 #import <AppKit/NSImage.h>
 #import <AppKit/NSColor.h>
 
+IMPLEMENT_ABSTRACT_CLASS(wxBitmapHandler, wxBitmapHandlerBase)
+
 // ========================================================================
 // wxBitmapRefData
 // ========================================================================
+
 class wxBitmapRefData: public wxGDIRefData
 {
     friend class wxBitmap;
@@ -41,6 +48,8 @@ public:
     wxBitmapRefData( const wxBitmapRefData& data );
     virtual ~wxBitmapRefData();
 
+    virtual bool IsOk() const { return m_ok; }
+
 protected:
     int                 m_width;
     int                 m_height;
@@ -53,8 +62,6 @@ protected:
     wxMask             *m_bitmapMask; // Optional mask
 };
 
-#define M_BITMAPDATA ((wxBitmapRefData *)m_refData)
-
 wxBitmapRefData::wxBitmapRefData()
 {
     m_ok = FALSE;
@@ -69,6 +76,8 @@ wxBitmapRefData::wxBitmapRefData()
 
 wxBitmapRefData::wxBitmapRefData( const wxBitmapRefData& data)
 {
+    wxAutoNSAutoreleasePool pool;
+
     m_width = data.m_width;
     m_height = data.m_height;
     m_depth = data.m_depth;
@@ -76,13 +85,13 @@ wxBitmapRefData::wxBitmapRefData( const wxBitmapRefData& data)
     m_numColors = data.m_numColors;
     m_bitmapPalette = data.m_bitmapPalette;
     m_quality = data.m_quality;
-    m_cocoaNSBitmapImageRep = [data.m_cocoaNSBitmapImageRep copyWithZone:nil];
+    m_cocoaNSBitmapImageRep = wxGCSafeRetain([[data.m_cocoaNSBitmapImageRep copyWithZone:nil] autorelease]);
     m_bitmapMask = data.m_bitmapMask?new wxMask(*data.m_bitmapMask):NULL;
 }
 
 wxBitmapRefData::~wxBitmapRefData()
 {
-    [m_cocoaNSBitmapImageRep release];
+    wxGCSafeRelease(m_cocoaNSBitmapImageRep);
     m_cocoaNSBitmapImageRep = NULL;
 
     delete m_bitmapMask;
@@ -92,6 +101,9 @@ wxBitmapRefData::~wxBitmapRefData()
 // ========================================================================
 // wxBitmap
 // ========================================================================
+
+#define M_BITMAPDATA ((wxBitmapRefData *)m_refData)
+
 IMPLEMENT_DYNAMIC_CLASS(wxBitmap, wxGDIObject)
 
 wxBitmap::wxBitmap()
@@ -115,12 +127,17 @@ wxBitmap::wxBitmap(const char bits[], int the_width, int the_height, int no_bits
     /* TODO: create the bitmap from data */
 }
 
-wxBitmap::wxBitmap(int w, int h, int d)
+wxBitmap::wxBitmap(NSImage* cocoaNSImage)
 {
-    (void)Create(w, h, d);
+    (void) Create(cocoaNSImage);
 }
 
-wxBitmap::wxBitmap(void *data, wxBitmapType type, int width, int height, int depth)
+wxBitmap::wxBitmap(NSBitmapImageRep* cocoaNSBitmapImageRep)
+{
+    (void) Create(cocoaNSBitmapImageRep);
+}
+
+wxBitmap::wxBitmap(const void* data, wxBitmapType type, int width, int height, int depth)
 {
     (void) Create(data, type, width, height, depth);
 }
@@ -130,12 +147,12 @@ wxBitmap::wxBitmap(const wxString& filename, wxBitmapType type)
     LoadFile(filename, type);
 }
 
-wxObjectRefData *wxBitmap::CreateRefData() const
+wxGDIRefData *wxBitmap::CreateGDIRefData() const
 {
     return new wxBitmapRefData;
 }
 
-wxObjectRefData *wxBitmap::CloneRefData(const wxObjectRefData *data) const
+wxGDIRefData *wxBitmap::CloneGDIRefData(const wxGDIRefData *data) const
 {
     return new wxBitmapRefData(*(wxBitmapRefData*)data);
 }
@@ -178,8 +195,8 @@ void wxBitmap::SetNSBitmapImageRep(WX_NSBitmapImageRep bitmapImageRep)
     if(!M_BITMAPDATA)
         return;
     // NOTE: No checking is done to make sure width/height agree
-    [bitmapImageRep retain];
-    [M_BITMAPDATA->m_cocoaNSBitmapImageRep release];
+    wxGCSafeRetain(bitmapImageRep);
+    wxGCSafeRelease(M_BITMAPDATA->m_cocoaNSBitmapImageRep);
     M_BITMAPDATA->m_cocoaNSBitmapImageRep = bitmapImageRep;
 }
 
@@ -239,11 +256,6 @@ void wxBitmap::SetMask(wxMask *mask)
     M_BITMAPDATA->m_bitmapMask = mask ;
 }
 
-bool wxBitmap::Ok() const
-{
-    return m_refData && M_BITMAPDATA->m_ok;
-}
-
 wxPalette* wxBitmap::GetPalette() const
 {
     if(!m_refData)
@@ -281,6 +293,8 @@ int wxBitmap::GetHeight() const
 
 bool wxBitmap::Create(int w, int h, int d)
 {
+    wxAutoNSAutoreleasePool pool;
+
     UnRef();
 
     m_refData = new wxBitmapRefData;
@@ -290,7 +304,7 @@ bool wxBitmap::Create(int w, int h, int d)
     M_BITMAPDATA->m_depth = d;
 
     /* TODO: create new bitmap */
-    M_BITMAPDATA->m_cocoaNSBitmapImageRep = [[NSBitmapImageRep alloc]
+    M_BITMAPDATA->m_cocoaNSBitmapImageRep = wxGCSafeRetain([[[NSBitmapImageRep alloc]
             initWithBitmapDataPlanes: NULL
             pixelsWide: w
             pixelsHigh: h
@@ -301,7 +315,7 @@ bool wxBitmap::Create(int w, int h, int d)
             colorSpaceName: NSCalibratedRGBColorSpace
             bytesPerRow: 0  // NOTE: Contrary to Apple documentation Mac OS
                             // 10.4 will add padding bytes when 0 is used here
-            bitsPerPixel: 0];
+            bitsPerPixel: 0] autorelease]);
 
     wxLogTrace(wxTRACE_COCOA,wxT("M_BITMAPDATA=%p NSBitmapImageRep bitmapData=%p"), M_BITMAPDATA, [M_BITMAPDATA->m_cocoaNSBitmapImageRep bitmapData]);
     M_BITMAPDATA->m_ok = true;
@@ -330,7 +344,7 @@ bool wxBitmap::LoadFile(const wxString& filename, wxBitmapType type)
         M_BITMAPDATA->m_ok = true;
         M_BITMAPDATA->m_numColors = 0;
         M_BITMAPDATA->m_quality = 0;
-        M_BITMAPDATA->m_cocoaNSBitmapImageRep = [imageRep retain];
+        M_BITMAPDATA->m_cocoaNSBitmapImageRep = wxGCSafeRetain(imageRep);
         M_BITMAPDATA->m_bitmapMask = NULL;
         return true;
     }
@@ -343,7 +357,34 @@ bool wxBitmap::LoadFile(const wxString& filename, wxBitmapType type)
     return true;
 }
 
-bool wxBitmap::Create(void *data, wxBitmapType type, int width, int height, int depth)
+bool wxBitmap::Create(NSImage* cocoaNSImage)
+{
+    wxAutoNSAutoreleasePool pool;
+    NSBitmapImageRep *bitmapImageRep = [NSBitmapImageRep imageRepWithData:[cocoaNSImage TIFFRepresentation]];
+    return Create(bitmapImageRep);
+}
+
+bool wxBitmap::Create(NSBitmapImageRep *imageRep)
+{
+    UnRef();
+    m_refData = new wxBitmapRefData;
+    if(imageRep != nil)
+    {
+        M_BITMAPDATA->m_width = [imageRep pixelsWide];
+        M_BITMAPDATA->m_height = [imageRep pixelsHigh];
+        M_BITMAPDATA->m_depth = [imageRep bitsPerPixel];
+        M_BITMAPDATA->m_ok = true;
+        M_BITMAPDATA->m_numColors = 0;
+        M_BITMAPDATA->m_quality = 0;
+        M_BITMAPDATA->m_cocoaNSBitmapImageRep = wxGCSafeRetain(imageRep);
+        M_BITMAPDATA->m_bitmapMask = NULL;
+        return true;
+    }
+    else
+        return false;
+}
+
+bool wxBitmap::Create(const void* data, wxBitmapType type, int width, int height, int depth)
 {
     UnRef();
 
@@ -359,18 +400,21 @@ bool wxBitmap::SaveFile(const wxString& filename, wxBitmapType type, const wxPal
 
 bool wxBitmap::CopyFromIcon(const wxIcon& icon)
 {
+    // Pool here due to lack of one during wx init phase
+    wxAutoNSAutoreleasePool pool;
+
     UnRef();
     if(!icon.GetNSImage());
     [icon.GetNSImage() lockFocus];
     NSRect imageRect;
     imageRect.origin.x = imageRect.origin.y = 0.0;
     imageRect.size = [icon.GetNSImage() size];
-    NSBitmapImageRep *newBitmapRep = [[NSBitmapImageRep alloc] initWithFocusedViewRect:imageRect];
+    NSBitmapImageRep *newBitmapRep = [[[NSBitmapImageRep alloc] initWithFocusedViewRect:imageRect] autorelease];
     [icon.GetNSImage() unlockFocus];
     if(!newBitmapRep)
         return false;
     m_refData = new wxBitmapRefData;
-    M_BITMAPDATA->m_cocoaNSBitmapImageRep = newBitmapRep;
+    M_BITMAPDATA->m_cocoaNSBitmapImageRep = wxGCSafeRetain(newBitmapRep);
     M_BITMAPDATA->m_width = [newBitmapRep pixelsWide];
     M_BITMAPDATA->m_height = [newBitmapRep pixelsHigh];
     M_BITMAPDATA->m_depth = [newBitmapRep bitsPerSample]*[newBitmapRep samplesPerPixel];
@@ -381,9 +425,26 @@ bool wxBitmap::CopyFromIcon(const wxIcon& icon)
     return true;
 }
 
-wxBitmap wxBitmap::GetSubBitmap(wxRect const&) const
+wxBitmap wxBitmap::GetSubBitmap(const wxRect& rect) const
 {
-    return wxNullBitmap;
+    wxAutoNSAutoreleasePool pool;
+    if(!Ok())
+        return wxNullBitmap;
+    NSImage *nsimage = GetNSImage(false);
+
+    [nsimage lockFocus];
+    NSRect imageRect = {{0,0}, [nsimage size]};
+    imageRect.origin.x = imageRect.size.width * rect.x / GetWidth();
+    imageRect.origin.y = imageRect.size.height * rect.y / GetHeight();
+    imageRect.size.width *= static_cast<CGFloat>(rect.width) / GetWidth();
+    imageRect.size.height *= static_cast<CGFloat>(rect.height) / GetHeight();
+
+    NSBitmapImageRep *newBitmapRep = [[NSBitmapImageRep alloc] initWithFocusedViewRect:imageRect];
+    [nsimage unlockFocus];
+
+    wxBitmap newBitmap(newBitmapRep);
+
+    return (newBitmap);
 }
 
 wxImage wxBitmap::ConvertToImage() const
@@ -410,26 +471,9 @@ wxImage wxBitmap::ConvertToImage() const
     return newImage;
 }
 
-bool wxBitmap::CreateFromXpm(const char **xpm)
-{
-#if wxUSE_IMAGE && wxUSE_XPM
-    UnRef();
-
-    wxCHECK_MSG( xpm, false, wxT("invalid XPM data") )
-
-    wxXPMDecoder decoder;
-    wxImage img = decoder.ReadData(xpm);
-    wxCHECK_MSG( img.Ok(), false, wxT("invalid XPM data") )
-
-    *this = wxBitmap(img);
-    return true;
-#else
-    return false;
-#endif
-}
-
 bool wxBitmap::CreateFromImage(const wxImage& image, int depth)
 {
+    wxAutoNSAutoreleasePool pool;
     UnRef();
 
     wxCHECK_MSG(image.Ok(), false, wxT("invalid image"));
@@ -439,7 +483,7 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth)
 
     M_BITMAPDATA->m_width = image.GetWidth();
     M_BITMAPDATA->m_height = image.GetHeight();
-    NSBitmapImageRep *bitmapImage = [[NSBitmapImageRep alloc]
+    NSBitmapImageRep *bitmapImage = [[[NSBitmapImageRep alloc]
             initWithBitmapDataPlanes: NULL
             pixelsWide: image.GetWidth()
             pixelsHigh: image.GetHeight()
@@ -449,7 +493,7 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth)
             isPlanar: NO
             colorSpaceName: NSCalibratedRGBColorSpace
             bytesPerRow: image.GetWidth()*3
-            bitsPerPixel: 0];
+            bitsPerPixel: 0] autorelease];
 
     // TODO: Specify bytesPerRow:0 and then use [bitmapImage bytesPerRow]
     // so that the rows are aligned suitably for altivec by the OS (Tiger)
@@ -460,7 +504,7 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth)
     M_BITMAPDATA->m_ok = true;
     M_BITMAPDATA->m_numColors = 0;
     M_BITMAPDATA->m_quality = 0;
-    M_BITMAPDATA->m_cocoaNSBitmapImageRep = bitmapImage;
+    M_BITMAPDATA->m_cocoaNSBitmapImageRep = wxGCSafeRetain(bitmapImage);
     M_BITMAPDATA->m_bitmapMask = new wxMask(*this,wxColour(image.GetMaskRed(),image.GetMaskGreen(),image.GetMaskBlue()));
     return true;
 }
@@ -476,7 +520,7 @@ void *wxBitmap::GetRawData(wxPixelDataBase& data, int bpp)
 
     if([bitmapRep bitsPerPixel]!=bpp)
     {
-        wxFAIL_MSG( _T("incorrect bitmap type in wxBitmap::GetRawData()") );
+        wxFAIL_MSG( wxT("incorrect bitmap type in wxBitmap::GetRawData()") );
         return NULL;
     }
     data.m_width = [bitmapRep pixelsWide];
@@ -496,10 +540,6 @@ void wxBitmap::UngetRawData(wxPixelDataBase& data)
 {   // TODO
 }
 
-void wxBitmap::UseAlpha()
-{   // TODO
-}
-
 // ========================================================================
 // wxMask
 // ========================================================================
@@ -536,15 +576,23 @@ wxMask::wxMask(const wxBitmap& bitmap)
     Create(bitmap);
 }
 
+// Copy constructor
+wxMask::wxMask(const wxMask& src)
+:   wxObject(src)
+,   m_cocoaNSBitmapImageRep(wxGCSafeRetain(src.m_cocoaNSBitmapImageRep))
+{
+}
+
 wxMask::~wxMask()
 {
-    [m_cocoaNSBitmapImageRep release];
+    wxGCSafeRelease(m_cocoaNSBitmapImageRep);
 }
 
 // Create a mask from a mono bitmap (copies the bitmap).
 bool wxMask::Create(const wxBitmap& bitmap)
 {
 // TODO
+    wxLogDebug(wxT("Cannot yet create a mask from a mono bitmap"));
     return FALSE;
 }
 
@@ -553,6 +601,7 @@ bool wxMask::Create(const wxBitmap& bitmap)
 bool wxMask::Create(const wxBitmap& bitmap, int paletteIndex)
 {
 // TODO
+    wxLogDebug(wxT("Cannot yet create a mask from a palette bitmap"));
     return FALSE;
 }
 
@@ -659,11 +708,18 @@ bool wxMask::Create(const wxBitmap& bitmap, const wxColour& colour)
         wxCHECK_MSG(wxMask_CreateFromBitmapData(pixelData, colour, dstData),
             false, wxT("Unable to access raw data"));
     }
+    else if([srcBitmapRep bitsPerPixel]==8 && [srcBitmapRep bitsPerSample]==8 && [srcBitmapRep samplesPerPixel]==1 && [srcBitmapRep hasAlpha]==NO)
+    // 8-bpp Grayscale, no alpha
+    {   // Force all RGB to access the same grayscale component
+        typedef wxPixelFormat<unsigned char,8,0,0,0> PixelFormat;
+        wxPixelData<wxBitmap,PixelFormat> pixelData(const_cast<wxBitmap&>(bitmap));
+        wxCHECK_MSG(wxMask_CreateFromBitmapData(pixelData, colour, dstData),
+            false, wxT("Unable to access raw data"));
+    }
     else
     {   wxCHECK_MSG(false,false,wxT("Unimplemented pixel format")); }
 
     // maskRep was autoreleased in case we had to exit quickly
-    m_cocoaNSBitmapImageRep = [maskRep retain];
+    m_cocoaNSBitmapImageRep = wxGCSafeRetain(maskRep);
     return true;
 }
-
