@@ -4,7 +4,7 @@
 // Author:      Andreas Pflug
 // Modified by:
 // Created:     2005-01-19
-// RCS-ID:      $Id: datectlg.cpp 53510 2008-05-09 22:15:55Z RD $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2005 Andreas Pflug <pgadmin@pse-consulting.de>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,13 +25,6 @@
 
 #if wxUSE_DATEPICKCTRL
 
-#include "wx/datectrl.h"
-
-// use this version if we're explicitly requested to do it or if it's the only
-// one we have
-#if !defined(wxHAS_NATIVE_DATEPICKCTRL) || \
-        (defined(wxUSE_DATEPICKCTRL_GENERIC) && wxUSE_DATEPICKCTRL_GENERIC)
-
 #ifndef WX_PRECOMP
     #include "wx/dialog.h"
     #include "wx/dcmemory.h"
@@ -40,31 +33,14 @@
     #include "wx/valtext.h"
 #endif
 
-#ifdef wxHAS_NATIVE_DATEPICKCTRL
-    // this header is not included from wx/datectrl.h if we have a native
-    // version, but we do need it here
-    #include "wx/generic/datectrl.h"
-#else
-    // we need to define _WX_DEFINE_DATE_EVENTS_ before including wx/dateevt.h to
-    // define the event types we use if we're the only date picker control version
-    // being compiled -- otherwise it's defined in the native version implementation
-    #define _WX_DEFINE_DATE_EVENTS_
-#endif
+#include "wx/datectrl.h"
+#include "wx/generic/datectrl.h"
 
-#include "wx/dateevt.h"
-
-#include "wx/calctrl.h"
-#include "wx/combo.h"
 
 // ----------------------------------------------------------------------------
 // constants
 // ----------------------------------------------------------------------------
 
-#if defined(__WXMSW__)
-    #define CALBORDER         0
-#else
-    #define CALBORDER         4
-#endif
 
 // ----------------------------------------------------------------------------
 // global variables
@@ -96,42 +72,13 @@ public:
     {
         if ( !wxCalendarCtrl::Create(parent, wxID_ANY, wxDefaultDateTime,
                               wxPoint(0, 0), wxDefaultSize,
-                              wxCAL_SHOW_HOLIDAYS | wxBORDER_SUNKEN) )
+                              wxCAL_SEQUENTIAL_MONTH_SELECTION
+                              | wxCAL_SHOW_HOLIDAYS | wxBORDER_SUNKEN) )
             return false;
 
-        wxWindow *yearControl = wxCalendarCtrl::GetYearControl();
+        SetFormat(GetLocaleDateFormat());
 
-        wxClientDC dc(yearControl);
-        dc.SetFont(yearControl->GetFont());
-        wxCoord width, dummy;
-        dc.GetTextExtent(wxT("2000"), &width, &dummy);
-        width += ConvertDialogToPixels(wxSize(20, 0)).x;
-
-        wxSize calSize = wxCalendarCtrl::GetBestSize();
-        wxSize yearSize = yearControl->GetSize();
-        yearSize.x = width;
-
-        wxPoint yearPosition = yearControl->GetPosition();
-
-        SetFormat(wxT("%x"));
-
-        width = yearPosition.x + yearSize.x+2+CALBORDER/2;
-        if (width < calSize.x-4)
-            width = calSize.x-4;
-
-        int calPos = (width-calSize.x)/2;
-        if (calPos == -1)
-        {
-            calPos = 0;
-            width += 2;
-        }
-        wxCalendarCtrl::SetSize(calPos, 0, calSize.x, calSize.y);
-        yearControl->SetSize(width-yearSize.x-CALBORDER/2, yearPosition.y,
-                             yearSize.x, yearSize.y);
-        wxCalendarCtrl::GetMonthControl()->Move(0, 0);
-
-        m_useSize.x = width+CALBORDER/2;
-        m_useSize.y = calSize.y-2+CALBORDER;
+        m_useSize  = wxCalendarCtrl::GetBestSize();
 
         wxWindow* tx = m_combo->GetTextCtrl();
         if ( !tx )
@@ -163,19 +110,24 @@ public:
         else // invalid date
         {
             wxASSERT_MSG( HasDPFlag(wxDP_ALLOWNONE),
-                            _T("this control must have a valid date") );
+                            wxT("this control must have a valid date") );
 
             m_combo->SetText(wxEmptyString);
         }
     }
 
+    bool IsTextEmpty() const
+    {
+        return m_combo->GetTextCtrl()->IsEmpty();
+    }
+
     bool ParseDateTime(const wxString& s, wxDateTime* pDt)
     {
         wxASSERT(pDt);
 
         if ( !s.empty() )
         {
-            pDt->ParseFormat(s, m_format);
+            pDt->ParseFormat(s.c_str(), m_format);
             if ( !pDt->IsValid() )
                 return false;
         }
@@ -185,14 +137,10 @@ public:
 
     void SendDateEvent(const wxDateTime& dt)
     {
-        //
         // Sends both wxCalendarEvent and wxDateEvent
         wxWindow* datePicker = m_combo->GetParent();
 
-        wxCalendarEvent cev((wxCalendarCtrl*) this, wxEVT_CALENDAR_SEL_CHANGED);
-        cev.SetEventObject(datePicker);
-        cev.SetId(datePicker->GetId());
-        cev.SetDate(dt);
+        wxCalendarEvent cev(datePicker, dt, wxEVT_CALENDAR_SEL_CHANGED);
         datePicker->GetEventHandler()->ProcessEvent(cev);
 
         wxDateEvent event(datePicker, dt, wxEVT_DATE_CHANGED);
@@ -239,7 +187,7 @@ private:
 
         if ( !dt.IsValid() && HasDPFlag(wxDP_ALLOWNONE) )
             return;
-        
+
         // notify that we had to change the date after validation
         if ( (dt.IsValid() && (!dtOld.IsValid() || dt != dtOld)) ||
                 (!dt.IsValid() && dtOld.IsValid()) )
@@ -249,54 +197,81 @@ private:
         }
     }
 
-    bool HasDPFlag(int flag)
+    bool HasDPFlag(int flag) const
     {
         return m_combo->GetParent()->HasFlag(flag);
     }
 
-    bool SetFormat(const wxChar *fmt)
+    // it expands "%x" format and changes %y to %Y if wxDP_SHOWCENTURY flag
+    // is given. If the locale format can't be easily analyzed (e.g. when
+    // the month is given as a name, not number), "%x" is returned
+    wxString GetLocaleDateFormat() const
     {
-        m_format.clear();
+        wxString x_format(wxT("%x"));
+        wxString fmt;
+        int year_cnt = 0, month_cnt = 0, day_cnt = 0;
 
         wxDateTime dt;
-        dt.ParseFormat(wxT("2003-10-13"), wxT("%Y-%m-%d"));
-        wxString str(dt.Format(fmt));
+        dt.ParseFormat(wxT("2003-10-17"), wxT("%Y-%m-%d"));
+        wxString str(dt.Format(x_format));
 
         const wxChar *p = str.c_str();
         while ( *p )
         {
-            int n=wxAtoi(p);
-            if (n == dt.GetDay())
+            if (wxIsdigit(*p))
             {
-                m_format.Append(wxT("%d"));
-                p += 2;
-            }
-            else if (n == (int)dt.GetMonth()+1)
-            {
-                m_format.Append(wxT("%m"));
-                p += 2;
-            }
-            else if (n == dt.GetYear())
-            {
-                m_format.Append(wxT("%Y"));
-                p += 4;
-            }
-            else if (n == (dt.GetYear() % 100))
-            {
-                if ( HasDPFlag(wxDP_SHOWCENTURY) )
-                    m_format.Append(wxT("%Y"));
+                int n=wxAtoi(p);
+                if (n == dt.GetDay())
+                {
+                    fmt.Append(wxT("%d"));
+                    day_cnt++;
+                    p += 2;
+                }
+                else if (n == (int)dt.GetMonth()+1)
+                {
+                    fmt.Append(wxT("%m"));
+                    month_cnt++;
+                    p += 2;
+                }
+                else if (n == dt.GetYear())
+                {
+                    fmt.Append(wxT("%Y"));
+                    year_cnt++;
+                    p += 4;
+                }
+                else if (n == (dt.GetYear() % 100))
+                {
+                    if ( HasDPFlag(wxDP_SHOWCENTURY) )
+                        fmt.Append(wxT("%Y"));
+                    else
+                        fmt.Append(wxT("%y"));
+                    year_cnt++;
+                    p += 2;
+                }
                 else
-                    m_format.Append(wxT("%y"));
-                p += 2;
+                    // this shouldn't happen
+                    return x_format;
+            }
+            else {
+                fmt.Append(*p);
+                p++;
             }
-            else
-                m_format.Append(*p++);
         }
 
+        if (year_cnt == 1 && month_cnt == 1 && day_cnt == 1)
+            return fmt;
+
+        return x_format;
+    }
+
+    bool SetFormat(const wxString& fmt)
+    {
+        m_format = fmt;
+
         if ( m_combo )
         {
             wxArrayString allowedChars;
-            for ( wxChar c = _T('0'); c <= _T('9'); c++ )
+            for ( wxChar c = wxT('0'); c <= wxT('9'); c++ )
                 allowedChars.Add(wxString(c, 1));
 
             const wxChar *p2 = m_format.c_str();
@@ -356,9 +331,7 @@ private:
 BEGIN_EVENT_TABLE(wxCalendarComboPopup, wxCalendarCtrl)
     EVT_KEY_DOWN(wxCalendarComboPopup::OnCalKey)
     EVT_CALENDAR_SEL_CHANGED(wxID_ANY, wxCalendarComboPopup::OnSelChange)
-    EVT_CALENDAR_DAY(wxID_ANY, wxCalendarComboPopup::OnSelChange)
-    EVT_CALENDAR_MONTH(wxID_ANY, wxCalendarComboPopup::OnSelChange)
-    EVT_CALENDAR_YEAR(wxID_ANY, wxCalendarComboPopup::OnSelChange)
+    EVT_CALENDAR_PAGE_CHANGED(wxID_ANY, wxCalendarComboPopup::OnSelChange)
     EVT_CALENDAR(wxID_ANY, wxCalendarComboPopup::OnSelChange)
 END_EVENT_TABLE()
 
@@ -391,7 +364,7 @@ bool wxDatePickerCtrlGeneric::Create(wxWindow *parent,
                                      const wxString& name)
 {
     wxASSERT_MSG( !(style & wxDP_SPIN),
-                  _T("wxDP_SPIN style not supported, use wxDP_DEFAULT") );
+                  wxT("wxDP_SPIN style not supported, use wxDP_DEFAULT") );
 
     if ( !wxControl::Create(parent, id, pos, size,
                             style | wxCLIP_CHILDREN | wxWANTS_CHARS | wxBORDER_NONE,
@@ -408,7 +381,6 @@ bool wxDatePickerCtrlGeneric::Create(wxWindow *parent,
     m_combo->SetCtrlMainWnd(this);
 
     m_popup = new wxCalendarComboPopup();
-    m_cal = m_popup;
 
 #if defined(__WXMSW__)
     // without this keyboard navigation in month control doesn't work
@@ -428,7 +400,6 @@ void wxDatePickerCtrlGeneric::Init()
 {
     m_combo = NULL;
     m_popup = NULL;
-    m_cal = NULL;
 }
 
 wxDatePickerCtrlGeneric::~wxDatePickerCtrlGeneric()
@@ -442,7 +413,6 @@ bool wxDatePickerCtrlGeneric::Destroy()
 
     m_combo = NULL;
     m_popup = NULL;
-    m_cal = NULL;
 
     return wxControl::Destroy();
 }
@@ -470,6 +440,8 @@ wxDatePickerCtrlGeneric::SetDateRange(const wxDateTime& lowerdate,
 
 wxDateTime wxDatePickerCtrlGeneric::GetValue() const
 {
+    if ( HasFlag(wxDP_ALLOWNONE) && m_popup->IsTextEmpty() )
+        return wxInvalidDateTime;
     return m_popup->GetDate();
 }
 
@@ -482,11 +454,7 @@ void wxDatePickerCtrlGeneric::SetValue(const wxDateTime& date)
 
 bool wxDatePickerCtrlGeneric::GetRange(wxDateTime *dt1, wxDateTime *dt2) const
 {
-    if (dt1)
-        *dt1 = m_popup->GetLowerDateLimit();
-    if (dt2)
-        *dt2 = m_popup->GetUpperDateLimit();
-    return true;
+    return m_popup->GetDateRange(dt1, dt2);
 }
 
 
@@ -496,6 +464,11 @@ wxDatePickerCtrlGeneric::SetRange(const wxDateTime &dt1, const wxDateTime &dt2)
     m_popup->SetDateRange(dt1, dt2);
 }
 
+wxCalendarCtrl *wxDatePickerCtrlGeneric::GetCalendar() const
+{
+    return m_popup;
+}
+
 // ----------------------------------------------------------------------------
 // event handlers
 // ----------------------------------------------------------------------------
@@ -514,12 +487,12 @@ void wxDatePickerCtrlGeneric::OnText(wxCommandEvent &ev)
 {
     ev.SetEventObject(this);
     ev.SetId(GetId());
-    GetParent()->ProcessEvent(ev);
+    GetParent()->GetEventHandler()->ProcessEvent(ev);
 
     // We'll create an additional event if the date is valid.
     // If the date isn't valid, the user's probably in the middle of typing
     wxDateTime dt;
-    if ( !m_popup->ParseDateTime(m_combo->GetValue(), &dt) )
+    if ( !m_popup || !m_popup->ParseDateTime(m_combo->GetValue(), &dt) )
         return;
 
     m_popup->SendDateEvent(dt);
@@ -532,7 +505,5 @@ void wxDatePickerCtrlGeneric::OnFocus(wxFocusEvent& WXUNUSED(event))
 }
 
 
-#endif // wxUSE_DATEPICKCTRL_GENERIC
-
 #endif // wxUSE_DATEPICKCTRL
 
