@@ -1,5 +1,5 @@
 ///////////////////////////////////////////////////////////////////////////////
-// Name:        generic/wizard.cpp
+// Name:        src/generic/wizard.cpp
 // Purpose:     generic implementation of wxWizard class
 // Author:      Vadim Zeitlin
 // Modified by: Robert Cavanaugh
@@ -8,7 +8,7 @@
 //              3) Fixed ShowPage() bug on displaying bitmaps
 //              Robert Vazan (sizers)
 // Created:     15.08.99
-// RCS-ID:      $Id: wizard.cpp,v 1.58 2004/10/12 18:15:59 JS Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1999 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -21,10 +21,6 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "wizardg.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -39,12 +35,15 @@
     #include "wx/intl.h"
     #include "wx/statbmp.h"
     #include "wx/button.h"
+    #include "wx/settings.h"
+    #include "wx/sizer.h"
 #endif //WX_PRECOMP
 
 #include "wx/statline.h"
-#include "wx/sizer.h"
 
+#include "wx/scrolwin.h"
 #include "wx/wizard.h"
+#include "wx/dcmemory.h"
 
 // ----------------------------------------------------------------------------
 // wxWizardSizer
@@ -55,17 +54,26 @@ class wxWizardSizer : public wxSizer
 public:
     wxWizardSizer(wxWizard *owner);
 
-    void RecalcSizes();
-    wxSize CalcMin();
+    virtual wxSizerItem *Insert(size_t index, wxSizerItem *item);
 
+    virtual void RecalcSizes();
+    virtual wxSize CalcMin();
+
+    // get the max size of all wizard pages
     wxSize GetMaxChildSize();
-    int Border() const;
+
+    // return the border which can be either set using wxWizard::SetBorder() or
+    // have default value
+    int GetBorder() const;
+
+    // hide the pages which we temporarily "show" when they're added to this
+    // sizer (see Insert())
+    void HidePages();
 
 private:
     wxSize SiblingSize(wxSizerItem *child);
 
     wxWizard *m_owner;
-    bool m_childSizeValid;
     wxSize m_childSize;
 };
 
@@ -73,11 +81,12 @@ private:
 // event tables and such
 // ----------------------------------------------------------------------------
 
-DEFINE_EVENT_TYPE(wxEVT_WIZARD_PAGE_CHANGED)
-DEFINE_EVENT_TYPE(wxEVT_WIZARD_PAGE_CHANGING)
-DEFINE_EVENT_TYPE(wxEVT_WIZARD_CANCEL)
-DEFINE_EVENT_TYPE(wxEVT_WIZARD_FINISHED)
-DEFINE_EVENT_TYPE(wxEVT_WIZARD_HELP)
+wxDEFINE_EVENT( wxEVT_WIZARD_PAGE_CHANGED, wxWizardEvent );
+wxDEFINE_EVENT( wxEVT_WIZARD_PAGE_CHANGING, wxWizardEvent );
+wxDEFINE_EVENT( wxEVT_WIZARD_CANCEL, wxWizardEvent );
+wxDEFINE_EVENT( wxEVT_WIZARD_FINISHED, wxWizardEvent );
+wxDEFINE_EVENT( wxEVT_WIZARD_HELP, wxWizardEvent );
+wxDEFINE_EVENT( wxEVT_WIZARD_PAGE_SHOWN, wxWizardEvent );
 
 BEGIN_EVENT_TABLE(wxWizard, wxDialog)
     EVT_BUTTON(wxID_CANCEL, wxWizard::OnCancel)
@@ -119,31 +128,17 @@ void wxWizardPage::Init()
 }
 
 wxWizardPage::wxWizardPage(wxWizard *parent,
-                           const wxBitmap& bitmap,
-                           const wxChar *resource)
+                           const wxBitmap& bitmap)
 {
-    Create(parent, bitmap, resource);
+    Create(parent, bitmap);
 }
 
 bool wxWizardPage::Create(wxWizard *parent,
-                          const wxBitmap& bitmap,
-                          const wxChar *resource)
+                          const wxBitmap& bitmap)
 {
     if ( !wxPanel::Create(parent, wxID_ANY) )
         return false;
 
-    if ( resource != NULL )
-    {
-#if wxUSE_WX_RESOURCES
-#if 0
-       if ( !LoadFromResource(this, resource) )
-        {
-            wxFAIL_MSG(wxT("wxWizardPage LoadFromResource failed!!!!"));
-        }
-#endif
-#endif // wxUSE_RESOURCES
-    }
-
     m_bitmap = bitmap;
 
     // initially the page is hidden, it's shown only when it becomes current
@@ -171,9 +166,36 @@ wxWizardPage *wxWizardPageSimple::GetNext() const
 // ----------------------------------------------------------------------------
 
 wxWizardSizer::wxWizardSizer(wxWizard *owner)
-    : m_owner(owner)
+             : m_owner(owner),
+               m_childSize(wxDefaultSize)
 {
-    m_childSizeValid = false;
+}
+
+wxSizerItem *wxWizardSizer::Insert(size_t index, wxSizerItem *item)
+{
+    m_owner->m_usingSizer = true;
+
+    if ( item->IsWindow() )
+    {
+        // we must pretend that the window is shown as otherwise it wouldn't be
+        // taken into account for the layout -- but avoid really showing it, so
+        // just set the internal flag instead of calling wxWindow::Show()
+        item->GetWindow()->wxWindowBase::Show();
+    }
+
+    return wxSizer::Insert(index, item);
+}
+
+void wxWizardSizer::HidePages()
+{
+    for ( wxSizerItemList::compatibility_iterator node = GetChildren().GetFirst();
+          node;
+          node = node->GetNext() )
+    {
+        wxSizerItem * const item = node->GetData();
+        if ( item->IsWindow() )
+            item->GetWindow()->wxWindowBase::Show(false);
+    }
 }
 
 void wxWizardSizer::RecalcSizes()
@@ -182,7 +204,7 @@ void wxWizardSizer::RecalcSizes()
     // it should be called whenever it changes (wxWizard::ShowPage)
     if ( m_owner->m_page )
     {
-        m_owner->m_page->SetSize(m_position.x,m_position.y, m_size.x,m_size.y);
+        m_owner->m_page->SetSize(wxRect(m_position, m_size));
     }
 }
 
@@ -193,49 +215,28 @@ wxSize wxWizardSizer::CalcMin()
 
 wxSize wxWizardSizer::GetMaxChildSize()
 {
-#if !defined(__WXDEBUG__)
-    if ( m_childSizeValid )
-        return m_childSize;
-#endif
-
     wxSize maxOfMin;
-    wxSizerItemList::compatibility_iterator childNode;
 
-    for(childNode = m_children.GetFirst(); childNode;
-        childNode = childNode->GetNext())
+    for ( wxSizerItemList::compatibility_iterator childNode = m_children.GetFirst();
+          childNode;
+          childNode = childNode->GetNext() )
     {
         wxSizerItem *child = childNode->GetData();
         maxOfMin.IncTo(child->CalcMin());
         maxOfMin.IncTo(SiblingSize(child));
     }
 
-#ifdef __WXDEBUG__
-    if ( m_childSizeValid && m_childSize != maxOfMin )
-    {
-        wxFAIL_MSG( _T("Size changed in wxWizard::GetPageAreaSizer()")
-                    _T("after RunWizard().\n")
-                    _T("Did you forget to call GetSizer()->Fit(this) ")
-                    _T("for some page?")) ;
-
-        return m_childSize;
-    }
-#endif // __WXDEBUG__
-
     if ( m_owner->m_started )
     {
-        m_childSizeValid = true;
         m_childSize = maxOfMin;
     }
 
     return maxOfMin;
 }
 
-int wxWizardSizer::Border() const
+int wxWizardSizer::GetBorder() const
 {
-    if ( m_owner->m_calledSetBorder )
-        return m_owner->m_border;
-
-    return m_children.IsEmpty() ? 5 : 0;
+    return m_owner->m_border;
 }
 
 wxSize wxWizardSizer::SiblingSize(wxSizerItem *child)
@@ -269,14 +270,18 @@ wxSize wxWizardSizer::SiblingSize(wxSizerItem *child)
 void wxWizard::Init()
 {
     m_posWizard = wxDefaultPosition;
-    m_page = (wxWizardPage *)NULL;
+    m_page = NULL;
     m_btnPrev = m_btnNext = NULL;
     m_statbmp = NULL;
     m_sizerBmpAndPage = NULL;
     m_sizerPage = NULL;
-    m_calledSetBorder = false;
-    m_border = 0;
+    m_border = 5;
     m_started = false;
+    m_wasModal = false;
+    m_usingSizer = false;
+    m_bitmapBackgroundColour = *wxWHITE;
+    m_bitmapPlacement = 0;
+    m_bitmapMinimumWidth = 115;
 }
 
 bool wxWizard::Create(wxWindow *parent,
@@ -296,6 +301,15 @@ bool wxWizard::Create(wxWindow *parent,
     return result;
 }
 
+wxWizard::~wxWizard()
+{
+    // normally we don't have to delete this sizer as it's deleted by the
+    // associated window but if we never used it or didn't set it as the window
+    // sizer yet, do delete it manually
+    if ( !m_usingSizer || !m_started )
+        delete m_sizerPage;
+}
+
 void wxWizard::AddBitmapRow(wxBoxSizer *mainColumn)
 {
     m_sizerBmpAndPage = new wxBoxSizer(wxHORIZONTAL);
@@ -312,7 +326,11 @@ void wxWizard::AddBitmapRow(wxBoxSizer *mainColumn)
 #if wxUSE_STATBMP
     if ( m_bitmap.Ok() )
     {
-        m_statbmp = new wxStaticBitmap(this, wxID_ANY, m_bitmap);
+        wxSize bitmapSize(wxDefaultSize);
+        if (GetBitmapPlacement())
+            bitmapSize.x = GetMinimumBitmapWidth();
+
+        m_statbmp = new wxStaticBitmap(this, wxID_ANY, m_bitmap, wxDefaultPosition, bitmapSize);
         m_sizerBmpAndPage->Add(
             m_statbmp,
             0, // No horizontal stretching
@@ -327,7 +345,7 @@ void wxWizard::AddBitmapRow(wxBoxSizer *mainColumn)
     }
 #endif
 
-    // Added to m_sizerBmpAndPage in FinishLayout
+    // Added to m_sizerBmpAndPage later
     m_sizerPage = new wxWizardSizer(this);
 }
 
@@ -352,8 +370,8 @@ void wxWizard::AddStaticLine(wxBoxSizer *mainColumn)
 void wxWizard::AddBackNextPair(wxBoxSizer *buttonRow)
 {
     wxASSERT_MSG( m_btnNext && m_btnPrev,
-                  _T("You must create the buttons before calling ")
-                  _T("wxWizard::AddBackNextPair") );
+                  wxT("You must create the buttons before calling ")
+                  wxT("wxWizard::AddBackNextPair") );
 
     // margin between Back and Next buttons
 #ifdef __WXMAC__
@@ -390,6 +408,9 @@ void wxWizard::AddButtonRow(wxBoxSizer *mainColumn)
     // key to TAB to the next entry field and page. This would not be possible, if the 'back' button
     // was created before the 'next' button.
 
+    bool isPda = (wxSystemSettings::GetScreenType() <= wxSYS_SCREEN_PDA);
+    int buttonStyle = isPda ? wxBU_EXACTFIT : 0;
+
     wxBoxSizer *buttonRow = new wxBoxSizer(wxHORIZONTAL);
 #ifdef __WXMAC__
     if (GetExtraStyle() & wxWIZARD_EX_HELPBUTTON)
@@ -411,16 +432,16 @@ void wxWizard::AddButtonRow(wxBoxSizer *mainColumn)
     wxButton *btnHelp=0;
 #ifdef __WXMAC__
     if (GetExtraStyle() & wxWIZARD_EX_HELPBUTTON)
-        btnHelp=new wxButton(this, wxID_HELP, _("&Help"));
+        btnHelp=new wxButton(this, wxID_HELP, wxEmptyString, wxDefaultPosition, wxDefaultSize, buttonStyle);
 #endif
 
     m_btnNext = new wxButton(this, wxID_FORWARD, _("&Next >"));
-    wxButton *btnCancel=new wxButton(this, wxID_CANCEL, _("&Cancel"));
+    wxButton *btnCancel=new wxButton(this, wxID_CANCEL, _("&Cancel"), wxDefaultPosition, wxDefaultSize, buttonStyle);
 #ifndef __WXMAC__
     if (GetExtraStyle() & wxWIZARD_EX_HELPBUTTON)
-        btnHelp=new wxButton(this, wxID_HELP, _("&Help"));
+        btnHelp=new wxButton(this, wxID_HELP, _("&Help"), wxDefaultPosition, wxDefaultSize, buttonStyle);
 #endif
-    m_btnPrev = new wxButton(this, wxID_BACKWARD, _("< &Back"));
+    m_btnPrev = new wxButton(this, wxID_BACKWARD, _("< &Back"), wxDefaultPosition, wxDefaultSize, buttonStyle);
 
     if (btnHelp)
     {
@@ -452,6 +473,11 @@ void wxWizard::DoCreateControls()
     if ( WasCreated() )
         return;
 
+    bool isPda = (wxSystemSettings::GetScreenType() <= wxSYS_SCREEN_PDA);
+
+    // Horizontal stretching, and if not PDA, border all around
+    int mainColumnSizerFlags = isPda ? wxEXPAND : wxALL|wxEXPAND ;
+
     // wxWindow::SetSizer will be called at end
     wxBoxSizer *windowSizer = new wxBoxSizer(wxVERTICAL);
 
@@ -459,42 +485,29 @@ void wxWizard::DoCreateControls()
     windowSizer->Add(
         mainColumn,
         1, // Vertical stretching
-        wxALL | wxEXPAND, // Border all around, horizontal stretching
+        mainColumnSizerFlags,
         5 // Border width
     );
 
     AddBitmapRow(mainColumn);
-    AddStaticLine(mainColumn);
+
+    if (!isPda)
+        AddStaticLine(mainColumn);
+
     AddButtonRow(mainColumn);
 
-    // wxWindow::SetSizer should be followed by wxWindow::Fit, but
-    // this is done in FinishLayout anyway so why duplicate it
     SetSizer(windowSizer);
 }
 
 void wxWizard::SetPageSize(const wxSize& size)
 {
-    wxCHECK_RET(!m_started,wxT("wxWizard::SetPageSize after RunWizard"));
+    wxCHECK_RET(!m_started, wxT("wxWizard::SetPageSize after RunWizard"));
     m_sizePage = size;
 }
 
-void wxWizard::FinishLayout()
-{
-    m_sizerBmpAndPage->Add(
-        m_sizerPage,
-        1, // Horizontal stretching
-        wxEXPAND | wxALL, // Vertically stretchable
-        m_sizerPage->Border()
-    );
-
-    GetSizer()->SetSizeHints(this);
-    if ( m_posWizard == wxDefaultPosition )
-        CentreOnScreen();
-}
-
 void wxWizard::FitToPage(const wxWizardPage *page)
 {
-    wxCHECK_RET(!m_started,wxT("wxWizard::FitToPage after RunWizard"));
+    wxCHECK_RET(!m_started, wxT("wxWizard::FitToPage after RunWizard"));
 
     while ( page )
     {
@@ -510,26 +523,31 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
 {
     wxASSERT_MSG( page != m_page, wxT("this is useless") );
 
-    // we'll use this to decide whether we have to change the label of this
-    // button or not (initially the label is "Next")
-    bool btnLabelWasNext = true;
+    wxSizerFlags flags(1);
+    flags.Border(wxALL, m_border).Expand();
 
-    // Modified 10-20-2001 Robert Cavanaugh.
-    // Fixed bug for displaying a new bitmap
-    // in each *consecutive* page
+    if ( !m_started )
+    {
+        if ( m_usingSizer )
+        {
+            m_sizerBmpAndPage->Add(m_sizerPage, flags);
 
-    // flag to indicate if this page uses a new bitmap
-    bool bmpIsDefault = true;
+            // now that our layout is computed correctly, hide the pages
+            // artificially shown in wxWizardSizer::Insert() back again
+            m_sizerPage->HidePages();
+        }
+    }
 
-    // use these labels to determine if we need to change the bitmap
-    // for this page
-    wxBitmap bmpPrev, bmpCur;
+
+    // remember the old bitmap (if any) to compare with the new one later
+    wxBitmap bmpPrev;
 
     // check for previous page
     if ( m_page )
     {
         // send the event to the old page
-        wxWizardEvent event(wxEVT_WIZARD_PAGE_CHANGING, GetId(), goingForward, m_page);
+        wxWizardEvent event(wxEVT_WIZARD_PAGE_CHANGING, GetId(),
+                            goingForward, m_page);
         if ( m_page->GetEventHandler()->ProcessEvent(event) &&
              !event.IsAllowed() )
         {
@@ -539,13 +557,10 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
 
         m_page->Hide();
 
-        btnLabelWasNext = HasNextPage(m_page);
+        bmpPrev = m_page->GetBitmap();
 
-        // Get the bitmap of the previous page (if it exists)
-        if ( m_page->GetBitmap().Ok() )
-        {
-            bmpPrev = m_page->GetBitmap();
-        }
+        if ( !m_usingSizer )
+            m_sizerBmpAndPage->Detach(m_page);
     }
 
     // set the new page
@@ -555,7 +570,15 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
     if ( !m_page )
     {
         // terminate successfully
-        EndModal(wxID_OK);
+        if ( IsModal() )
+        {
+            EndModal(wxID_OK);
+        }
+        else
+        {
+            SetReturnCode(wxID_OK);
+            Hide();
+        }
 
         // and notify the user code (this is especially useful for modeless
         // wizards)
@@ -568,47 +591,48 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
     // position and show the new page
     (void)m_page->TransferDataToWindow();
 
-    // wxWizardSizer::RecalcSizes wants to be called when m_page changes
-    m_sizerPage->RecalcSizes();
-
-    // check if bitmap needs to be updated
-    // update default flag as well
-    if ( m_page->GetBitmap().Ok() )
+    if ( m_usingSizer )
     {
-        bmpCur = m_page->GetBitmap();
-        bmpIsDefault = false;
+        // wxWizardSizer::RecalcSizes wants to be called when m_page changes
+        m_sizerPage->RecalcSizes();
+    }
+    else // pages are not managed by the sizer
+    {
+        m_sizerBmpAndPage->Add(m_page, flags);
+        m_sizerBmpAndPage->SetItemMinSize(m_page, GetPageSize());
     }
 
 #if wxUSE_STATBMP
-    // change the bitmap if:
-    // 1) a default bitmap was selected in constructor
-    // 2) this page was constructed with a bitmap
-    // 3) this bitmap is not the previous bitmap
-    if ( m_statbmp && (bmpCur != bmpPrev) )
+    // update the bitmap if:it changed
+    wxBitmap bmp;
+    if ( m_statbmp )
     {
-        wxBitmap bmp;
-        if ( bmpIsDefault )
+        bmp = m_page->GetBitmap();
+        if ( !bmp.Ok() )
             bmp = m_bitmap;
-        else
-            bmp = m_page->GetBitmap();
-        m_statbmp->SetBitmap(bmp);
+
+        if ( !bmpPrev.Ok() )
+            bmpPrev = m_bitmap;
+
+        if (!GetBitmapPlacement())
+        {
+            if ( !bmp.IsSameAs(bmpPrev) )
+                m_statbmp->SetBitmap(bmp);
+        }
     }
-#endif
+#endif // wxUSE_STATBMP
+
 
     // and update the buttons state
     m_btnPrev->Enable(HasPrevPage(m_page));
 
-    bool hasNext = HasNextPage(m_page);
-    if ( btnLabelWasNext != hasNext )
-    {
-        // need to update
-        if (btnLabelWasNext)
-            m_btnNext->SetLabel(_("&Finish"));
-        else
-            m_btnNext->SetLabel(_("&Next >"));
-    }
+    const bool hasNext = HasNextPage(m_page);
+    const wxString label = hasNext ? _("&Next >") : _("&Finish");
+    if ( label != m_btnNext->GetLabel() )
+        m_btnNext->SetLabel(label);
+
     m_btnNext->SetDefault();
-    // nothing to do: the label was already correct
+
 
     // send the change event to the new page now
     wxWizardEvent event(wxEVT_WIZARD_PAGE_CHANGED, GetId(), goingForward, m_page);
@@ -618,23 +642,60 @@ bool wxWizard::ShowPage(wxWizardPage *page, bool goingForward)
     m_page->Show();
     m_page->SetFocus();
 
+    if ( !m_usingSizer )
+        m_sizerBmpAndPage->Layout();
+
+    if ( !m_started )
+    {
+        m_started = true;
+
+        DoWizardLayout();
+    }
+
+    if (GetBitmapPlacement() && m_statbmp)
+    {
+        ResizeBitmap(bmp);
+
+        if ( !bmp.IsSameAs(bmpPrev) )
+            m_statbmp->SetBitmap(bmp);
+
+        if (m_usingSizer)
+            m_sizerPage->RecalcSizes();
+    }
+
+    wxWizardEvent pageShownEvent(wxEVT_WIZARD_PAGE_SHOWN, GetId(),
+        goingForward, m_page);
+    m_page->GetEventHandler()->ProcessEvent(pageShownEvent);
+
     return true;
 }
 
+/// Do fit, and adjust to screen size if necessary
+void wxWizard::DoWizardLayout()
+{
+    if ( wxSystemSettings::GetScreenType() > wxSYS_SCREEN_PDA )
+    {
+        if (CanDoLayoutAdaptation())
+            DoLayoutAdaptation();
+        else
+            GetSizer()->SetSizeHints(this);
+
+        if ( m_posWizard == wxDefaultPosition )
+            CentreOnScreen();
+    }
+
+    SetLayoutAdaptationDone(true);
+}
+
 bool wxWizard::RunWizard(wxWizardPage *firstPage)
 {
     wxCHECK_MSG( firstPage, false, wxT("can't run empty wizard") );
 
-    // Set before FinishLayout to enable wxWizardSizer::GetMaxChildSize
-    m_started = true;
-
-    // This cannot be done sooner, because user can change layout options
-    // up to this moment
-    FinishLayout();
-
     // can't return false here because there is no old page
     (void)ShowPage(firstPage, true /* forward */);
 
+    m_wasModal = true;
+
     return ShowModal() == wxID_OK;
 }
 
@@ -645,8 +706,39 @@ wxWizardPage *wxWizard::GetCurrentPage() const
 
 wxSize wxWizard::GetPageSize() const
 {
-    wxSize pageSize(GetManualPageSize());
-    pageSize.IncTo(m_sizerPage->GetMaxChildSize());
+    // default width and height of the page
+    int DEFAULT_PAGE_WIDTH,
+        DEFAULT_PAGE_HEIGHT;
+    if ( wxSystemSettings::GetScreenType() <= wxSYS_SCREEN_PDA )
+    {
+        // Make the default page size small enough to fit on screen
+        DEFAULT_PAGE_WIDTH = wxSystemSettings::GetMetric(wxSYS_SCREEN_X) / 2;
+        DEFAULT_PAGE_HEIGHT = wxSystemSettings::GetMetric(wxSYS_SCREEN_Y) / 2;
+    }
+    else // !PDA
+    {
+        DEFAULT_PAGE_WIDTH =
+        DEFAULT_PAGE_HEIGHT = 270;
+    }
+
+    // start with default minimal size
+    wxSize pageSize(DEFAULT_PAGE_WIDTH, DEFAULT_PAGE_HEIGHT);
+
+    // make the page at least as big as specified by user
+    pageSize.IncTo(m_sizePage);
+
+    if ( m_statbmp )
+    {
+        // make the page at least as tall as the bitmap
+        pageSize.IncTo(wxSize(0, m_bitmap.GetHeight()));
+    }
+
+    if ( m_usingSizer )
+    {
+        // make it big enough to contain all pages added to the sizer
+        pageSize.IncTo(m_sizerPage->GetMaxChildSize());
+    }
+
     return pageSize;
 }
 
@@ -657,34 +749,11 @@ wxSizer *wxWizard::GetPageAreaSizer() const
 
 void wxWizard::SetBorder(int border)
 {
-    wxCHECK_RET(!m_started,wxT("wxWizard::SetBorder after RunWizard"));
+    wxCHECK_RET(!m_started, wxT("wxWizard::SetBorder after RunWizard"));
 
-    m_calledSetBorder = true;
     m_border = border;
 }
 
-wxSize wxWizard::GetManualPageSize() const
-{
-    // default width and height of the page
-    static const int DEFAULT_PAGE_WIDTH = 270;
-    //static const int DEFAULT_PAGE_HEIGHT = 290;
-    // For compatibility with 2.4: there's too much
-    // space under the bitmap, probably due to differences in
-    // the sizer implementation. This makes it reasonable again.
-    static const int DEFAULT_PAGE_HEIGHT = 270;
-
-    wxSize totalPageSize(DEFAULT_PAGE_WIDTH,DEFAULT_PAGE_HEIGHT);
-
-    totalPageSize.IncTo(m_sizePage);
-
-    if ( m_statbmp )
-    {
-        totalPageSize.IncTo(wxSize(0, m_bitmap.GetHeight()));
-    }
-
-    return totalPageSize;
-}
-
 void wxWizard::OnCancel(wxCommandEvent& WXUNUSED(eventUnused))
 {
     // this function probably can never be called when we don't have an active
@@ -695,7 +764,15 @@ void wxWizard::OnCancel(wxCommandEvent& WXUNUSED(eventUnused))
     if ( !win->GetEventHandler()->ProcessEvent(event) || event.IsAllowed() )
     {
         // no objections - close the dialog
-        EndModal(wxID_CANCEL);
+        if(IsModal())
+        {
+            EndModal(wxID_CANCEL);
+        }
+        else
+        {
+            SetReturnCode(wxID_CANCEL);
+            Hide();
+        }
     }
     //else: request to Cancel ignored
 }
@@ -706,10 +783,12 @@ void wxWizard::OnBackOrNext(wxCommandEvent& event)
                   (event.GetEventObject() == m_btnPrev),
                   wxT("unknown button") );
 
+    wxCHECK_RET( m_page, wxT("should have a valid current page") );
+
     // ask the current page first: notice that we do it before calling
     // GetNext/Prev() because the data transfered from the controls of the page
     // may change the value returned by these methods
-    if ( m_page && (!m_page->Validate() || !m_page->TransferDataFromWindow()) )
+    if ( !m_page->Validate() || !m_page->TransferDataFromWindow() )
     {
         // the page data is incorrect, don't do anything
         return;
@@ -729,7 +808,7 @@ void wxWizard::OnBackOrNext(wxCommandEvent& event)
         wxASSERT_MSG( page, wxT("\"<Back\" button should have been disabled") );
     }
 
-    // just pass to the new page (or may be not - but we don't care here)
+    // just pass to the new page (or maybe not - but we don't care here)
     (void)ShowPage(page, forward);
 }
 
@@ -755,36 +834,35 @@ void wxWizard::OnWizEvent(wxWizardEvent& event)
     {
         // the event will be propagated anyhow
         event.Skip();
-        return;
     }
-
-    wxWindow *parent = GetParent();
-
-    if ( !parent || !parent->GetEventHandler()->ProcessEvent(event) )
+    else
     {
-        event.Skip();
+        wxWindow *parent = GetParent();
+
+        if ( !parent || !parent->GetEventHandler()->ProcessEvent(event) )
+        {
+            event.Skip();
+        }
+    }
+
+    if ( ( !m_wasModal ) &&
+         event.IsAllowed() &&
+         ( event.GetEventType() == wxEVT_WIZARD_FINISHED ||
+           event.GetEventType() == wxEVT_WIZARD_CANCEL
+         )
+       )
+    {
+        Destroy();
     }
 }
 
-// ----------------------------------------------------------------------------
-// our public interface
-// ----------------------------------------------------------------------------
-
-#if WXWIN_COMPATIBILITY_2_2
-
-/* static */
-wxWizard *wxWizardBase::Create(wxWindow *parent,
-                               int id,
-                               const wxString& title,
-                               const wxBitmap& bitmap,
-                               const wxPoint& pos,
-                               const wxSize& WXUNUSED(size))
+void wxWizard::SetBitmap(const wxBitmap& bitmap)
 {
-    return new wxWizard(parent, id, title, bitmap, pos);
+    m_bitmap = bitmap;
+    if (m_statbmp)
+        m_statbmp->SetBitmap(m_bitmap);
 }
 
-#endif // WXWIN_COMPATIBILITY_2_2
-
 // ----------------------------------------------------------------------------
 // wxWizardEvent
 // ----------------------------------------------------------------------------
@@ -798,4 +876,131 @@ wxWizardEvent::wxWizardEvent(wxEventType type, int id, bool direction, wxWizardP
     m_page = page;
 }
 
+/// Do the adaptation
+bool wxWizard::DoLayoutAdaptation()
+{
+    wxWindowList windows;
+    wxWindowList pages;
+
+    // Make all the pages (that use sizers) scrollable
+    for ( wxSizerItemList::compatibility_iterator node = m_sizerPage->GetChildren().GetFirst(); node; node = node->GetNext() )
+    {
+        wxSizerItem * const item = node->GetData();
+        if ( item->IsWindow() )
+        {
+            wxWizardPage* page = wxDynamicCast(item->GetWindow(), wxWizardPage);
+            if (page)
+            {
+                while (page)
+                {
+                    if (!pages.Find(page) && page->GetSizer())
+                    {
+                        // Create a scrolled window and reparent
+                        wxScrolledWindow* scrolledWindow = new wxScrolledWindow(page, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL|wxVSCROLL|wxHSCROLL|wxBORDER_NONE);
+                        wxSizer* oldSizer = page->GetSizer();
+
+                        wxSizer* newSizer = new wxBoxSizer(wxVERTICAL);
+                        newSizer->Add(scrolledWindow,1, wxEXPAND, 0);
+
+                        page->SetSizer(newSizer, false /* don't delete the old sizer */);
+
+                        scrolledWindow->SetSizer(oldSizer);
+
+                        wxStandardDialogLayoutAdapter::DoReparentControls(page, scrolledWindow);
+
+                        pages.Append(page);
+                        windows.Append(scrolledWindow);
+                    }
+                    page = page->GetNext();
+                }
+            }
+        }
+    }
+
+    wxStandardDialogLayoutAdapter::DoFitWithScrolling(this, windows);
+
+    // Size event doesn't get sent soon enough on wxGTK
+    DoLayout();
+
+    SetLayoutAdaptationDone(true);
+
+    return true;
+}
+
+bool wxWizard::ResizeBitmap(wxBitmap& bmp)
+{
+    if (!GetBitmapPlacement())
+        return false;
+
+    if (bmp.Ok())
+    {
+        wxSize pageSize = m_sizerPage->GetSize();
+        if (pageSize == wxSize(0,0))
+            pageSize = GetPageSize();
+        int bitmapWidth = wxMax(bmp.GetWidth(), GetMinimumBitmapWidth());
+        int bitmapHeight = pageSize.y;
+
+        if (!m_statbmp->GetBitmap().Ok() || m_statbmp->GetBitmap().GetHeight() != bitmapHeight)
+        {
+            wxBitmap bitmap(bitmapWidth, bitmapHeight);
+            {
+                wxMemoryDC dc;
+                dc.SelectObject(bitmap);
+                dc.SetBackground(wxBrush(m_bitmapBackgroundColour));
+                dc.Clear();
+
+                if (GetBitmapPlacement() & wxWIZARD_TILE)
+                {
+                    TileBitmap(wxRect(0, 0, bitmapWidth, bitmapHeight), dc, bmp);
+                }
+                else
+                {
+                    int x, y;
+
+                    if (GetBitmapPlacement() & wxWIZARD_HALIGN_LEFT)
+                        x = 0;
+                    else if (GetBitmapPlacement() & wxWIZARD_HALIGN_RIGHT)
+                        x = bitmapWidth - bmp.GetWidth();
+                    else
+                        x = (bitmapWidth - bmp.GetWidth())/2;
+
+                    if (GetBitmapPlacement() & wxWIZARD_VALIGN_TOP)
+                        y = 0;
+                    else if (GetBitmapPlacement() & wxWIZARD_VALIGN_BOTTOM)
+                        y = bitmapHeight - bmp.GetHeight();
+                    else
+                        y = (bitmapHeight - bmp.GetHeight())/2;
+
+                    dc.DrawBitmap(bmp, x, y, true);
+                    dc.SelectObject(wxNullBitmap);
+                }
+            }
+
+            bmp = bitmap;
+        }
+    }
+
+    return true;
+}
+
+bool wxWizard::TileBitmap(const wxRect& rect, wxDC& dc, const wxBitmap& bitmap)
+{
+    int w = bitmap.GetWidth();
+    int h = bitmap.GetHeight();
+
+    wxMemoryDC dcMem;
+
+    dcMem.SelectObjectAsSource(bitmap);
+
+    int i, j;
+    for (i = rect.x; i < rect.x + rect.width; i += w)
+    {
+        for (j = rect.y; j < rect.y + rect.height; j+= h)
+            dc.Blit(i, j, bitmap.GetWidth(), bitmap.GetHeight(), & dcMem, 0, 0);
+    }
+    dcMem.SelectObject(wxNullBitmap);
+
+    return true;
+}
+
 #endif // wxUSE_WIZARDDLG
