@@ -2,7 +2,7 @@
 // Name:        src/html/m_tables.cpp
 // Purpose:     wxHtml module for tables
 // Author:      Vaclav Slavik
-// RCS-ID:      $Id: m_tables.cpp,v 1.40 2006/09/05 20:47:15 VZ Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1999 Vaclav Slavik
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -15,7 +15,8 @@
 
 #if wxUSE_HTML && wxUSE_STREAMS
 
-#ifndef WXPRECOMP
+#ifndef WX_PRECOMP
+    #include "wx/wxcrtvararg.h"
 #endif
 
 #include "wx/html/forcelnk.h"
@@ -70,8 +71,6 @@ class wxHtmlTableCell : public wxHtmlContainerCell
 protected:
     /* These are real attributes: */
 
-    // should we draw borders or not?
-    bool m_HasBorders;
     // number of columns; rows
     int m_NumCols, m_NumRows;
     // array of column information
@@ -118,7 +117,7 @@ private:
     // only once, before first Layout().
     void ComputeMinMaxWidths();
 
-    DECLARE_NO_COPY_CLASS(wxHtmlTableCell)
+    wxDECLARE_NO_COPY_CLASS(wxHtmlTableCell);
 };
 
 
@@ -127,8 +126,6 @@ wxHtmlTableCell::wxHtmlTableCell(wxHtmlContainerCell *parent, const wxHtmlTag& t
  : wxHtmlContainerCell(parent)
 {
     m_PixelScale = pixel_scale;
-    m_HasBorders =
-            (tag.HasParam(wxT("BORDER")) && tag.GetParam(wxT("BORDER")) != wxT("0"));
     m_ColsInfo = NULL;
     m_NumCols = m_NumRows = 0;
     m_CellInfo = NULL;
@@ -136,7 +133,11 @@ wxHtmlTableCell::wxHtmlTableCell(wxHtmlContainerCell *parent, const wxHtmlTag& t
 
     /* scan params: */
     if (tag.HasParam(wxT("BGCOLOR")))
+    {
         tag.GetParamAsColour(wxT("BGCOLOR"), &m_tBkg);
+        if (m_tBkg.Ok())
+            SetBackgroundColour(m_tBkg);
+    }
     if (tag.HasParam(wxT("VALIGN")))
         m_tValign = tag.GetParam(wxT("VALIGN"));
     else
@@ -148,8 +149,20 @@ wxHtmlTableCell::wxHtmlTableCell(wxHtmlContainerCell *parent, const wxHtmlTag& t
     m_Spacing = (int)(m_PixelScale * (double)m_Spacing);
     m_Padding = (int)(m_PixelScale * (double)m_Padding);
 
-    if (m_HasBorders)
-        SetBorder(TABLE_BORDER_CLR_1, TABLE_BORDER_CLR_2);
+    if(tag.HasParam(wxT("BORDER")))
+    {
+        if(tag.GetParam("BORDER").IsEmpty())
+            m_Border = 1;
+        else
+            tag.GetParamAsInt(wxT("BORDER"), &m_Border);
+    }
+    if (m_Border == 1)
+        SetBorder(TABLE_BORDER_CLR_1, TABLE_BORDER_CLR_2, m_Border); // special case see wxHtmlContainerCell::Draw
+    else if (m_Border> 0)
+        SetBorder(TABLE_BORDER_CLR_1, TABLE_BORDER_CLR_2, (int)(m_PixelScale * (double)m_Border));
+    else
+        m_Border = 0;
+
 }
 
 
@@ -277,14 +290,19 @@ void wxHtmlTableCell::AddCell(wxHtmlContainerCell *cell, const wxHtmlTag& tag)
 
             if (wd[wd.length()-1] == wxT('%'))
             {
-                wxSscanf(wd.c_str(), wxT("%i%%"), &m_ColsInfo[c].width);
-                m_ColsInfo[c].units = wxHTML_UNITS_PERCENT;
+                if ( wxSscanf(wd.c_str(), wxT("%i%%"), &m_ColsInfo[c].width) == 1 )
+                {
+                    m_ColsInfo[c].units = wxHTML_UNITS_PERCENT;
+                }
             }
             else
             {
-                wxSscanf(wd.c_str(), wxT("%i"), &m_ColsInfo[c].width);
-                m_ColsInfo[c].width = (int)(m_PixelScale * (double)m_ColsInfo[c].width);
-                m_ColsInfo[c].units = wxHTML_UNITS_PIXELS;
+                long width;
+                if ( wd.ToLong(&width) )
+                {
+                    m_ColsInfo[c].width = (int)(m_PixelScale * (double)width);
+                    m_ColsInfo[c].units = wxHTML_UNITS_PIXELS;
+                }
             }
         }
     }
@@ -329,7 +347,7 @@ void wxHtmlTableCell::AddCell(wxHtmlContainerCell *cell, const wxHtmlTag& tag)
         if (bk.Ok())
             cell->SetBackgroundColour(bk);
     }
-    if (m_HasBorders)
+    if (m_Border > 0)
         cell->SetBorder(TABLE_BORDER_CLR_2, TABLE_BORDER_CLR_1);
 
     // vertical alignment:
@@ -403,7 +421,7 @@ void wxHtmlTableCell::ComputeMinMaxWidths()
     else
         m_MaxTotalWidth = m_MaxTotalWidth * 100 / (100 - percentage);
 
-    m_MaxTotalWidth += (m_NumCols + 1) * m_Spacing;
+    m_MaxTotalWidth += (m_NumCols + 1) * m_Spacing +  2 * m_Border;
 }
 
 void wxHtmlTableCell::Layout(int w)
@@ -451,7 +469,7 @@ void wxHtmlTableCell::Layout(int w)
            The algorithm tries to keep the table size less than w if possible.
        */
     {
-        int wpix = m_Width - (m_NumCols + 1) * m_Spacing;
+        int wpix = m_Width - (m_NumCols + 1) * m_Spacing - 2 * m_Border;  // Available space for cell content
         int i, j;
 
         // 1a. setup fixed-width columns:
@@ -490,7 +508,7 @@ void wxHtmlTableCell::Layout(int w)
             else
                 newWidth = newWidth * 100 / (100 - percentage);
 
-            newWidth = wxMin(newWidth, w - (m_NumCols + 1) * m_Spacing);
+            newWidth = wxMin(newWidth, w - (m_NumCols + 1) * m_Spacing - 2 * m_Border);
             wpix -= m_Width - newWidth;
             m_Width = newWidth;
         }
@@ -504,7 +522,7 @@ void wxHtmlTableCell::Layout(int w)
                 m_ColsInfo[i].pixwidth = wxMin(m_ColsInfo[i].width, 100) * wpix / 100;
 
                 // Make sure to leave enough space for the other columns
-                int minRequired = 0;
+                int minRequired = m_Border;
                 for (j = 0; j < m_NumCols; j++)
                 {
                     if ((m_ColsInfo[j].units == wxHTML_UNITS_PERCENT && j > i) ||
@@ -515,7 +533,7 @@ void wxHtmlTableCell::Layout(int w)
 
                 wtemp -= m_ColsInfo[i].pixwidth;
             }
-        wpix = wtemp;
+       wpix = wtemp; // minimum cells width
 
         // 1d. setup default columns (no width specification supplied):
         // The algorithm assigns calculates the maximum possible width if line
@@ -526,8 +544,8 @@ void wxHtmlTableCell::Layout(int w)
 
         for (i = j = 0; i < m_NumCols; i++)
             if (m_ColsInfo[i].width == 0) j++;
-        if (wpix < 0)
-            wpix = 0;
+        if (wpix < m_Border)
+            wpix = m_Border;
 
         // Assign widths
         for (i = 0; i < m_NumCols; i++)
@@ -565,12 +583,16 @@ void wxHtmlTableCell::Layout(int w)
 
     /* 2.  compute positions of columns: */
     {
-        int wpos = m_Spacing;
+        int wpos = m_Spacing + m_Border;
         for (int i = 0; i < m_NumCols; i++)
         {
             m_ColsInfo[i].leftpos = wpos;
             wpos += m_ColsInfo[i].pixwidth + m_Spacing;
         }
+
+        // add the remaining space to the last column
+        if (m_NumCols > 0 && wpos < m_Width - m_Border)
+            m_ColsInfo[m_NumCols-1].pixwidth += m_Width - wpos - m_Border;
     }
 
     /* 3.  sub-layout all cells: */
@@ -581,7 +603,7 @@ void wxHtmlTableCell::Layout(int w)
         int fullwid;
         wxHtmlContainerCell *actcell;
 
-        ypos[0] = m_Spacing;
+        ypos[0] = m_Spacing + m_Border;
         for (actrow = 1; actrow <= m_NumRows; actrow++) ypos[actrow] = -1;
         for (actrow = 0; actrow < m_NumRows; actrow++)
         {
@@ -623,7 +645,7 @@ void wxHtmlTableCell::Layout(int w)
                 actcell->SetPos(m_ColsInfo[actcol].leftpos, ypos[actrow]);
             }
         }
-        m_Height = ypos[m_NumRows];
+        m_Height = ypos[m_NumRows] + m_Border;
         delete[] ypos;
     }
 
@@ -631,7 +653,7 @@ void wxHtmlTableCell::Layout(int w)
     if (m_NumCols > 0)
     {
         int twidth = m_ColsInfo[m_NumCols-1].leftpos +
-                     m_ColsInfo[m_NumCols-1].pixwidth + m_Spacing;
+                     m_ColsInfo[m_NumCols-1].pixwidth + m_Spacing + m_Border;
         if (twidth > m_Width)
             m_Width = twidth;
     }
@@ -652,10 +674,12 @@ TAG_HANDLER_BEGIN(TABLE, "TABLE,TR,TD,TH")
     TAG_HANDLER_VARS
         wxHtmlTableCell* m_Table;
         wxString m_tAlign, m_rAlign;
+        wxHtmlContainerCell *m_enclosingContainer;
 
     TAG_HANDLER_CONSTR(TABLE)
     {
         m_Table = NULL;
+        m_enclosingContainer = NULL;
         m_tAlign = m_rAlign = wxEmptyString;
     }
 
@@ -668,11 +692,11 @@ TAG_HANDLER_BEGIN(TABLE, "TABLE,TR,TD,TH")
         if (tag.GetName() == wxT("TABLE"))
         {
             wxHtmlTableCell *oldt = m_Table;
-            wxHtmlContainerCell *oldcont;
 
-            oldcont = c = m_WParser->OpenContainer();
+            wxHtmlContainerCell *oldEnclosing = m_enclosingContainer;
+            m_enclosingContainer = c = m_WParser->OpenContainer();
 
-            m_Table = new wxHtmlTableCell(c, tag);
+            m_Table = new wxHtmlTableCell(c, tag, m_WParser->GetPixelScale());
 
             // width:
             {
@@ -704,11 +728,13 @@ TAG_HANDLER_BEGIN(TABLE, "TABLE,TR,TD,TH")
             ParseInner(tag);
 
             m_WParser->SetAlign(oldAlign);
-            m_WParser->SetContainer(oldcont);
+            m_WParser->SetContainer(m_enclosingContainer);
             m_WParser->CloseContainer();
 
             m_Table = oldt;
-            return true;
+            m_enclosingContainer = oldEnclosing;
+
+            return true; // ParseInner() called
         }
 
 
@@ -731,27 +757,56 @@ TAG_HANDLER_BEGIN(TABLE, "TABLE,TR,TD,TH")
 
                 m_WParser->OpenContainer();
 
-                if (tag.GetName() == wxT("TH")) /*header style*/
-                    m_WParser->SetAlign(wxHTML_ALIGN_CENTER);
-                else
-                    m_WParser->SetAlign(wxHTML_ALIGN_LEFT);
+                const bool isHeader = tag.GetName() == wxT("TH");
 
                 wxString als;
-
-                als = m_rAlign;
                 if (tag.HasParam(wxT("ALIGN")))
                     als = tag.GetParam(wxT("ALIGN"));
+                else
+                    als = m_rAlign;
                 als.MakeUpper();
+
                 if (als == wxT("RIGHT"))
                     m_WParser->SetAlign(wxHTML_ALIGN_RIGHT);
                 else if (als == wxT("LEFT"))
                     m_WParser->SetAlign(wxHTML_ALIGN_LEFT);
                 else if (als == wxT("CENTER"))
                     m_WParser->SetAlign(wxHTML_ALIGN_CENTER);
+                else // use default alignment
+                    m_WParser->SetAlign(isHeader ? wxHTML_ALIGN_CENTER
+                                                 : wxHTML_ALIGN_LEFT);
 
                 m_WParser->OpenContainer();
+
+                // the header should be rendered in bold by default
+                int boldOld = 0;
+                if ( isHeader )
+                {
+                    boldOld = m_WParser->GetFontBold();
+                    m_WParser->SetFontBold(true);
+                    m_WParser->GetContainer()->InsertCell(
+                        new wxHtmlFontCell(m_WParser->CreateCurrentFont()));
+                }
+
+                ParseInner(tag);
+
+                if ( isHeader )
+                {
+                    m_WParser->SetFontBold(boldOld);
+                    m_WParser->GetContainer()->InsertCell(
+                        new wxHtmlFontCell(m_WParser->CreateCurrentFont()));
+                }
+
+                // set the current container back to the enclosing one so that
+                // text outside of <th> and <td> isn't included in any cell
+                // (this happens often enough in practice because it's common
+                // to use whitespace between </td> and the next <td>):
+                m_WParser->SetContainer(m_enclosingContainer);
+
+                return true; // ParseInner() called
             }
         }
+
         return false;
     }
 
