@@ -1,10 +1,10 @@
 ////////////////////////////////////////////////////////////////////////////
-// Name:        bitmap.cpp
+// Name:        src/msw/bitmap.cpp
 // Purpose:     wxBitmap
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: bitmap.cpp,v 1.134 2005/07/22 17:13:45 ABX Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -17,10 +17,6 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "bitmap.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -28,6 +24,8 @@
     #pragma hdrstop
 #endif
 
+#include "wx/bitmap.h"
+
 #ifndef WX_PRECOMP
     #include <stdio.h>
 
@@ -36,22 +34,20 @@
     #include "wx/app.h"
     #include "wx/palette.h"
     #include "wx/dcmemory.h"
-    #include "wx/bitmap.h"
     #include "wx/icon.h"
+    #include "wx/log.h"
+    #include "wx/image.h"
 #endif
 
 #include "wx/msw/private.h"
-#include "wx/log.h"
+#include "wx/msw/dc.h"
 
 #if wxUSE_WXDIB
-#include "wx/msw/dib.h"
+    #include "wx/msw/dib.h"
 #endif
 
-#include "wx/image.h"
-#include "wx/xpmdecod.h"
-
-#ifdef wxHAVE_RAW_BITMAP
-#include "wx/rawbmp.h"
+#ifdef wxHAS_RAW_BITMAP
+    #include "wx/rawbmp.h"
 #endif
 
 // missing from mingw32 header
@@ -60,7 +56,7 @@
 #endif // no CLR_INVALID
 
 // ----------------------------------------------------------------------------
-// Bitmap data
+// wxBitmapRefData
 // ----------------------------------------------------------------------------
 
 class WXDLLEXPORT wxBitmapRefData : public wxGDIImageRefData
@@ -96,12 +92,12 @@ public:
     // MSW-specific
     // ------------
 
-#ifdef __WXDEBUG__
+#if wxDEBUG_LEVEL
     // this field is solely for error checking: we detect selecting a bitmap
     // into more than one DC at once or deleting a bitmap still selected into a
     // DC (both are serious programming errors under Windows)
     wxDC         *m_selectedInto;
-#endif // __WXDEBUG__
+#endif // wxDEBUG_LEVEL
 
 #if wxUSE_WXDIB
     // when GetRawData() is called for a DDB we need to convert it to a DIB
@@ -189,7 +185,7 @@ IMPLEMENT_DYNAMIC_CLASS(wxBitmapHandler, wxObject)
 
 wxBitmapRefData::wxBitmapRefData()
 {
-#ifdef __WXDEBUG__
+#if wxDEBUG_LEVEL
     m_selectedInto = NULL;
 #endif
     m_bitmapMask = NULL;
@@ -206,15 +202,20 @@ wxBitmapRefData::wxBitmapRefData()
 wxBitmapRefData::wxBitmapRefData(const wxBitmapRefData& data)
                : wxGDIImageRefData(data)
 {
-#ifdef __WXDEBUG__
+#if wxDEBUG_LEVEL
     m_selectedInto = NULL;
 #endif
 
-    // can't copy the mask as the other bitmap destroys it
+    // (deep) copy the mask if present
     m_bitmapMask = NULL;
+    if (data.m_bitmapMask)
+        m_bitmapMask = new wxMask(*data.m_bitmapMask);
+
+    // FIXME: we don't copy m_hBitmap currently but we should, see wxBitmap::
+    //        CloneGDIRefData()
 
     wxASSERT_MSG( !data.m_isDIB,
-                    _T("can't copy bitmap locked for raw access!") );
+                    wxT("can't copy bitmap locked for raw access!") );
     m_isDIB = false;
 
     m_hasAlpha = data.m_hasAlpha;
@@ -226,7 +227,7 @@ void wxBitmapRefData::Free()
                   wxT("deleting bitmap still selected into wxMemoryDC") );
 
 #if wxUSE_WXDIB
-    wxASSERT_MSG( !m_dib, _T("forgot to call wxBitmap::UngetRawData()!") );
+    wxASSERT_MSG( !m_dib, wxT("forgot to call wxBitmap::UngetRawData()!") );
 #endif
 
     if ( m_hBitmap)
@@ -237,54 +238,62 @@ void wxBitmapRefData::Free()
         }
     }
 
-    delete m_bitmapMask;
-    m_bitmapMask = NULL;
+    wxDELETE(m_bitmapMask);
 }
 
 // ----------------------------------------------------------------------------
 // wxBitmap creation
 // ----------------------------------------------------------------------------
 
-// this function should be called from all wxBitmap ctors
-void wxBitmap::Init()
-{
-    // m_refData = NULL; done in the base class ctor
-}
-
 wxGDIImageRefData *wxBitmap::CreateData() const
 {
     return new wxBitmapRefData;
 }
 
-wxObjectRefData *wxBitmap::CloneRefData(const wxObjectRefData *dataOrig) const
+wxGDIRefData *wxBitmap::CloneGDIRefData(const wxGDIRefData *dataOrig) const
 {
     const wxBitmapRefData *
-        data = wx_static_cast(const wxBitmapRefData *, dataOrig);
+        data = static_cast<const wxBitmapRefData *>(dataOrig);
     if ( !data )
         return NULL;
 
-    wxBitmap *self = wx_const_cast(wxBitmap *, this);
+    // FIXME: this method is backwards, it should just create a new
+    //        wxBitmapRefData using its copy ctor but instead it modifies this
+    //        bitmap itself and then returns its m_refData -- which works, of
+    //        course (except in !wxUSE_WXDIB), but is completely illogical
+    wxBitmap *self = const_cast<wxBitmap *>(this);
 
+    wxBitmapRefData *selfdata;
 #if wxUSE_WXDIB
     // copy the other bitmap
     if ( data->m_hBitmap )
     {
         wxDIB dib((HBITMAP)(data->m_hBitmap));
         self->CopyFromDIB(dib);
+
+        selfdata = static_cast<wxBitmapRefData *>(m_refData);
+        selfdata->m_hasAlpha = data->m_hasAlpha;
     }
     else
 #endif // wxUSE_WXDIB
     {
-        // don't copy the bitmap data, but do copy the size, depth, ...
-        self->m_refData = new wxBitmapRefData(*data);
+        // copy the bitmap data
+        selfdata = new wxBitmapRefData(*data);
+        self->m_refData = selfdata;
     }
 
-    return m_refData;
+    // copy also the mask
+    wxMask * const maskSrc = data->GetMask();
+    if ( maskSrc )
+    {
+        selfdata->SetMask(new wxMask(*maskSrc));
+    }
+
+    return selfdata;
 }
 
-#ifdef __WIN32__
-
-bool wxBitmap::CopyFromIconOrCursor(const wxGDIImage& icon)
+bool wxBitmap::CopyFromIconOrCursor(const wxGDIImage& icon,
+                                    wxBitmapTransparency transp)
 {
 #if !defined(__WXMICROWIN__) && !defined(__WXWINCE__)
     // it may be either HICON or HCURSOR
@@ -310,31 +319,51 @@ bool wxBitmap::CopyFromIconOrCursor(const wxGDIImage& icon)
 
     refData->m_hBitmap = (WXHBITMAP)iconInfo.hbmColor;
 
-#if wxUSE_WXDIB
-    // If the icon is 32 bits per pixel then it may have alpha channel data,
-    // although there are some icons that are 32 bpp but have no alpha... So
-    // convert to a DIB and manually check the 4th byte for each pixel.
-    BITMAP bm;
-    if ( ::GetObject(iconInfo.hbmColor, sizeof(BITMAP), (LPVOID)&bm)
-         && bm.bmBitsPixel == 32)
+    switch ( transp )
     {
-        wxDIB dib(iconInfo.hbmColor);
-        if (dib.IsOk())
-        {
-            unsigned char* pixels = dib.GetData();
-            for (int idx=0; idx<w*h*4; idx+=4)
+        default:
+            wxFAIL_MSG( wxT("unknown wxBitmapTransparency value") );
+
+        case wxBitmapTransparency_None:
+            // nothing to do, refData->m_hasAlpha is false by default
+            break;
+
+        case wxBitmapTransparency_Auto:
+#if wxUSE_WXDIB
+            // If the icon is 32 bits per pixel then it may have alpha channel
+            // data, although there are some icons that are 32 bpp but have no
+            // alpha... So convert to a DIB and manually check the 4th byte for
+            // each pixel.
             {
-                if (pixels[idx+3] != 0)
+                BITMAP bm;
+                if ( ::GetObject(iconInfo.hbmColor, sizeof(bm), &bm) &&
+                        (bm.bmBitsPixel == 32) )
                 {
-                    // If there is an alpha byte that is non-zero then set the
-                    // alpha flag and bail out of the loop.
-                    refData->m_hasAlpha = true;
-                    break;
+                    wxDIB dib(iconInfo.hbmColor);
+                    if (dib.IsOk())
+                    {
+                        const unsigned char* pixels = dib.GetData();
+                        for (int idx = 0; idx < w*h*4; idx+=4)
+                        {
+                            if (pixels[idx+3] != 0)
+                            {
+                                // If there is an alpha byte that is non-zero
+                                // then set the alpha flag and stop checking
+                                refData->m_hasAlpha = true;
+                                break;
+                            }
+                        }
+                    }
                 }
             }
-        }
+            break;
+#endif // wxUSE_WXDIB
+
+        case wxBitmapTransparency_Always:
+            refData->m_hasAlpha = true;
+            break;
     }
-#endif
+
     if ( !refData->m_hasAlpha )
     {
         // the mask returned by GetIconInfo() is inverted compared to the usual
@@ -346,39 +375,39 @@ bool wxBitmap::CopyFromIconOrCursor(const wxGDIImage& icon)
     ::DeleteObject(iconInfo.hbmMask);
 
     return true;
-#else
+#else // __WXMICROWIN__ || __WXWINCE__
     wxUnusedVar(icon);
+    wxUnusedVar(transp);
+
     return false;
-#endif
+#endif // !__WXWINCE__/__WXWINCE__
 }
 
-#endif // Win32
-
-bool wxBitmap::CopyFromCursor(const wxCursor& cursor)
+bool wxBitmap::CopyFromCursor(const wxCursor& cursor, wxBitmapTransparency transp)
 {
     UnRef();
 
     if ( !cursor.Ok() )
         return false;
 
-    return CopyFromIconOrCursor(cursor);
+    return CopyFromIconOrCursor(cursor, transp);
 }
 
-bool wxBitmap::CopyFromIcon(const wxIcon& icon)
+bool wxBitmap::CopyFromIcon(const wxIcon& icon, wxBitmapTransparency transp)
 {
     UnRef();
 
     if ( !icon.Ok() )
         return false;
 
-    return CopyFromIconOrCursor(icon);
+    return CopyFromIconOrCursor(icon, transp);
 }
 
 #ifndef NEVER_USE_DIB
 
 bool wxBitmap::CopyFromDIB(const wxDIB& dib)
 {
-    wxCHECK_MSG( dib.IsOk(), false, _T("invalid DIB in CopyFromDIB") );
+    wxCHECK_MSG( dib.IsOk(), false, wxT("invalid DIB in CopyFromDIB") );
 
 #ifdef SOMETIMES_USE_DIB
     HBITMAP hbitmap = dib.CreateDDB();
@@ -420,8 +449,6 @@ wxBitmap::~wxBitmap()
 
 wxBitmap::wxBitmap(const char bits[], int width, int height, int depth)
 {
-    Init();
-
 #ifndef __WXMICROWIN__
     wxBitmapRefData *refData = new wxBitmapRefData;
     m_refData = refData;
@@ -456,7 +483,7 @@ wxBitmap::wxBitmap(const char bits[], int width, int height, int depth)
                     reversed |= (unsigned char)(val & 0x01);
                     val >>= 1;
                 }
-                *dst++ = reversed;
+                *dst++ = ~reversed;
             }
 
             if ( padding )
@@ -484,52 +511,19 @@ wxBitmap::wxBitmap(const char bits[], int width, int height, int depth)
 #endif
 }
 
-// Create from XPM data
-bool wxBitmap::CreateFromXpm(const char **data)
-{
-#if wxUSE_IMAGE && wxUSE_XPM
-    Init();
-
-    wxCHECK_MSG( data != NULL, false, wxT("invalid bitmap data") )
-
-    wxXPMDecoder decoder;
-    wxImage img = decoder.ReadData(data);
-    wxCHECK_MSG( img.Ok(), false, wxT("invalid bitmap data") )
-
-    *this = wxBitmap(img);
-    return true;
-#else
-    wxUnusedVar(data);
-    return false;
-#endif
-}
-
-wxBitmap::wxBitmap(int w, int h, int d)
-{
-    Init();
-
-    (void)Create(w, h, d);
-}
-
 wxBitmap::wxBitmap(int w, int h, const wxDC& dc)
 {
-    Init();
-
     (void)Create(w, h, dc);
 }
 
-wxBitmap::wxBitmap(void *data, long type, int width, int height, int depth)
+wxBitmap::wxBitmap(const void* data, wxBitmapType type, int width, int height, int depth)
 {
-    Init();
-
     (void)Create(data, type, width, height, depth);
 }
 
 wxBitmap::wxBitmap(const wxString& filename, wxBitmapType type)
 {
-    Init();
-
-    LoadFile(filename, (int)type);
+    LoadFile(filename, type);
 }
 
 bool wxBitmap::Create(int width, int height, int depth)
@@ -539,9 +533,14 @@ bool wxBitmap::Create(int width, int height, int depth)
 
 bool wxBitmap::Create(int width, int height, const wxDC& dc)
 {
-    wxCHECK_MSG( dc.Ok(), false, _T("invalid HDC in wxBitmap::Create()") );
+    wxCHECK_MSG( dc.IsOk(), false, wxT("invalid HDC in wxBitmap::Create()") );
 
-    return DoCreate(width, height, -1, dc.GetHDC());
+    const wxMSWDCImpl *impl = wxDynamicCast( dc.GetImpl(), wxMSWDCImpl );
+
+    if (impl)
+        return DoCreate(width, height, -1, impl->GetHDC());
+    else
+        return false;
 }
 
 bool wxBitmap::DoCreate(int w, int h, int d, WXHDC hdc)
@@ -802,8 +801,6 @@ wxImage wxBitmap::ConvertToImage() const
 // wxImage to/from conversions
 // ----------------------------------------------------------------------------
 
-#if wxUSE_WXDIB
-
 bool wxBitmap::CreateFromImage(const wxImage& image, int depth)
 {
     return CreateFromImage(image, depth, 0);
@@ -811,12 +808,19 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth)
 
 bool wxBitmap::CreateFromImage(const wxImage& image, const wxDC& dc)
 {
-    wxCHECK_MSG( dc.Ok(), false,
-                    _T("invalid HDC in wxBitmap::CreateFromImage()") );
+    wxCHECK_MSG( dc.IsOk(), false,
+                    wxT("invalid HDC in wxBitmap::CreateFromImage()") );
 
-    return CreateFromImage(image, -1, dc.GetHDC());
+    const wxMSWDCImpl *impl = wxDynamicCast( dc.GetImpl(), wxMSWDCImpl );
+
+    if (impl)
+        return CreateFromImage(image, -1, impl->GetHDC());
+    else
+        return false;
 }
 
+#if wxUSE_WXDIB
+
 bool wxBitmap::CreateFromImage(const wxImage& image, int depth, WXHDC hdc)
 {
     wxCHECK_MSG( image.Ok(), false, wxT("invalid image") );
@@ -831,12 +835,17 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth, WXHDC hdc)
     if ( !dib.IsOk() )
         return false;
 
+    const bool hasAlpha = image.HasAlpha();
+
+    if (depth == -1)
+      depth = dib.GetDepth();
 
     // store the bitmap parameters
-    wxBitmapRefData *refData = new wxBitmapRefData;
+    wxBitmapRefData * const refData = new wxBitmapRefData;
     refData->m_width = w;
     refData->m_height = h;
-    refData->m_hasAlpha = image.HasAlpha();
+    refData->m_hasAlpha = hasAlpha;
+    refData->m_depth = depth;
 
     m_refData = refData;
 
@@ -847,20 +856,17 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth, WXHDC hdc)
     // are we going to use DIB?
     //
     // NB: DDBs don't support alpha so if we have alpha channel we must use DIB
-    if ( image.HasAlpha() || wxShouldCreateDIB(w, h, depth, hdc) )
+    if ( hasAlpha || wxShouldCreateDIB(w, h, depth, hdc) )
     {
         // don't delete the DIB section in dib object dtor
         hbitmap = dib.Detach();
 
         refData->m_isDIB = true;
-        refData->m_depth = dib.GetDepth();
     }
 #ifndef ALWAYS_USE_DIB
     else // we need to convert DIB to DDB
     {
         hbitmap = dib.CreateDDB((HDC)hdc);
-
-        refData->m_depth = depth == -1 ? dib.GetDepth() : depth;
     }
 #endif // !ALWAYS_USE_DIB
 
@@ -898,7 +904,7 @@ bool wxBitmap::CreateFromImage(const wxImage& image, int depth, WXHDC hdc)
         hbitmap = ::CreateBitmap(w, h, 1, 1, data);
         if ( !hbitmap )
         {
-            wxLogLastError(_T("CreateBitmap(mask)"));
+            wxLogLastError(wxT("CreateBitmap(mask)"));
         }
         else
         {
@@ -994,7 +1000,22 @@ wxImage wxBitmap::ConvertToImage() const
     return image;
 }
 
-#endif // wxUSE_WXDIB
+#else // !wxUSE_WXDIB
+
+bool
+wxBitmap::CreateFromImage(const wxImage& WXUNUSED(image),
+                          int WXUNUSED(depth),
+                          WXHDC WXUNUSED(hdc))
+{
+    return false;
+}
+
+wxImage wxBitmap::ConvertToImage() const
+{
+    return wxImage();
+}
+
+#endif // wxUSE_WXDIB/!wxUSE_WXDIB
 
 #endif // wxUSE_IMAGE
 
@@ -1002,7 +1023,7 @@ wxImage wxBitmap::ConvertToImage() const
 // loading and saving bitmaps
 // ----------------------------------------------------------------------------
 
-bool wxBitmap::LoadFile(const wxString& filename, long type)
+bool wxBitmap::LoadFile(const wxString& filename, wxBitmapType type)
 {
     UnRef();
 
@@ -1014,7 +1035,7 @@ bool wxBitmap::LoadFile(const wxString& filename, long type)
 
         return handler->LoadFile(this, filename, type, -1, -1);
     }
-#if wxUSE_IMAGE
+#if wxUSE_IMAGE && wxUSE_WXDIB
     else // no bitmap handler found
     {
         wxImage image;
@@ -1030,7 +1051,7 @@ bool wxBitmap::LoadFile(const wxString& filename, long type)
     return false;
 }
 
-bool wxBitmap::Create(void *data, long type, int width, int height, int depth)
+bool wxBitmap::Create(const void* data, wxBitmapType type, int width, int height, int depth)
 {
     UnRef();
 
@@ -1049,8 +1070,8 @@ bool wxBitmap::Create(void *data, long type, int width, int height, int depth)
 }
 
 bool wxBitmap::SaveFile(const wxString& filename,
-                        int type,
-                        const wxPalette *palette)
+                        wxBitmapType type,
+                        const wxPalette *palette) const
 {
     wxBitmapHandler *handler = wxDynamicCast(FindHandler(type), wxBitmapHandler);
 
@@ -1058,7 +1079,7 @@ bool wxBitmap::SaveFile(const wxString& filename,
     {
         return handler->SaveFile(this, filename, type, palette);
     }
-#if wxUSE_IMAGE
+#if wxUSE_IMAGE && wxUSE_WXDIB
     else // no bitmap handler found
     {
         // FIXME what about palette? shouldn't we use it?
@@ -1076,8 +1097,14 @@ bool wxBitmap::SaveFile(const wxString& filename,
 // ----------------------------------------------------------------------------
 // sub bitmap extraction
 // ----------------------------------------------------------------------------
+wxBitmap wxBitmap::GetSubBitmap( const wxRect& rect ) const
+{
+        MemoryHDC dcSrc;
+        SelectInHDC selectSrc(dcSrc, GetHbitmap());
+        return GetSubBitmapOfHDC( rect, (WXHDC)dcSrc );
+}
 
-wxBitmap wxBitmap::GetSubBitmap( const wxRect& rect) const
+wxBitmap wxBitmap::GetSubBitmapOfHDC( const wxRect& rect, WXHDC hdc ) const
 {
     wxCHECK_MSG( Ok() &&
                  (rect.x >= 0) && (rect.y >= 0) &&
@@ -1098,18 +1125,17 @@ wxBitmap wxBitmap::GetSubBitmap( const wxRect& rect) const
               dcDst;
 
     {
-        SelectInHDC selectSrc(dcSrc, GetHbitmap()),
-                    selectDst(dcDst, GetHbitmapOf(ret));
+        SelectInHDC selectDst(dcDst, GetHbitmapOf(ret));
 
-        if ( !selectSrc || !selectDst )
+        if ( !selectDst )
         {
-            wxLogLastError(_T("SelectObjct(hBitmap)"));
+            wxLogLastError(wxT("SelectObject(destBitmap)"));
         }
 
         if ( !::BitBlt(dcDst, 0, 0, rect.width, rect.height,
-                       dcSrc, rect.x, rect.y, SRCCOPY) )
+                       (HDC)hdc, rect.x, rect.y, SRCCOPY) )
         {
-            wxLogLastError(_T("BitBlt"));
+            wxLogLastError(wxT("BitBlt"));
         }
     }
 
@@ -1124,7 +1150,7 @@ wxBitmap wxBitmap::GetSubBitmap( const wxRect& rect) const
         if ( !::BitBlt(dcDst, 0, 0, rect.width, rect.height,
                        dcSrc, rect.x, rect.y, SRCCOPY) )
         {
-            wxLogLastError(_T("BitBlt"));
+            wxLogLastError(wxT("BitBlt"));
         }
 
         wxMask *mask = new wxMask((WXHBITMAP) hbmpMask);
@@ -1143,13 +1169,13 @@ wxBitmap wxBitmap::GetSubBitmap( const wxRect& rect) const
 wxPalette* wxBitmap::GetPalette() const
 {
     return GetBitmapData() ? &GetBitmapData()->m_bitmapPalette
-                           : (wxPalette *) NULL;
+                           : NULL;
 }
 #endif
 
 wxMask *wxBitmap::GetMask() const
 {
-    return GetBitmapData() ? GetBitmapData()->GetMask() : (wxMask *) NULL;
+    return GetBitmapData() ? GetBitmapData()->GetMask() : NULL;
 }
 
 wxBitmap wxBitmap::GetMaskBitmap() const
@@ -1161,24 +1187,15 @@ wxBitmap wxBitmap::GetMaskBitmap() const
     return bmp;
 }
 
-#ifdef __WXDEBUG__
-
 wxDC *wxBitmap::GetSelectedInto() const
 {
-    return GetBitmapData() ? GetBitmapData()->m_selectedInto : (wxDC *) NULL;
-}
-
+#if wxDEBUG_LEVEL
+    return GetBitmapData() ? GetBitmapData()->m_selectedInto : NULL;
+#else
+    return NULL;
 #endif
-
-#if WXWIN_COMPATIBILITY_2_4
-
-int wxBitmap::GetQuality() const
-{
-    return 0;
 }
 
-#endif // WXWIN_COMPATIBILITY_2_4
-
 void wxBitmap::UseAlpha()
 {
     if ( GetBitmapData() )
@@ -1194,15 +1211,15 @@ bool wxBitmap::HasAlpha() const
 // wxBitmap setters
 // ----------------------------------------------------------------------------
 
-#ifdef __WXDEBUG__
-
 void wxBitmap::SetSelectedInto(wxDC *dc)
 {
+#if wxDEBUG_LEVEL
     if ( GetBitmapData() )
         GetBitmapData()->m_selectedInto = dc;
-}
-
+#else
+    wxUnusedVar(dc);
 #endif
+}
 
 #if wxUSE_PALETTE
 
@@ -1222,19 +1239,12 @@ void wxBitmap::SetMask(wxMask *mask)
     GetBitmapData()->SetMask(mask);
 }
 
-#if WXWIN_COMPATIBILITY_2_4
-
-void wxBitmap::SetQuality(int WXUNUSED(quality))
-{
-}
-
-#endif // WXWIN_COMPATIBILITY_2_4
-
 // ----------------------------------------------------------------------------
 // raw bitmap access support
 // ----------------------------------------------------------------------------
 
-#ifdef wxHAVE_RAW_BITMAP
+#ifdef wxHAS_RAW_BITMAP
+
 void *wxBitmap::GetRawData(wxPixelDataBase& data, int bpp)
 {
 #if wxUSE_WXDIB
@@ -1251,7 +1261,7 @@ void *wxBitmap::GetRawData(wxPixelDataBase& data, int bpp)
     if ( !GetBitmapData()->m_isDIB )
     {
         wxCHECK_MSG( !GetBitmapData()->m_dib, NULL,
-                        _T("GetRawData() may be called only once") );
+                        wxT("GetRawData() may be called only once") );
 
         wxDIB *dib = new wxDIB(*this);
         if ( !dib->IsOk() )
@@ -1274,7 +1284,7 @@ void *wxBitmap::GetRawData(wxPixelDataBase& data, int bpp)
     DIBSECTION ds;
     if ( ::GetObject(hDIB, sizeof(ds), &ds) != sizeof(DIBSECTION) )
     {
-        wxFAIL_MSG( _T("failed to get DIBSECTION from a DIB?") );
+        wxFAIL_MSG( wxT("failed to get DIBSECTION from a DIB?") );
 
         return NULL;
     }
@@ -1282,7 +1292,7 @@ void *wxBitmap::GetRawData(wxPixelDataBase& data, int bpp)
     // check that the bitmap is in correct format
     if ( ds.dsBm.bmBitsPixel != bpp )
     {
-        wxFAIL_MSG( _T("incorrect bitmap type in wxBitmap::GetRawData()") );
+        wxFAIL_MSG( wxT("incorrect bitmap type in wxBitmap::GetRawData()") );
 
         return NULL;
     }
@@ -1340,7 +1350,7 @@ void wxBitmap::UngetRawData(wxPixelDataBase& dataBase)
     }
 #endif // wxUSE_WXDIB
 }
-#endif // #ifdef wxHAVE_RAW_BITMAP
+#endif // wxHAS_RAW_BITMAP
 
 // ----------------------------------------------------------------------------
 // wxMask
@@ -1351,6 +1361,39 @@ wxMask::wxMask()
     m_maskBitmap = 0;
 }
 
+// Copy constructor
+wxMask::wxMask(const wxMask &mask)
+      : wxObject()
+{
+    BITMAP bmp;
+
+    HDC srcDC = CreateCompatibleDC(0);
+    HDC destDC = CreateCompatibleDC(0);
+
+    // GetBitmapDimensionEx won't work if SetBitmapDimensionEx wasn't used
+    // so we'll use GetObject() API here:
+    if (::GetObject((HGDIOBJ)mask.m_maskBitmap, sizeof(bmp), &bmp) == 0)
+    {
+        wxFAIL_MSG(wxT("Cannot retrieve the dimensions of the wxMask to copy"));
+        return;
+    }
+
+    // create our HBITMAP
+    int w = bmp.bmWidth, h = bmp.bmHeight;
+    m_maskBitmap = (WXHBITMAP)CreateCompatibleBitmap(srcDC, w, h);
+
+    // copy the mask's HBITMAP into our HBITMAP
+    SelectObject(srcDC, (HBITMAP) mask.m_maskBitmap);
+    SelectObject(destDC, (HBITMAP) m_maskBitmap);
+
+    BitBlt(destDC, 0, 0, w, h, srcDC, 0, 0, SRCCOPY);
+
+    SelectObject(srcDC, 0);
+    DeleteDC(srcDC);
+    SelectObject(destDC, 0);
+    DeleteDC(destDC);
+}
+
 // Construct a mask from a bitmap and a colour indicating
 // the transparent area
 wxMask::wxMask(const wxBitmap& bitmap, const wxColour& colour)
@@ -1385,7 +1428,7 @@ bool wxMask::Create(const wxBitmap& bitmap)
 {
 #ifndef __WXMICROWIN__
     wxCHECK_MSG( bitmap.Ok() && bitmap.GetDepth() == 1, false,
-                 _T("can't create mask from invalid or not monochrome bitmap") );
+                 wxT("can't create mask from invalid or not monochrome bitmap") );
 
     if ( m_maskBitmap )
     {
@@ -1444,7 +1487,7 @@ bool wxMask::Create(const wxBitmap& bitmap, int paletteIndex)
 bool wxMask::Create(const wxBitmap& bitmap, const wxColour& colour)
 {
 #ifndef __WXMICROWIN__
-    wxCHECK_MSG( bitmap.Ok(), false, _T("invalid bitmap in wxMask::Create") );
+    wxCHECK_MSG( bitmap.Ok(), false, wxT("invalid bitmap in wxMask::Create") );
 
     if ( m_maskBitmap )
     {
@@ -1471,7 +1514,7 @@ bool wxMask::Create(const wxBitmap& bitmap, const wxColour& colour)
 
     // SelectObject() will fail
     wxASSERT_MSG( !bitmap.GetSelectedInto(),
-                  _T("bitmap can't be selected in another DC") );
+                  wxT("bitmap can't be selected in another DC") );
 
     HGDIOBJ hbmpSrcOld = ::SelectObject(srcDC, GetHbitmapOf(bitmap));
     if ( !hbmpSrcOld )
@@ -1516,37 +1559,37 @@ bool wxMask::Create(const wxBitmap& bitmap, const wxColour& colour)
 // ----------------------------------------------------------------------------
 
 bool wxBitmapHandler::Create(wxGDIImage *image,
-                             void *data,
-                             long flags,
+                             const void* data,
+                             wxBitmapType type,
                              int width, int height, int depth)
 {
     wxBitmap *bitmap = wxDynamicCast(image, wxBitmap);
 
-    return bitmap ? Create(bitmap, data, flags, width, height, depth) : false;
+    return bitmap && Create(bitmap, data, type, width, height, depth);
 }
 
 bool wxBitmapHandler::Load(wxGDIImage *image,
                            const wxString& name,
-                           long flags,
+                           wxBitmapType type,
                            int width, int height)
 {
     wxBitmap *bitmap = wxDynamicCast(image, wxBitmap);
 
-    return bitmap ? LoadFile(bitmap, name, flags, width, height) : false;
+    return bitmap && LoadFile(bitmap, name, type, width, height);
 }
 
-bool wxBitmapHandler::Save(wxGDIImage *image,
+bool wxBitmapHandler::Save(const wxGDIImage *image,
                            const wxString& name,
-                           int type)
+                           wxBitmapType type) const
 {
     wxBitmap *bitmap = wxDynamicCast(image, wxBitmap);
 
-    return bitmap ? SaveFile(bitmap, name, type) : false;
+    return bitmap && SaveFile(bitmap, name, type);
 }
 
 bool wxBitmapHandler::Create(wxBitmap *WXUNUSED(bitmap),
-                             void *WXUNUSED(data),
-                             long WXUNUSED(type),
+                             const void* WXUNUSED(data),
+                             wxBitmapType WXUNUSED(type),
                              int WXUNUSED(width),
                              int WXUNUSED(height),
                              int WXUNUSED(depth))
@@ -1556,72 +1599,21 @@ bool wxBitmapHandler::Create(wxBitmap *WXUNUSED(bitmap),
 
 bool wxBitmapHandler::LoadFile(wxBitmap *WXUNUSED(bitmap),
                                const wxString& WXUNUSED(name),
-                               long WXUNUSED(type),
+                               wxBitmapType WXUNUSED(type),
                                int WXUNUSED(desiredWidth),
                                int WXUNUSED(desiredHeight))
 {
     return false;
 }
 
-bool wxBitmapHandler::SaveFile(wxBitmap *WXUNUSED(bitmap),
+bool wxBitmapHandler::SaveFile(const wxBitmap *WXUNUSED(bitmap),
                                const wxString& WXUNUSED(name),
-                               int WXUNUSED(type),
-                               const wxPalette *WXUNUSED(palette))
+                               wxBitmapType WXUNUSED(type),
+                               const wxPalette *WXUNUSED(palette)) const
 {
     return false;
 }
 
-// ----------------------------------------------------------------------------
-// DIB functions
-// ----------------------------------------------------------------------------
-
-#ifndef __WXMICROWIN__
-bool wxCreateDIB(long xSize, long ySize, long bitsPerPixel,
-                 HPALETTE hPal, LPBITMAPINFO* lpDIBHeader)
-{
-   unsigned long   i, headerSize;
-
-   // Allocate space for a DIB header
-   headerSize = (sizeof(BITMAPINFOHEADER) + (256 * sizeof(PALETTEENTRY)));
-   LPBITMAPINFO lpDIBheader = (BITMAPINFO *) malloc(headerSize);
-   LPPALETTEENTRY lpPe = (PALETTEENTRY *)((BYTE*)lpDIBheader + sizeof(BITMAPINFOHEADER));
-
-   GetPaletteEntries(hPal, 0, 256, lpPe);
-
-   memset(lpDIBheader, 0x00, sizeof(BITMAPINFOHEADER));
-
-   // Fill in the static parts of the DIB header
-   lpDIBheader->bmiHeader.biSize = sizeof(BITMAPINFOHEADER);
-   lpDIBheader->bmiHeader.biWidth = xSize;
-   lpDIBheader->bmiHeader.biHeight = ySize;
-   lpDIBheader->bmiHeader.biPlanes = 1;
-
-   // this value must be 1, 4, 8 or 24 so PixelDepth can only be
-   lpDIBheader->bmiHeader.biBitCount = (WORD)(bitsPerPixel);
-   lpDIBheader->bmiHeader.biCompression = BI_RGB;
-   lpDIBheader->bmiHeader.biSizeImage = (xSize * abs(ySize) * bitsPerPixel) >> 3;
-   lpDIBheader->bmiHeader.biClrUsed = 256;
-
-
-   // Initialize the DIB palette
-   for (i = 0; i < 256; i++) {
-      lpDIBheader->bmiColors[i].rgbReserved = lpPe[i].peFlags;
-      lpDIBheader->bmiColors[i].rgbRed = lpPe[i].peRed;
-      lpDIBheader->bmiColors[i].rgbGreen = lpPe[i].peGreen;
-      lpDIBheader->bmiColors[i].rgbBlue = lpPe[i].peBlue;
-   }
-
-   *lpDIBHeader = lpDIBheader;
-
-   return true;
-}
-
-void wxFreeDIB(LPBITMAPINFO lpDIBHeader)
-{
-    free(lpDIBHeader);
-}
-#endif
-
 // ----------------------------------------------------------------------------
 // global helper functions implemented here
 // ----------------------------------------------------------------------------
@@ -1639,23 +1631,33 @@ HICON wxBitmapToIconOrCursor(const wxBitmap& bmp,
         return 0;
     }
 
-    wxMask* mask;
-    wxBitmap newbmp;
     if ( bmp.HasAlpha() )
     {
-        // Convert alpha to a mask.  NOTE: It would be better to actually put
-        // the alpha into the icon instead of making a mask, but I don't have
-        // time to figure that out today.
-        wxImage img = bmp.ConvertToImage();
-        img.ConvertAlphaToMask();
-        newbmp = wxBitmap(img);
-        mask = newbmp.GetMask();
-    }
-    else
-    {
-        mask = bmp.GetMask();
+        // Create an empty mask bitmap.
+        // it doesn't seem to work if we mess with the mask at all.
+        HBITMAP hMonoBitmap = CreateBitmap(bmp.GetWidth(),bmp.GetHeight(),1,1,NULL);
+
+        ICONINFO iconInfo;
+        wxZeroMemory(iconInfo);
+        iconInfo.fIcon = iconWanted;  // do we want an icon or a cursor?
+        if ( !iconWanted )
+        {
+            iconInfo.xHotspot = hotSpotX;
+            iconInfo.yHotspot = hotSpotY;
+        }
+
+        iconInfo.hbmMask = hMonoBitmap;
+        iconInfo.hbmColor = GetHbitmapOf(bmp);
+
+        HICON hicon = ::CreateIconIndirect(&iconInfo);
+
+        ::DeleteObject(hMonoBitmap);
+
+        return hicon;
     }
 
+    wxMask* mask = bmp.GetMask();
+
     if ( !mask )
     {
         // we must have a mask for an icon, so even if it's probably incorrect,
@@ -1686,7 +1688,7 @@ HICON wxBitmapToIconOrCursor(const wxBitmap& bmp,
         if ( !::BitBlt(dcDst, 0, 0, bmp.GetWidth(), bmp.GetHeight(),
                        dcSrc, 0, 0, SRCAND) )
         {
-            wxLogLastError(_T("BitBlt"));
+            wxLogLastError(wxT("BitBlt"));
         }
     }
 
@@ -1717,7 +1719,7 @@ HCURSOR wxBitmapToHCURSOR(const wxBitmap& bmp, int hotSpotX, int hotSpotY)
 HBITMAP wxInvertMask(HBITMAP hbmpMask, int w, int h)
 {
 #ifndef __WXMICROWIN__
-    wxCHECK_MSG( hbmpMask, 0, _T("invalid bitmap in wxInvertMask") );
+    wxCHECK_MSG( hbmpMask, 0, wxT("invalid bitmap in wxInvertMask") );
 
     // get width/height from the bitmap if not given
     if ( !w || !h )
