@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        choice.cpp
+// Name:        src/msw/choice.cpp
 // Purpose:     wxChoice
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin to derive from wxChoiceBase
 // Created:     04/01/98
-// RCS-ID:      $Id: choice.cpp,v 1.107.2.2 2006/03/24 13:41:17 JS Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -17,10 +17,6 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "choice.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -30,9 +26,11 @@
 
 #if wxUSE_CHOICE && !(defined(__SMARTPHONE__) && defined(__WXWINCE__))
 
+#include "wx/choice.h"
+
 #ifndef WX_PRECOMP
-    #include "wx/choice.h"
     #include "wx/utils.h"
+    #include "wx/app.h"
     #include "wx/log.h"
     #include "wx/brush.h"
     #include "wx/settings.h"
@@ -73,7 +71,7 @@ wxBEGIN_FLAGS( wxChoiceStyle )
 
 wxEND_FLAGS( wxChoiceStyle )
 
-IMPLEMENT_DYNAMIC_CLASS_XTI(wxChoice, wxControl,"wx/choice.h")
+IMPLEMENT_DYNAMIC_CLASS_XTI(wxChoice, wxControlWithItems,"wx/choice.h")
 
 wxBEGIN_PROPERTIES_TABLE(wxChoice)
     wxEVENT_PROPERTY( Select , wxEVT_COMMAND_CHOICE_SELECTED , wxCommandEvent )
@@ -89,7 +87,7 @@ wxEND_HANDLERS_TABLE()
 
 wxCONSTRUCTOR_4( wxChoice , wxWindow* , Parent , wxWindowID , Id , wxPoint , Position , wxSize , Size )
 #else
-IMPLEMENT_DYNAMIC_CLASS(wxChoice, wxControl)
+IMPLEMENT_DYNAMIC_CLASS(wxChoice, wxControlWithItems)
 #endif
 /*
     TODO PROPERTIES
@@ -120,8 +118,8 @@ bool wxChoice::Create(wxWindow *parent,
     wxASSERT_MSG( !(style & wxCB_DROPDOWN) &&
                   !(style & wxCB_READONLY) &&
                   !(style & wxCB_SIMPLE),
-                  _T("this style flag is ignored by wxChoice, you ")
-                  _T("probably want to use a wxComboBox") );
+                  wxT("this style flag is ignored by wxChoice, you ")
+                  wxT("probably want to use a wxComboBox") );
 
     return CreateAndInit(parent, id, pos, size, n, choices, style,
                          validator, name);
@@ -145,10 +143,6 @@ bool wxChoice::CreateAndInit(wxWindow *parent,
         return false;
 
 
-    // choice/combobox normally has "white" (depends on colour scheme, of
-    // course) background rather than inheriting the parent's background
-    SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
-
     // initialize the controls contents
     for ( int i = 0; i < n; i++ )
     {
@@ -156,11 +150,26 @@ bool wxChoice::CreateAndInit(wxWindow *parent,
     }
 
     // and now we may finally size the control properly (if needed)
-    SetBestSize(size);
+    SetInitialSize(size);
 
     return true;
 }
 
+void wxChoice::SetLabel(const wxString& label)
+{
+    if ( FindString(label) == wxNOT_FOUND )
+    {
+        // unless we explicitly do this here, CB_GETCURSEL will continue to
+        // return the index of the previously selected item which will result
+        // in wrongly replacing the value being set now with the previously
+        // value if the user simply opens and closes (without selecting
+        // anything) the combobox popup
+        SetSelection(-1);
+    }
+
+    wxChoiceBase::SetLabel(label);
+}
+
 bool wxChoice::Create(wxWindow *parent,
                       wxWindowID id,
                       const wxPoint& pos,
@@ -212,95 +221,109 @@ WXDWORD wxChoice::MSWGetStyle(long style, WXDWORD *exstyle) const
     return msStyle;
 }
 
+#ifndef EP_EDITTEXT
+    #define EP_EDITTEXT         1
+    #define ETS_NORMAL          1
+#endif
+
+wxVisualAttributes
+wxChoice::GetClassDefaultAttributes(wxWindowVariant WXUNUSED(variant))
+{
+    // it is important to return valid values for all attributes from here,
+    // GetXXX() below rely on this
+    wxVisualAttributes attrs;
+
+    // FIXME: Use better dummy window?
+    wxWindow* wnd = wxTheApp->GetTopWindow();
+    if (!wnd)
+        return attrs;
+
+    attrs.font = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+
+    // there doesn't seem to be any way to get the text colour using themes
+    // API: TMT_TEXTCOLOR doesn't work neither for EDIT nor COMBOBOX
+    attrs.colFg = wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT);
+
+    // NB: use EDIT, not COMBOBOX (the latter works in XP but not Vista)
+    attrs.colBg = wnd->MSWGetThemeColour(L"EDIT",
+                                         EP_EDITTEXT, 
+                                         ETS_NORMAL,
+                                         ThemeColourBackground,
+                                         wxSYS_COLOUR_WINDOW);
+
+    return attrs;
+}
+
 wxChoice::~wxChoice()
 {
-    Free();
+    Clear();
 }
 
 // ----------------------------------------------------------------------------
 // adding/deleting items to/from the list
 // ----------------------------------------------------------------------------
 
-int wxChoice::DoAppend(const wxString& item)
+int wxChoice::DoInsertItems(const wxArrayStringsAdapter& items,
+                            unsigned int pos,
+                            void **clientData, wxClientDataType type)
 {
-    int n = (int)SendMessage(GetHwnd(), CB_ADDSTRING, 0, (LPARAM)item.c_str());
-    if ( n == CB_ERR )
+    MSWAllocStorage(items, CB_INITSTORAGE);
+
+    const bool append = pos == GetCount();
+
+    // use CB_ADDSTRING when appending at the end to make sure the control is
+    // resorted if it has wxCB_SORT style
+    const unsigned msg = append ? CB_ADDSTRING : CB_INSERTSTRING;
+
+    if ( append )
+        pos = 0;
+
+    int n = wxNOT_FOUND;
+    const unsigned numItems = items.GetCount();
+    for ( unsigned i = 0; i < numItems; ++i )
     {
-        wxLogLastError(wxT("SendMessage(CB_ADDSTRING)"));
-    }
-    else // ok
-    {
-        // we need to refresh our size in order to have enough space for the
-        // newly added items
-        if ( !IsFrozen() )
-            UpdateVisibleHeight();
+        n = MSWInsertOrAppendItem(pos, items[i], msg);
+        if ( n == wxNOT_FOUND )
+            return n;
+
+        if ( !append )
+            pos++;
+
+        AssignNewItemClientData(n, clientData, i, type);
     }
 
+    // we need to refresh our size in order to have enough space for the
+    // newly added items
+    if ( !IsFrozen() )
+        MSWUpdateDropDownHeight();
+
     InvalidateBestSize();
+
     return n;
 }
 
-int wxChoice::DoInsert(const wxString& item, int pos)
+void wxChoice::DoDeleteOneItem(unsigned int n)
 {
-    wxCHECK_MSG(!(GetWindowStyle() & wxCB_SORT), -1, wxT("can't insert into sorted list"));
-    wxCHECK_MSG((pos>=0) && (pos<=GetCount()), -1, wxT("invalid index"));
-
-    int n = (int)SendMessage(GetHwnd(), CB_INSERTSTRING, pos, (LPARAM)item.c_str());
-    if ( n == CB_ERR )
-    {
-        wxLogLastError(wxT("SendMessage(CB_INSERTSTRING)"));
-    }
-    else // ok
-    {
-        if ( !IsFrozen() )
-            UpdateVisibleHeight();
-    }
-
-    InvalidateBestSize();
-    return n;
-}
-
-void wxChoice::Delete(int n)
-{
-    wxCHECK_RET( n < GetCount(), wxT("invalid item index in wxChoice::Delete") );
-
-    if ( HasClientObjectData() )
-    {
-        delete GetClientObject(n);
-    }
+    wxCHECK_RET( IsValid(n), wxT("invalid item index in wxChoice::Delete") );
 
     SendMessage(GetHwnd(), CB_DELETESTRING, n, 0);
 
     if ( !IsFrozen() )
-        UpdateVisibleHeight();
+        MSWUpdateDropDownHeight();
 
     InvalidateBestSize();
 }
 
-void wxChoice::Clear()
+void wxChoice::DoClear()
 {
-    Free();
-
     SendMessage(GetHwnd(), CB_RESETCONTENT, 0, 0);
 
     if ( !IsFrozen() )
-        UpdateVisibleHeight();
+        MSWUpdateDropDownHeight();
 
     InvalidateBestSize();
 }
 
-void wxChoice::Free()
-{
-    if ( HasClientObjectData() )
-    {
-        size_t count = GetCount();
-        for ( size_t n = 0; n < count; n++ )
-        {
-            delete GetClientObject(n);
-        }
-    }
-}
-
 // ----------------------------------------------------------------------------
 // selection
 // ----------------------------------------------------------------------------
@@ -330,21 +353,21 @@ void wxChoice::SetSelection(int n)
 // string list functions
 // ----------------------------------------------------------------------------
 
-int wxChoice::GetCount() const
+unsigned int wxChoice::GetCount() const
 {
-    return (int)SendMessage(GetHwnd(), CB_GETCOUNT, 0, 0);
+    return (unsigned int)SendMessage(GetHwnd(), CB_GETCOUNT, 0, 0);
 }
 
-int wxChoice::FindString(const wxString& s) const
+int wxChoice::FindString(const wxString& s, bool bCase) const
 {
 #if defined(__WATCOMC__) && defined(__WIN386__)
     // For some reason, Watcom in WIN386 mode crashes in the CB_FINDSTRINGEXACT message.
     // wxChoice::Do it the long way instead.
-    int count = GetCount();
-    for ( int i = 0; i < count; i++ )
+    unsigned int count = GetCount();
+    for ( unsigned int i = 0; i < count; i++ )
     {
         // as CB_FINDSTRINGEXACT is case insensitive, be case insensitive too
-        if ( GetString(i).IsSameAs(s, false) )
+        if (GetString(i).IsSameAs(s, bCase))
             return i;
     }
 
@@ -354,57 +377,58 @@ int wxChoice::FindString(const wxString& s) const
    //passed to SendMessage, so we have to do it ourselves in that case
    if ( s.empty() )
    {
-     int count = GetCount();
-     for ( int i = 0; i < count; i++ )
-     {
-       if ( GetString(i).empty() )
-           return i;
-     }
+       unsigned int count = GetCount();
+       for ( unsigned int i = 0; i < count; i++ )
+       {
+         if (GetString(i).empty())
+             return i;
+       }
 
-     return wxNOT_FOUND;
+       return wxNOT_FOUND;
+   }
+   else if (bCase)
+   {
+       // back to base class search for not native search type
+       return wxItemContainerImmutable::FindString( s, bCase );
    }
    else
    {
-     int pos = (int)SendMessage(GetHwnd(), CB_FINDSTRINGEXACT,
-                                (WPARAM)-1, (LPARAM)s.c_str());
+       int pos = (int)SendMessage(GetHwnd(), CB_FINDSTRINGEXACT,
+                                  (WPARAM)-1, (LPARAM)s.wx_str());
 
-     return pos == LB_ERR ? wxNOT_FOUND : pos;
+       return pos == LB_ERR ? wxNOT_FOUND : pos;
    }
 #endif // Watcom/!Watcom
 }
 
-void wxChoice::SetString(int n, const wxString& s)
+void wxChoice::SetString(unsigned int n, const wxString& s)
 {
-    wxCHECK_RET( n >= 0 && n < GetCount(),
-                 wxT("invalid item index in wxChoice::SetString") );
+    wxCHECK_RET( IsValid(n), wxT("invalid item index in wxChoice::SetString") );
 
     // we have to delete and add back the string as there is no way to change a
     // string in place
 
-    // we need to preserve the client data
-    void *data;
-    if ( m_clientDataItemsType != wxClientData_None )
-    {
-        data = DoGetItemClientData(n);
-    }
-    else // no client data
-    {
-        data = NULL;
-    }
+    // we need to preserve the client data manually
+    void *oldData = NULL;
+    wxClientData *oldObjData = NULL;
+    if ( HasClientUntypedData() )
+        oldData = GetClientData(n);
+    else if ( HasClientObjectData() )
+        oldObjData = GetClientObject(n);
 
     ::SendMessage(GetHwnd(), CB_DELETESTRING, n, 0);
-    ::SendMessage(GetHwnd(), CB_INSERTSTRING, n, (LPARAM)s.c_str() );
+    ::SendMessage(GetHwnd(), CB_INSERTSTRING, n, (LPARAM)s.wx_str() );
 
-    if ( data )
-    {
-        DoSetItemClientData(n, data);
-    }
-    //else: it's already NULL by default
+    // restore the client data
+    if ( oldData )
+        SetClientData(n, oldData);
+    else if ( oldObjData )
+        SetClientObject(n, oldObjData);
 
     InvalidateBestSize();
 }
 
-wxString wxChoice::GetString(int n) const
+wxString wxChoice::GetString(unsigned int n) const
 {
     int len = (int)::SendMessage(GetHwnd(), CB_GETLBTEXTLEN, n, 0);
 
@@ -430,7 +454,7 @@ wxString wxChoice::GetString(int n) const
 // client data
 // ----------------------------------------------------------------------------
 
-void wxChoice::DoSetItemClientData( int n, void* clientData )
+void wxChoice::DoSetItemClientData(unsigned int n, void* clientData)
 {
     if ( ::SendMessage(GetHwnd(), CB_SETITEMDATA,
                        n, (LPARAM)clientData) == CB_ERR )
@@ -439,7 +463,7 @@ void wxChoice::DoSetItemClientData( int n, void* clientData )
     }
 }
 
-void* wxChoice::DoGetItemClientData( int n ) const
+void* wxChoice::DoGetItemClientData(unsigned int n) const
 {
     LPARAM rc = SendMessage(GetHwnd(), CB_GETITEMDATA, n, 0);
     if ( rc == CB_ERR )
@@ -453,24 +477,46 @@ void* wxChoice::DoGetItemClientData( int n ) const
     return (void *)rc;
 }
 
-void wxChoice::DoSetItemClientObject( int n, wxClientData* clientData )
-{
-    DoSetItemClientData(n, clientData);
-}
-
-wxClientData* wxChoice::DoGetItemClientObject( int n ) const
-{
-    return (wxClientData *)DoGetItemClientData(n);
-}
-
 // ----------------------------------------------------------------------------
-// wxMSW specific helpers
+// wxMSW-specific geometry management
 // ----------------------------------------------------------------------------
 
-void wxChoice::UpdateVisibleHeight()
+namespace
+{
+
+// there is a difference between the height passed to CB_SETITEMHEIGHT and the
+// real height of the combobox; it is probably not constant for all Windows
+// versions/settings but right now I don't know how to find what it is so it is
+// temporarily hardcoded to its value under XP systems with normal fonts sizes
+const int COMBO_HEIGHT_ADJ = 6;
+
+} // anonymous namespace
+
+void wxChoice::MSWUpdateVisibleHeight()
+{
+    if ( m_heightOwn != wxDefaultCoord )
+    {
+        ::SendMessage(GetHwnd(), CB_SETITEMHEIGHT,
+                      (WPARAM)-1, m_heightOwn - COMBO_HEIGHT_ADJ);
+    }
+}
+
+#if wxUSE_DEFERRED_SIZING
+void wxChoice::MSWEndDeferWindowPos()
+{
+    // we can only set the height of the choice itself now as it is reset to
+    // default every time the control is resized
+    MSWUpdateVisibleHeight();
+
+    wxChoiceBase::MSWEndDeferWindowPos();
+}
+#endif // wxUSE_DEFERRED_SIZING
+
+void wxChoice::MSWUpdateDropDownHeight()
 {
     // be careful to not change the width here
-    DoSetSize(wxDefaultCoord, wxDefaultCoord, wxDefaultCoord, GetSize().y, wxSIZE_USE_EXISTING);
+    DoSetSize(wxDefaultCoord, wxDefaultCoord, wxDefaultCoord, GetSize().y,
+              wxSIZE_USE_EXISTING);
 }
 
 void wxChoice::DoMoveWindow(int x, int y, int width, int height)
@@ -493,110 +539,108 @@ void wxChoice::DoMoveWindow(int x, int y, int width, int height)
 
 void wxChoice::DoGetSize(int *w, int *h) const
 {
+    wxControl::DoGetSize(w, h);
+
     // this is weird: sometimes, the height returned by Windows is clearly the
     // total height of the control including the drop down list -- but only
-    // sometimes, and normally it isn't... I have no idea about what to do with
-    // this
-    wxControl::DoGetSize(w, h);
+    // sometimes, and sometimes it isn't so work around this here by using our
+    // own stored value if we have it
+    if ( h && m_heightOwn != wxDefaultCoord )
+        *h = m_heightOwn;
 }
 
 void wxChoice::DoSetSize(int x, int y,
                          int width, int height,
                          int sizeFlags)
 {
-    int heightOrig = height;
+    const int heightBest = GetBestSize().y;
+
+    // we need the real height below so get the current one if it's not given
+    if ( height == wxDefaultCoord )
+    {
+        // height not specified, use the same as before
+        DoGetSize(NULL, &height);
+    }
+    else if ( height == heightBest )
+    {
+        // we don't need to manually manage our height, let the system use the
+        // default one
+        m_heightOwn = wxDefaultCoord;
+    }
+    else // non-default height specified
+    {
+        // set our new own height but be careful not to make it too big: the
+        // native control apparently stores it as a single byte and so setting
+        // own height to 256 pixels results in default height being used (255
+        // is still ok)
+        m_heightOwn = height;
+
+        if ( m_heightOwn > UCHAR_MAX )
+            m_heightOwn = UCHAR_MAX;
+        // nor too small: see MSWUpdateVisibleHeight()
+        else if ( m_heightOwn < COMBO_HEIGHT_ADJ )
+            m_heightOwn = COMBO_HEIGHT_ADJ;
+    }
 
-    int widthCurrent, heightCurrent;
-    DoGetSize(&widthCurrent, &heightCurrent);
 
     // the height which we must pass to Windows should be the total height of
     // the control including the drop down list while the height given to us
-    // is, of course, just the height of the permanently visible part of it
-    if ( height != wxDefaultCoord && height != heightCurrent )
+    // is, of course, just the height of the permanently visible part of it so
+    // add the drop down height to it
+
+    // don't make the drop down list too tall, arbitrarily limit it to 30
+    // items max and also don't make it too small if it's currently empty
+    size_t nItems = GetCount();
+    if (!HasFlag(wxCB_SIMPLE))
     {
-        // don't make the drop down list too tall, arbitrarily limit it to 40
-        // items max and also don't leave it empty
-        size_t nItems = GetCount();
         if ( !nItems )
             nItems = 9;
-        else if ( nItems > 24 )
-            nItems = 24;
-
-        // add space for the drop down list
-        const int hItem = SendMessage(GetHwnd(), CB_GETITEMHEIGHT, 0, 0);
-        height += hItem*(nItems + 1);
+        else if ( nItems > 30 )
+            nItems = 30;
     }
-    else // keep the same height as now
+
+    const int hItem = SendMessage(GetHwnd(), CB_GETITEMHEIGHT, 0, 0);
+    int heightWithItems = 0;
+    if (!HasFlag(wxCB_SIMPLE))
+        heightWithItems = height + hItem*nItems;
+    else
+        heightWithItems = SetHeightSimpleComboBox(nItems);
+
+
+    // do resize the native control
+    wxControl::DoSetSize(x, y, width, heightWithItems, sizeFlags);
+
+
+    // make the control itself of the requested height: notice that this
+    // must be done after changing its size or it has no effect (apparently
+    // the height is reset to default during the control layout) and that it's
+    // useless to to do it when using the deferred sizing -- in this case it
+    // will be done from MSWEndDeferWindowPos()
+#if wxUSE_DEFERRED_SIZING
+    if ( m_pendingSize == wxDefaultSize )
     {
-        // normally wxWindow::DoSetSize() checks if we set the same size as the
-        // window already has and does nothing in this case, but for us the
-        // check fails as the size we pass to it includes the dropdown while
-        // the size returned by our GetSize() does not, so test if the size
-        // didn't really change ourselves here
-        if ( width == wxDefaultCoord || width == widthCurrent )
-        {
-            // size doesn't change, what about position?
-            int xCurrent, yCurrent;
-            DoGetPosition(&xCurrent, &yCurrent);
-            const bool defMeansUnchanged = !(sizeFlags & wxSIZE_ALLOW_MINUS_ONE);
-            if ( ((x == wxDefaultCoord && defMeansUnchanged) || x == xCurrent)
-                    &&
-                 ((y == wxDefaultCoord && defMeansUnchanged) || y == yCurrent) )
-            {
-                // nothing changes, nothing to do
-                return;
-            }
-        }
-
-        // We cannot pass wxDefaultCoord as height to wxControl. wxControl uses
-        // wxGetWindowRect() to determine the current height of the combobox,
-        // and then again sets the combobox's height to that value. Unfortunately,
-        // wxGetWindowRect doesn't include the dropdown list's height (at least
-        // on Win2K), so this would result in a combobox with dropdown height of
-        // 1 pixel. We have to determine the default height ourselves and call
-        // wxControl with that value instead.
-        //
-        // Also notice that sometimes CB_GETDROPPEDCONTROLRECT seems to return
-        // wildly incorrect values (~32000) which looks like a bug in it, just
-        // ignore them in this case
-        RECT r;
-        if ( ::SendMessage(GetHwnd(), CB_GETDROPPEDCONTROLRECT, 0, (LPARAM) &r)
-                    && r.bottom < 30000 )
-        {
-            height = heightCurrent + r.bottom - r.top;
-        }
+        // not using deferred sizing, update it immediately
+        MSWUpdateVisibleHeight();
     }
-
-    wxControl::DoSetSize(x, y, width, height, sizeFlags);
-
-    // I'm commenting this out since the code appears to make choices
-    // and comboxes too high when they have associated sizers. I'm sure this
-    // is not the end of the story, which is why I'm leaving it #if'ed out for
-    // now. JACS.
-#if 0
-    // if the height specified for the visible part of the control is
-    // different from the current one, we need to change it separately
-    // as it is not affected by normal WM_SETSIZE
-    if ( height != wxDefaultCoord )
+    else // in the middle of deferred sizing
     {
-        const int delta = heightOrig - GetSize().y;
-        if ( delta )
-        {
-            int h = ::SendMessage(GetHwnd(), CB_GETITEMHEIGHT, (WPARAM)-1, 0);
-            SendMessage(GetHwnd(), CB_SETITEMHEIGHT, (WPARAM)-1, h + delta);
-        }
+        // we need to report the size of the visible part of the control back
+        // in GetSize() and not height stored by DoSetSize() in m_pendingSize
+        m_pendingSize = wxSize(width, height);
     }
-#else
-    wxUnusedVar(heightOrig);
-#endif
+#else // !wxUSE_DEFERRED_SIZING
+    // always update the visible height immediately
+    MSWUpdateVisibleHeight();
+#endif // wxUSE_DEFERRED_SIZING
 }
 
 wxSize wxChoice::DoGetBestSize() const
 {
     // find the widest string
     int wChoice = 0;
-    const size_t nItems = GetCount();
-    for ( size_t i = 0; i < nItems; i++ )
+    int hChoice;
+    const unsigned int nItems = GetCount();
+    for ( unsigned int i = 0; i < nItems; i++ )
     {
         int wLine;
         GetTextExtent(GetString(i), &wLine, NULL);
@@ -611,12 +655,30 @@ wxSize wxChoice::DoGetBestSize() const
 
     // the combobox should be slightly larger than the widest string
     wChoice += 5*GetCharWidth();
+    if( HasFlag( wxCB_SIMPLE ) )
+    {
+        hChoice = SetHeightSimpleComboBox( nItems );
+    }
+    else
+        hChoice = EDIT_HEIGHT_FROM_CHAR_HEIGHT(GetCharHeight());
 
-    wxSize best(wChoice, EDIT_HEIGHT_FROM_CHAR_HEIGHT(GetCharHeight()));
+    wxSize best(wChoice, hChoice);
     CacheBestSize(best);
     return best;
 }
 
+int wxChoice::SetHeightSimpleComboBox(int nItems) const
+{
+    int cx, cy;
+    wxGetCharSize( GetHWND(), &cx, &cy, GetFont() );
+    int hItem = SendMessage(GetHwnd(), CB_GETITEMHEIGHT, (WPARAM)-1, 0);
+    return EDIT_HEIGHT_FROM_CHAR_HEIGHT( cy ) * wxMin( wxMax( nItems, 3 ), 6 ) + hItem - 1;
+}
+
+// ----------------------------------------------------------------------------
+// MSW message handlers
+// ----------------------------------------------------------------------------
+
 WXLRESULT wxChoice::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam)
 {
     switch ( nMsg )
@@ -658,20 +720,61 @@ WXLRESULT wxChoice::MSWWindowProc(WXUINT nMsg, WXWPARAM wParam, WXLPARAM lParam)
 
 bool wxChoice::MSWCommand(WXUINT param, WXWORD WXUNUSED(id))
 {
+    /*
+        The native control provides a great variety in the events it sends in
+        the different selection scenarios (undoubtedly for greater amusement of
+        the programmers using it). For the reference, here are the cases when
+        the final selection is accepted (things are quite interesting when it
+        is cancelled too):
+
+        A. Selecting with just the arrows without opening the dropdown:
+            1. CBN_SELENDOK
+            2. CBN_SELCHANGE
+
+        B. Opening dropdown with F4 and selecting with arrows:
+            1. CBN_DROPDOWN
+            2. many CBN_SELCHANGE while changing selection in the list
+            3. CBN_SELENDOK
+            4. CBN_CLOSEUP
+
+        C. Selecting with the mouse:
+            1. CBN_DROPDOWN
+            -- no intermediate CBN_SELCHANGEs --
+            2. CBN_SELENDOK
+            3. CBN_CLOSEUP
+            4. CBN_SELCHANGE
+
+        Admire the different order of messages in all of those cases, it must
+        surely have taken a lot of effort to Microsoft developers to achieve
+        such originality.
+     */
     switch ( param )
     {
         case CBN_DROPDOWN:
-            // we don't want to track selection using CB_GETCURSEL while the
-            // dropdown is opened
+            // we use this value both because we don't want to track selection
+            // using CB_GETCURSEL while the dropdown is opened and because we
+            // need to reset the selection back to it if it's eventually
+            // cancelled by user
             m_lastAcceptedSelection = GetCurrentSelection();
             break;
 
         case CBN_CLOSEUP:
-            // it should be safe to use CB_GETCURSEL again
-            m_lastAcceptedSelection = wxID_NONE;
+            // if the selection was accepted by the user, it should have been
+            // reset to wxID_NONE by CBN_SELENDOK, otherwise the selection was
+            // cancelled and we must restore the old one
+            if ( m_lastAcceptedSelection != wxID_NONE )
+            {
+                SetSelection(m_lastAcceptedSelection);
+                m_lastAcceptedSelection = wxID_NONE;
+            }
             break;
 
-        case CBN_SELCHANGE:
+        case CBN_SELENDOK:
+            // reset it to prevent CBN_CLOSEUP from undoing the selection (it's
+            // ok to reset it now as GetCurrentSelection() will now return the
+            // same thing anyhow)
+            m_lastAcceptedSelection = wxID_NONE;
+
             {
                 const int n = GetSelection();
 
@@ -682,18 +785,24 @@ bool wxChoice::MSWCommand(WXUINT param, WXWORD WXUNUSED(id))
                 if ( n > -1 )
                 {
                     event.SetString(GetStringSelection());
-                    if ( HasClientObjectData() )
-                        event.SetClientObject( GetClientObject(n) );
-                    else if ( HasClientUntypedData() )
-                        event.SetClientData( GetClientData(n) );
+                    InitCommandEventWithItems(event, n);
                 }
 
                 ProcessCommand(event);
             }
-            return true;
+            break;
+
+        // don't handle CBN_SELENDCANCEL: just leave m_lastAcceptedSelection
+        // valid and the selection will be undone in CBN_CLOSEUP above
+
+        // don't handle CBN_SELCHANGE neither, we don't want to generate events
+        // while the dropdown is opened -- but do add it if we ever need this
+
+        default:
+            return false;
     }
 
-    return false;
+    return true;
 }
 
 WXHBRUSH wxChoice::MSWControlColor(WXHDC hDC, WXHWND hWnd)
@@ -705,4 +814,3 @@ WXHBRUSH wxChoice::MSWControlColor(WXHDC hDC, WXHWND hWnd)
 }
 
 #endif // wxUSE_CHOICE && !(__SMARTPHONE__ && __WXWINCE__)
-
