@@ -1,10 +1,10 @@
 /////////////////////////////////////////////////////////////////////////////
-// Name:        clipbrd.cpp
+// Name:        src/msw/clipbrd.cpp
 // Purpose:     Clipboard functionality
 // Author:      Julian Smart
 // Modified by:
 // Created:     04/01/98
-// RCS-ID:      $Id: clipbrd.cpp,v 1.64 2005/03/11 11:49:44 VZ Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -17,10 +17,6 @@
 // headers
 // ---------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "clipbrd.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -30,6 +26,8 @@
 
 #if wxUSE_CLIPBOARD
 
+#include "wx/clipbrd.h"
+
 #ifndef WX_PRECOMP
     #include "wx/object.h"
     #include "wx/event.h"
@@ -38,14 +36,14 @@
     #include "wx/bitmap.h"
     #include "wx/utils.h"
     #include "wx/intl.h"
+    #include "wx/log.h"
+    #include "wx/dataobj.h"
 #endif
 
 #if wxUSE_METAFILE
     #include "wx/metafile.h"
 #endif
 
-#include "wx/log.h"
-#include "wx/clipbrd.h"
 
 #include <string.h>
 
@@ -60,10 +58,6 @@
 // the functions using wxDataObject in wxClipboard
 //#define wxUSE_DATAOBJ wxUSE_DRAG_AND_DROP
 
-#if wxUSE_DATAOBJ
-    #include "wx/dataobj.h"
-#endif
-
 #if wxUSE_OLE && !defined(__WXWINCE__)
     // use OLE clipboard
     #define wxUSE_OLE_CLIPBOARD 1
@@ -96,7 +90,9 @@ bool wxOpenClipboard()
         gs_wxClipboardIsOpen = ::OpenClipboard((HWND)win->GetHWND()) != 0;
 
         if ( !gs_wxClipboardIsOpen )
+        {
             wxLogSysError(_("Failed to open the clipboard."));
+        }
 
         return gs_wxClipboardIsOpen;
     }
@@ -298,8 +294,8 @@ bool wxSetClipboardData(wxDataFormat dataFormat,
                 handle = SetClipboardData(dataFormat, hGlobalMemory);
                 break;
             }
-            // Only tested with non-Unicode, Visual C++ 6.0 so far
-#if defined(__VISUALC__) && !defined(UNICODE)
+            // Only tested with Visual C++ 6.0 so far
+#if defined(__VISUALC__)
         case wxDF_HTML:
             {
                 char* html = (char *)data;
@@ -336,19 +332,19 @@ bool wxSetClipboardData(wxDataFormat dataFormat,
                 // string when you overwrite it so you follow up with code to replace
                 // the 0 appended at the end with a '\r'...
                 char *ptr = strstr(buf, "StartHTML");
-                wsprintf(ptr+10, "%08u", strstr(buf, "<html>") - buf);
+                sprintf(ptr+10, "%08u", strstr(buf, "<html>") - buf);
                 *(ptr+10+8) = '\r';
 
                 ptr = strstr(buf, "EndHTML");
-                wsprintf(ptr+8, "%08u", strlen(buf));
+                sprintf(ptr+8, "%08u", strlen(buf));
                 *(ptr+8+8) = '\r';
 
                 ptr = strstr(buf, "StartFragment");
-                wsprintf(ptr+14, "%08u", strstr(buf, "<!--StartFrag") - buf);
+                sprintf(ptr+14, "%08u", strstr(buf, "<!--StartFrag") - buf);
                 *(ptr+14+8) = '\r';
 
                 ptr = strstr(buf, "EndFragment");
-                wsprintf(ptr+12, "%08u", strstr(buf, "<!--EndFrag") - buf);
+                sprintf(ptr+12, "%08u", strstr(buf, "<!--EndFrag") - buf);
                 *(ptr+12+8) = '\r';
 
                 // Now you have everything in place ready to put on the
@@ -535,13 +531,13 @@ wxClipboard::wxClipboard()
     wxOleInitialize();
 #endif
 
-    m_clearOnExit = false;
+    m_lastDataObject = NULL;
     m_isOpened = false;
 }
 
 wxClipboard::~wxClipboard()
 {
-    if ( m_clearOnExit )
+    if ( m_lastDataObject )
     {
         Clear();
     }
@@ -553,11 +549,23 @@ wxClipboard::~wxClipboard()
 
 void wxClipboard::Clear()
 {
+    if ( IsUsingPrimarySelection() )
+        return;
+
 #if wxUSE_OLE_CLIPBOARD
-    HRESULT hr = OleSetClipboard(NULL);
-    if ( FAILED(hr) )
+    if (m_lastDataObject)
     {
-        wxLogApiError(wxT("OleSetClipboard(NULL)"), hr);
+        // don't touch data set by other applications
+        HRESULT hr = OleIsCurrentClipboard(m_lastDataObject);
+        if (S_OK == hr)
+        {
+            hr = OleSetClipboard(NULL);
+            if ( FAILED(hr) )
+            {
+                wxLogApiError(wxT("OleSetClipboard(NULL)"), hr);
+            }
+        }
+        m_lastDataObject = NULL;
     }
 #endif // wxUSE_OLE_CLIPBOARD
 }
@@ -565,19 +573,24 @@ void wxClipboard::Clear()
 bool wxClipboard::Flush()
 {
 #if wxUSE_OLE_CLIPBOARD
-    HRESULT hr = OleFlushClipboard();
-    if ( FAILED(hr) )
+    if (m_lastDataObject)
     {
-        wxLogApiError(wxT("OleFlushClipboard"), hr);
+        // don't touch data set by other applications
+        HRESULT hr = OleIsCurrentClipboard(m_lastDataObject);
+        m_lastDataObject = NULL;
+        if (S_OK == hr)
+        {
+            hr = OleFlushClipboard();
+            if ( FAILED(hr) )
+            {
+                wxLogApiError(wxT("OleFlushClipboard"), hr);
 
-        return false;
-    }
-    else
-    {
-        m_clearOnExit = false;
-
-        return true;
+                return false;
+            }
+            return true;
+        }
     }
+    return false;
 #else // !wxUSE_OLE_CLIPBOARD
     return false;
 #endif // wxUSE_OLE_CLIPBOARD/!wxUSE_OLE_CLIPBOARD
@@ -605,6 +618,9 @@ bool wxClipboard::IsOpened() const
 
 bool wxClipboard::SetData( wxDataObject *data )
 {
+    if ( IsUsingPrimarySelection() )
+        return false;
+
 #if !wxUSE_OLE_CLIPBOARD
     (void)wxEmptyClipboard();
 #endif // wxUSE_OLE_CLIPBOARD
@@ -617,6 +633,9 @@ bool wxClipboard::SetData( wxDataObject *data )
 
 bool wxClipboard::AddData( wxDataObject *data )
 {
+    if ( IsUsingPrimarySelection() )
+        return false;
+
     wxCHECK_MSG( data, false, wxT("data is invalid") );
 
 #if wxUSE_OLE_CLIPBOARD
@@ -630,6 +649,12 @@ bool wxClipboard::AddData( wxDataObject *data )
         return false;
     }
 
+    // we have to call either OleSetClipboard(NULL) or OleFlushClipboard() when
+    // using OLE clipboard when the app terminates - by default, we call
+    // OleSetClipboard(NULL) which won't waste RAM, but the app can call
+    // wxClipboard::Flush() to change this
+    m_lastDataObject = data->GetInterface();
+
     // we have a problem here because we should delete wxDataObject, but we
     // can't do it because IDataObject which we just gave to the clipboard
     // would try to use it when it will need the data. IDataObject is ref
@@ -637,12 +662,6 @@ bool wxClipboard::AddData( wxDataObject *data )
     // and tell it to delete wxDataObject when it is deleted itself.
     data->SetAutoDelete();
 
-    // we have to call either OleSetClipboard(NULL) or OleFlushClipboard() when
-    // using OLE clipboard when the app terminates - by default, we call
-    // OleSetClipboard(NULL) which won't waste RAM, but the app can call
-    // wxClipboard::Flush() to chaneg this
-    m_clearOnExit = true;
-
     return true;
 #elif wxUSE_DATAOBJ
     wxCHECK_MSG( wxIsClipboardOpened(), false, wxT("clipboard not open") );
@@ -710,11 +729,14 @@ void wxClipboard::Close()
 
 bool wxClipboard::IsSupported( const wxDataFormat& format )
 {
-    return wxIsClipboardFormatAvailable(format);
+    return !IsUsingPrimarySelection() && wxIsClipboardFormatAvailable(format);
 }
 
 bool wxClipboard::GetData( wxDataObject& data )
 {
+    if ( IsUsingPrimarySelection() )
+        return false;
+
 #if wxUSE_OLE_CLIPBOARD
     IDataObject *pDataObject = NULL;
     HRESULT hr = OleGetClipboard(&pDataObject);
@@ -750,7 +772,7 @@ bool wxClipboard::GetData( wxDataObject& data )
     // enumerate all explicit formats on the clipboard.
     // note that this does not include implicit / synthetic (automatically
     // converted) formats.
-#ifdef __WXDEBUG__
+#if wxDEBUG_LEVEL >= 2
     // get the format enumerator
     IEnumFORMATETC *pEnumFormatEtc = NULL;
     hr = pDataObject->EnumFormatEtc(DATADIR_GET, &pEnumFormatEtc);
@@ -783,7 +805,7 @@ bool wxClipboard::GetData( wxDataObject& data )
 
         pEnumFormatEtc->Release();
     }
-#endif // Debug
+#endif // wxDEBUG_LEVEL >= 2
 
     STGMEDIUM medium;
     // stop at the first valid format found on the clipboard
@@ -921,4 +943,3 @@ bool wxClipboard::GetData( wxDataObject& data )
 }
 
 #endif // wxUSE_CLIPBOARD
-
