@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     10.05.98
-// RCS-ID:      $Id: dataobj.cpp,v 1.87 2005/07/01 13:39:03 ABX Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -17,10 +17,6 @@
 // headers
 // ----------------------------------------------------------------------------
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-    #pragma implementation "dataobj.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -31,6 +27,8 @@
 #ifndef WX_PRECOMP
     #include "wx/intl.h"
     #include "wx/log.h"
+    #include "wx/utils.h"
+    #include "wx/wxcrtvararg.h"
 #endif
 
 #include "wx/dataobj.h"
@@ -56,18 +54,18 @@
 #include "wx/msw/dib.h"
 
 #ifndef CFSTR_SHELLURL
-#define CFSTR_SHELLURL _T("UniformResourceLocator")
+#define CFSTR_SHELLURL wxT("UniformResourceLocator")
 #endif
 
 // ----------------------------------------------------------------------------
 // functions
 // ----------------------------------------------------------------------------
 
-#ifdef __WXDEBUG__
+#if wxDEBUG_LEVEL
     static const wxChar *GetTymedName(DWORD tymed);
-#else // !Debug
+#else // !wxDEBUG_LEVEL
     #define GetTymedName(tymed) wxEmptyString
-#endif // Debug/!Debug
+#endif // wxDEBUG_LEVEL/!wxDEBUG_LEVEL
 
 // ----------------------------------------------------------------------------
 // wxIEnumFORMATETC interface implementation
@@ -79,20 +77,20 @@ public:
     wxIEnumFORMATETC(const wxDataFormat* formats, ULONG nCount);
     virtual ~wxIEnumFORMATETC() { delete [] m_formats; }
 
-    DECLARE_IUNKNOWN_METHODS;
-
     // IEnumFORMATETC
     STDMETHODIMP Next(ULONG celt, FORMATETC *rgelt, ULONG *pceltFetched);
     STDMETHODIMP Skip(ULONG celt);
     STDMETHODIMP Reset();
     STDMETHODIMP Clone(IEnumFORMATETC **ppenum);
 
+    DECLARE_IUNKNOWN_METHODS;
+
 private:
     CLIPFORMAT *m_formats;  // formats we can provide data in
     ULONG       m_nCount,   // number of formats we support
                 m_nCurrent; // current enum position
 
-    DECLARE_NO_COPY_CLASS(wxIEnumFORMATETC)
+    wxDECLARE_NO_COPY_CLASS(wxIEnumFORMATETC);
 };
 
 // ----------------------------------------------------------------------------
@@ -110,8 +108,6 @@ public:
     // when this object is deleted - setting this flag enables such logic
     void SetDeleteFlag() { m_mustDelete = true; }
 
-    DECLARE_IUNKNOWN_METHODS;
-
     // IDataObject
     STDMETHODIMP GetData(FORMATETC *pformatetcIn, STGMEDIUM *pmedium);
     STDMETHODIMP GetDataHere(FORMATETC *pformatetc, STGMEDIUM *pmedium);
@@ -123,12 +119,14 @@ public:
     STDMETHODIMP DUnadvise(DWORD dwConnection);
     STDMETHODIMP EnumDAdvise(IEnumSTATDATA **ppenumAdvise);
 
+    DECLARE_IUNKNOWN_METHODS;
+
 private:
     wxDataObject *m_pDataObject;      // pointer to C++ class we belong to
 
     bool m_mustDelete;
 
-    DECLARE_NO_COPY_CLASS(wxIDataObject)
+    wxDECLARE_NO_COPY_CLASS(wxIDataObject);
 };
 
 // ============================================================================
@@ -139,9 +137,9 @@ private:
 // wxDataFormat
 // ----------------------------------------------------------------------------
 
-void wxDataFormat::SetId(const wxChar *format)
+void wxDataFormat::SetId(const wxString& format)
 {
-    m_format = (wxDataFormat::NativeFormat)::RegisterClipboardFormat(format);
+    m_format = (wxDataFormat::NativeFormat)::RegisterClipboardFormat(format.wx_str());
     if ( !m_format )
     {
         wxLogError(_("Couldn't register clipboard format '%s'."), format);
@@ -162,6 +160,7 @@ wxString wxDataFormat::GetId() const
     if ( !len )
     {
         wxLogError(_("The clipboard format '%d' doesn't exist."), m_format);
+        return wxEmptyString;
     }
 
     return s;
@@ -323,13 +322,8 @@ STDMETHODIMP wxIDataObject::GetData(FORMATETC *pformatetcIn, STGMEDIUM *pmedium)
                 return DV_E_FORMATETC;
             }
 
-            if ( !format.IsStandard() ) {
-                // for custom formats, put the size with the data - alloc the
-                // space for it
-                // MB: not completely sure this is correct,
-                //     even if I can't figure out what's wrong
-                size += m_pDataObject->GetBufferOffset( format );
-            }
+            // we may need extra space for the buffer size
+            size += m_pDataObject->GetBufferOffset( format );
 
             HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE | GMEM_SHARE, size);
             if ( hGlobal == NULL ) {
@@ -391,10 +385,14 @@ STDMETHODIMP wxIDataObject::GetDataHere(FORMATETC *pformatetc,
                 }
 
                 wxDataFormat format = pformatetc->cfFormat;
-                if ( !format.IsStandard() ) {
-                    // for custom formats, put the size with the data
-                    pBuf = m_pDataObject->SetSizeInBuffer( pBuf, GlobalSize(hGlobal), format );
-                }
+
+                // possibly put the size in the beginning of the buffer
+                pBuf = m_pDataObject->SetSizeInBuffer
+                                      (
+                                        pBuf,
+                                        ::GlobalSize(hGlobal),
+                                        format
+                                      );
 
                 if ( !m_pDataObject->GetDataHere(format, pBuf) )
                     return E_UNEXPECTED;
@@ -496,14 +494,9 @@ STDMETHODIMP wxIDataObject::SetData(FORMATETC *pformatetc,
                         break;
 #endif
                     default:
-                        {
-                            // we suppose that the size precedes the data
-                            pBuf = m_pDataObject->GetSizeFromBuffer( pBuf, &size, format );
-                            if (! format.IsStandard() ) {
-                                // see GetData for corresponding increment
-                                size -= m_pDataObject->GetBufferOffset( format  );
-                            }
-                        }
+                        pBuf = m_pDataObject->
+                                    GetSizeFromBuffer(pBuf, &size, format);
+                        size -= m_pDataObject->GetBufferOffset(format);
                 }
 
                 bool ok = m_pDataObject->SetData(format, size, pBuf);
@@ -624,7 +617,7 @@ STDMETHODIMP wxIDataObject::EnumFormatEtc(DWORD dwDir,
     wxDataObject::Direction dir = dwDir == DATADIR_GET ? wxDataObject::Get
                                                        : wxDataObject::Set;
 
-    size_t nFormatCount = m_pDataObject->GetFormatCount(dir);
+    ULONG nFormatCount = wx_truncate_cast(ULONG, m_pDataObject->GetFormatCount(dir));
     wxDataFormat format;
     wxDataFormat *formats;
     formats = nFormatCount == 1 ? &format : new wxDataFormat[nFormatCount];
@@ -687,30 +680,46 @@ void wxDataObject::SetAutoDelete()
     m_pIDataObject = NULL;
 }
 
-size_t wxDataObject::GetBufferOffset( const wxDataFormat& WXUNUSED(format) )
+size_t wxDataObject::GetBufferOffset(const wxDataFormat& format )
 {
-    return sizeof(size_t);
+    // if we prepend the size of the data to the buffer itself, account for it
+    return NeedsVerbatimData(format) ? 0 : sizeof(size_t);
 }
 
-const void* wxDataObject::GetSizeFromBuffer( const void* buffer, size_t* size,
-                                             const wxDataFormat& WXUNUSED(format) )
+const void *wxDataObject::GetSizeFromBuffer(const void *buffer,
+                                            size_t *size,
+                                            const wxDataFormat& WXUNUSED(format))
 {
-    size_t* p = (size_t*)buffer;
-    *size = *p;
+    // hack: the third parameter is declared non-const in Wine's headers so
+    // cast away the const
+    const size_t realsz = ::HeapSize(::GetProcessHeap(), 0,
+                                     const_cast<void*>(buffer));
+    if ( realsz == (size_t)-1 )
+    {
+        // note that HeapSize() does not set last error
+        wxLogApiError(wxT("HeapSize"), 0);
+        return NULL;
+    }
 
-    return p + 1;
+    *size = realsz;
+
+    return buffer;
 }
 
 void* wxDataObject::SetSizeInBuffer( void* buffer, size_t size,
-                                       const wxDataFormat& WXUNUSED(format) )
+                                      const wxDataFormat& format )
 {
-    size_t* p = (size_t*)buffer;
-    *p = size;
+    size_t* p = (size_t *)buffer;
+    if ( !NeedsVerbatimData(format) )
+    {
+        // prepend the size to the data and skip it
+        *p++ = size;
+    }
 
-    return p + 1;
+    return p;
 }
 
-#ifdef __WXDEBUG__
+#if wxDEBUG_LEVEL
 
 const wxChar *wxDataObject::GetFormatName(wxDataFormat format)
 {
@@ -755,7 +764,7 @@ const wxChar *wxDataObject::GetFormatName(wxDataFormat format)
     #endif // VC++
 }
 
-#endif // Debug
+#endif // wxDEBUG_LEVEL
 
 // ----------------------------------------------------------------------------
 // wxBitmapDataObject supports CF_DIB format
@@ -1031,22 +1040,44 @@ void wxFileDataObject::AddFile(const wxString& file)
 size_t wxFileDataObject::GetDataSize() const
 {
 #ifndef __WXWINCE__
-    // size returned will be the size of the DROPFILES structure,
-    // plus the list of filesnames (null byte separated), plus
-    // a double null at the end
+    // size returned will be the size of the DROPFILES structure, plus the list
+    // of filesnames (null byte separated), plus a double null at the end
 
     // if no filenames in list, size is 0
-    if ( m_filenames.GetCount() == 0 )
+    if ( m_filenames.empty() )
         return 0;
 
-    // inital size of DROPFILES struct + null byte
-    size_t sz = sizeof(DROPFILES) + (1 * sizeof(wxChar));
+#if wxUSE_UNICODE_MSLU
+    size_t sizeOfChar;
+    if ( wxGetOsVersion() == wxOS_WINDOWS_9X )
+    {
+        // Win9x always uses ANSI file names and MSLU doesn't help with this
+        sizeOfChar = sizeof(char);
+    }
+    else
+    {
+        sizeOfChar = sizeof(wxChar);
+    }
+#else // !wxUSE_UNICODE_MSLU
+    static const size_t sizeOfChar = sizeof(wxChar);
+#endif // wxUSE_UNICODE_MSLU/!wxUSE_UNICODE_MSLU
 
-    size_t count = m_filenames.GetCount();
+    // inital size of DROPFILES struct + null byte
+    size_t sz = sizeof(DROPFILES) + sizeOfChar;
+
+    const size_t count = m_filenames.size();
     for ( size_t i = 0; i < count; i++ )
     {
         // add filename length plus null byte
-        sz += (m_filenames[i].Len() + 1) * sizeof(wxChar);
+        size_t len;
+#if wxUSE_UNICODE_MSLU
+        if ( sizeOfChar == sizeof(char) )
+            len = strlen(m_filenames[i].mb_str(*wxConvFileName));
+        else
+#endif // wxUSE_UNICODE_MSLU
+            len = m_filenames[i].length();
+
+        sz += (len + 1) * sizeOfChar;
     }
 
     return sz;
@@ -1062,7 +1093,7 @@ bool wxFileDataObject::GetDataHere(void *WXUNUSED_IN_WINCE(pData)) const
     // created using the size returned by GetDataSize()
 
     // if pData is NULL, or there are no files, return
-    if ( !pData || m_filenames.GetCount() == 0 )
+    if ( !pData || m_filenames.empty() )
         return false;
 
     // convert data pointer to a DROPFILES struct pointer
@@ -1071,27 +1102,44 @@ bool wxFileDataObject::GetDataHere(void *WXUNUSED_IN_WINCE(pData)) const
     // initialize DROPFILES struct
     pDrop->pFiles = sizeof(DROPFILES);
     pDrop->fNC = FALSE;                 // not non-client coords
-#if wxUSE_UNICODE
-    pDrop->fWide = TRUE;
-#else // ANSI
-    pDrop->fWide = FALSE;
-#endif // Unicode/Ansi
+#if wxUSE_UNICODE_MSLU
+    pDrop->fWide = wxGetOsVersion() != wxOS_WINDOWS_9X ? TRUE : FALSE;
+#else
+    pDrop->fWide = wxUSE_UNICODE;
+#endif
+
+    const size_t sizeOfChar = pDrop->fWide ? sizeof(wchar_t) : sizeof(char);
 
     // set start of filenames list (null separated)
-    wxChar *pbuf = (wxChar*) ((BYTE *)pDrop + sizeof(DROPFILES));
+    BYTE *pbuf = (BYTE *)(pDrop + 1);
 
-    size_t count = m_filenames.GetCount();
-    for (size_t i = 0; i < count; i++ )
+    const size_t count = m_filenames.size();
+    for ( size_t i = 0; i < count; i++ )
     {
         // copy filename to pbuf and add null terminator
-        size_t len = m_filenames[i].Len();
-        memcpy(pbuf, m_filenames[i], len*sizeof(wxChar));
-        pbuf += len;
-        *pbuf++ = wxT('\0');
+        size_t len;
+#if wxUSE_UNICODE_MSLU
+        if ( sizeOfChar == sizeof(char) )
+        {
+            wxCharBuffer buf(m_filenames[i].mb_str(*wxConvFileName));
+            len = strlen(buf);
+            memcpy(pbuf, buf, len*sizeOfChar);
+        }
+        else
+#endif // wxUSE_UNICODE_MSLU
+        {
+            len = m_filenames[i].length();
+            memcpy(pbuf, m_filenames[i].wx_str(), len*sizeOfChar);
+        }
+
+        pbuf += len*sizeOfChar;
+
+        memset(pbuf, 0, sizeOfChar);
+        pbuf += sizeOfChar;
     }
 
     // add final null terminator
-    *pbuf = wxT('\0');
+    memset(pbuf, 0, sizeOfChar);
 
     return true;
 #else
@@ -1103,11 +1151,17 @@ bool wxFileDataObject::GetDataHere(void *WXUNUSED_IN_WINCE(pData)) const
 // wxURLDataObject
 // ----------------------------------------------------------------------------
 
+// Work around bug in Wine headers
+#if defined(__WINE__) && defined(CFSTR_SHELLURL) && wxUSE_UNICODE
+#undef CFSTR_SHELLURL
+#define CFSTR_SHELLURL wxT("CFSTR_SHELLURL")
+#endif
+
 class CFSTR_SHELLURLDataObject : public wxCustomDataObject
 {
 public:
     CFSTR_SHELLURLDataObject() : wxCustomDataObject(CFSTR_SHELLURL) {}
-protected:
+
     virtual size_t GetBufferOffset( const wxDataFormat& WXUNUSED(format) )
     {
         return 0;
@@ -1128,38 +1182,24 @@ protected:
         return buffer;
     }
 
-#if wxUSE_UNICODE
-    virtual bool GetDataHere( void* buffer ) const
-    {
-        // CFSTR_SHELLURL is _always_ ANSI!
-        wxCharBuffer char_buffer( GetDataSize() );
-        wxCustomDataObject::GetDataHere( (void*)char_buffer.data() );
-        wxString unicode_buffer( char_buffer, wxConvLibc );
-        memcpy( buffer, unicode_buffer.c_str(),
-                ( unicode_buffer.length() + 1 ) * sizeof(wxChar) );
-
-        return true;
-    }
-    virtual bool GetDataHere(const wxDataFormat& WXUNUSED(format),
-                             void *buf) const
-        { return GetDataHere(buf); }
-#endif
-
-    DECLARE_NO_COPY_CLASS(CFSTR_SHELLURLDataObject)
+    wxDECLARE_NO_COPY_CLASS(CFSTR_SHELLURLDataObject);
 };
 
 
 
-wxURLDataObject::wxURLDataObject()
+wxURLDataObject::wxURLDataObject(const wxString& url)
 {
-    // we support CF_TEXT and CFSTR_SHELLURL formats which are basicly the same
-    // but it seems that some browsers only provide one of them so we have to
-    // support both
+    // we support CF_TEXT and CFSTR_SHELLURL formats which are basically the
+    // same but it seems that some browsers only provide one of them so we have
+    // to support both
     Add(new wxTextDataObject);
     Add(new CFSTR_SHELLURLDataObject());
 
     // we don't have any data yet
     m_dataObjectLast = NULL;
+
+    if ( !url.empty() )
+        SetURL(url);
 }
 
 bool wxURLDataObject::SetData(const wxDataFormat& format,
@@ -1177,31 +1217,63 @@ bool wxURLDataObject::SetData(const wxDataFormat& format,
 wxString wxURLDataObject::GetURL() const
 {
     wxString url;
-    wxCHECK_MSG( m_dataObjectLast, url, _T("no data in wxURLDataObject") );
+    wxCHECK_MSG( m_dataObjectLast, url, wxT("no data in wxURLDataObject") );
 
-    size_t len = m_dataObjectLast->GetDataSize();
+    if ( m_dataObjectLast->GetPreferredFormat() == CFSTR_SHELLURL )
+    {
+        const size_t len = m_dataObjectLast->GetDataSize();
+        if ( !len )
+            return wxString();
 
-    m_dataObjectLast->GetDataHere(wxStringBuffer(url, len));
+        // CFSTR_SHELLURL is always ANSI so we need to convert it from it in
+        // Unicode build
+#if wxUSE_UNICODE
+        wxCharBuffer buf(len);
+
+        if ( m_dataObjectLast->GetDataHere(buf.data()) )
+            url = buf;
+#else // !wxUSE_UNICODE
+        // in ANSI build no conversion is necessary
+        m_dataObjectLast->GetDataHere(wxStringBuffer(url, len));
+#endif // wxUSE_UNICODE/!wxUSE_UNICODE
+    }
+    else // must be wxTextDataObject
+    {
+        url = static_cast<wxTextDataObject *>(m_dataObjectLast)->GetText();
+    }
 
     return url;
 }
 
 void wxURLDataObject::SetURL(const wxString& url)
 {
-    SetData(wxDataFormat(wxUSE_UNICODE ? wxDF_UNICODETEXT : wxDF_TEXT),
-            url.Length()+1, url.c_str());
+    wxCharBuffer urlMB(url.mb_str());
+    if ( urlMB )
+    {
+        const size_t len = strlen(urlMB);
 
-    // CFSTR_SHELLURL is always supposed to be ANSI...
-    wxWX2MBbuf urlA = (wxWX2MBbuf)url.mbc_str();
-    size_t len = strlen(urlA);
-    SetData(wxDataFormat(CFSTR_SHELLURL), len+1, (const char*)urlA);
+#if !wxUSE_UNICODE
+        // wxTextDataObject takes the number of characters in the string, not
+        // the size of the buffer (which would be len+1)
+        SetData(wxDF_TEXT, len, urlMB);
+#endif // !wxUSE_UNICODE
+
+        // however CFSTR_SHELLURLDataObject doesn't append NUL automatically
+        // but we probably still want to have it on the clipboard (just to be
+        // safe), so do append it
+        SetData(wxDataFormat(CFSTR_SHELLURL), len + 1, urlMB);
+    }
+
+#if wxUSE_UNICODE
+    SetData(wxDF_UNICODETEXT, url.length()*sizeof(wxChar), url.wc_str());
+#endif
 }
 
 // ----------------------------------------------------------------------------
 // private functions
 // ----------------------------------------------------------------------------
 
-#ifdef __WXDEBUG__
+#if wxDEBUG_LEVEL
 
 static const wxChar *GetTymedName(DWORD tymed)
 {
@@ -1242,12 +1314,10 @@ void wxDataObject::SetAutoDelete()
 {
 }
 
-#ifdef __WXDEBUG__
 const wxChar *wxDataObject::GetFormatName(wxDataFormat WXUNUSED(format))
 {
     return NULL;
 }
-#endif // __WXDEBUG__
 
 #endif // wxUSE_DATAOBJ
 
