@@ -1,10 +1,10 @@
 ///////////////////////////////////////////////////////////////////////////////
-// Name:        msw/registry.cpp
+// Name:        src/msw/registry.cpp
 // Purpose:     implementation of registry classes and functions
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     03.04.98
-// RCS-ID:      $Id: registry.cpp,v 1.77 2005/07/22 17:13:46 ABX Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1998 Vadim Zeitlin <zeitlin@dptmaths.ens-cachan.fr>
 // Licence:     wxWindows licence
 // TODO:        - parsing of registry key names
@@ -12,27 +12,27 @@
 //              - add high level functions (RegisterOleServer, ...)
 ///////////////////////////////////////////////////////////////////////////////
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "registry.h"
-#endif
-
 // for compilers that support precompilation, includes "wx.h".
 #include  "wx/wxprec.h"
 
 #ifdef __BORLANDC__
-#pragma hdrstop
+    #pragma hdrstop
 #endif
 
-// other wxWidgets headers
-#include  "wx/string.h"
-#include  "wx/intl.h"
-#include  "wx/log.h"
-#include  "wx/file.h"
-#include  "wx/wfstream.h"
+#if wxUSE_REGKEY
+
+#ifndef WX_PRECOMP
+    #include "wx/msw/wrapwin.h"
+    #include "wx/string.h"
+    #include "wx/intl.h"
+    #include "wx/log.h"
+    #include "wx/crt.h"
+#endif
+
+#include "wx/file.h"
+#include "wx/wfstream.h"
 
 // Windows headers
-#include  "wx/msw/wrapwin.h"
-
 #ifdef __WXWINCE__
 #include "wx/msw/private.h"
 #include <winbase.h>
@@ -54,6 +54,18 @@
 typedef unsigned char *RegString;
 typedef BYTE* RegBinary;
 
+#ifndef HKEY_PERFORMANCE_DATA
+    #define HKEY_PERFORMANCE_DATA ((HKEY)0x80000004)
+#endif
+
+#ifndef HKEY_CURRENT_CONFIG
+    #define HKEY_CURRENT_CONFIG ((HKEY)0x80000005)
+#endif
+
+#ifndef HKEY_DYN_DATA
+    #define HKEY_DYN_DATA ((HKEY)0x80000006)
+#endif
+
 // ----------------------------------------------------------------------------
 // constants
 // ----------------------------------------------------------------------------
@@ -72,15 +84,9 @@ aStdKeys[] =
   { HKEY_CURRENT_USER,      wxT("HKEY_CURRENT_USER"),      wxT("HKCU") },
   { HKEY_LOCAL_MACHINE,     wxT("HKEY_LOCAL_MACHINE"),     wxT("HKLM") },
   { HKEY_USERS,             wxT("HKEY_USERS"),             wxT("HKU")  }, // short name?
-#ifndef __WXWINCE__
   { HKEY_PERFORMANCE_DATA,  wxT("HKEY_PERFORMANCE_DATA"),  wxT("HKPD") },
-#endif
-#ifdef HKEY_CURRENT_CONFIG
   { HKEY_CURRENT_CONFIG,    wxT("HKEY_CURRENT_CONFIG"),    wxT("HKCC") },
-#endif
-#ifdef HKEY_DYN_DATA
   { HKEY_DYN_DATA,          wxT("HKEY_DYN_DATA"),          wxT("HKDD") }, // short name?
-#endif
 };
 
 // the registry name separator (perhaps one day MS will change it to '/' ;-)
@@ -108,11 +114,16 @@ aStdKeys[] =
 static inline void RemoveTrailingSeparator(wxString& str);
 
 // returns true if given registry key exists
-static bool KeyExists(WXHKEY hRootKey, const wxChar *szKey);
+static bool KeyExists(WXHKEY hRootKey, const wxString& szKey);
 
-// combines value and key name (uses static buffer!)
-static const wxChar *GetFullName(const wxRegKey *pKey,
-                               const wxChar *szValue = NULL);
+// combines value and key name
+static wxString GetFullName(const wxRegKey *pKey);
+static wxString GetFullName(const wxRegKey *pKey, const wxString& szValue);
+
+// returns "value" argument of wxRegKey methods converted into a value that can
+// be passed to win32 registry functions; specifically, converts empty string
+// to NULL
+static inline const wxChar *RegValueStr(const wxString& szValue);
 
 // ============================================================================
 // implementation of wxRegKey class
@@ -146,12 +157,10 @@ wxRegKey::StdKey wxRegKey::ExtractKeyName(wxString& strKey)
 {
   wxString strRoot = strKey.BeforeFirst(REG_SEPARATOR);
 
-  HKEY hRootKey = 0;
   size_t ui;
   for ( ui = 0; ui < nStdKeys; ui++ ) {
     if ( strRoot.CmpNoCase(aStdKeys[ui].szName) == 0 ||
          strRoot.CmpNoCase(aStdKeys[ui].szShortName) == 0 ) {
-      hRootKey = aStdKeys[ui].hkey;
       break;
     }
   }
@@ -159,7 +168,7 @@ wxRegKey::StdKey wxRegKey::ExtractKeyName(wxString& strKey)
   if ( ui == nStdKeys ) {
     wxFAIL_MSG(wxT("invalid key prefix in wxRegKey::ExtractKeyName."));
 
-    hRootKey = HKEY_CLASSES_ROOT;
+    ui = HKCR;
   }
   else {
     strKey = strKey.After(REG_SEPARATOR);
@@ -167,13 +176,13 @@ wxRegKey::StdKey wxRegKey::ExtractKeyName(wxString& strKey)
       strKey.Truncate(strKey.Len() - 1);
   }
 
-  return (wxRegKey::StdKey)(int)hRootKey;
+  return (StdKey)ui;
 }
 
 wxRegKey::StdKey wxRegKey::GetStdKeyFromHkey(WXHKEY hkey)
 {
   for ( size_t ui = 0; ui < nStdKeys; ui++ ) {
-    if ( (int) aStdKeys[ui].hkey == (int) hkey )
+    if ( aStdKeys[ui].hkey == (HKEY)hkey )
       return (StdKey)ui;
   }
 
@@ -285,6 +294,18 @@ void wxRegKey::SetHkey(WXHKEY hKey)
   Close();
 
   m_hKey = hKey;
+
+  // we don't know the parent of this key, assume HKLM by default
+  m_hRootKey = HKEY_LOCAL_MACHINE;
+
+  // we don't know in which mode was this key opened but we can't reopen it
+  // anyhow because we don't know its name, so the only thing we can is to hope
+  // that it allows all the operations which we're going to perform on it
+  m_mode = Write;
+
+  // reset old data
+  m_strKey.empty();
+  m_dwLastError = 0;
 }
 
 // ----------------------------------------------------------------------------
@@ -295,7 +316,7 @@ void wxRegKey::SetHkey(WXHKEY hKey)
 bool wxRegKey::Exists() const
 {
   // opened key has to exist, try to open it if not done yet
-  return IsOpened() ? true : KeyExists(m_hRootKey, m_strKey);
+  return IsOpened() ? true : KeyExists(m_hRootKey, m_strKey.wx_str());
 }
 
 // returns the full name of the key (prefix is abbreviated if bShortPrefix)
@@ -305,7 +326,7 @@ wxString wxRegKey::GetName(bool bShortPrefix) const
   wxString str = bShortPrefix ? aStdKeys[key].szShortName
                               : aStdKeys[key].szName;
   if ( !m_strKey.empty() )
-    str << _T("\\") << m_strKey;
+    str << wxT("\\") << m_strKey;
 
   return str;
 }
@@ -323,7 +344,7 @@ bool wxRegKey::GetKeyInfo(size_t *pnSubKeys,
 #endif
 
   // it might be unexpected to some that this function doesn't open the key
-  wxASSERT_MSG( IsOpened(), _T("key should be opened in GetKeyInfo") );
+  wxASSERT_MSG( IsOpened(), wxT("key should be opened in GetKeyInfo") );
 
   m_dwLastError = ::RegQueryInfoKey
                   (
@@ -376,7 +397,7 @@ bool wxRegKey::Open(AccessMode mode)
     m_dwLastError = ::RegOpenKeyEx
                     (
                         (HKEY) m_hRootKey,
-                        m_strKey,
+                        m_strKey.t_str(),
                         RESERVED,
                         mode == Read ? KEY_READ : KEY_ALL_ACCESS,
                         &tmpKey
@@ -408,7 +429,7 @@ bool wxRegKey::Create(bool bOkIfExists)
   HKEY tmpKey;
 #ifdef __WXWINCE__
   DWORD disposition;
-  m_dwLastError = RegCreateKeyEx((HKEY) m_hRootKey, m_strKey,
+  m_dwLastError = RegCreateKeyEx((HKEY) m_hRootKey, m_strKey.wx_str(),
       NULL, // reserved
       NULL, // class string
       0,
@@ -417,7 +438,7 @@ bool wxRegKey::Create(bool bOkIfExists)
       &tmpKey,
       &disposition);
 #else
-  m_dwLastError = RegCreateKey((HKEY) m_hRootKey, m_strKey, &tmpKey);
+  m_dwLastError = RegCreateKey((HKEY) m_hRootKey, m_strKey.t_str(), &tmpKey);
 #endif
   if ( m_dwLastError != ERROR_SUCCESS ) {
     wxLogSysError(m_dwLastError, _("Can't create registry key '%s'"),
@@ -449,7 +470,8 @@ bool wxRegKey::Close()
   return true;
 }
 
-bool wxRegKey::RenameValue(const wxChar *szValueOld, const wxChar *szValueNew)
+bool
+wxRegKey::RenameValue(const wxString& szValueOld, const wxString& szValueNew)
 {
     bool ok = true;
     if ( HasValue(szValueNew) ) {
@@ -470,13 +492,14 @@ bool wxRegKey::RenameValue(const wxChar *szValueOld, const wxChar *szValueNew)
     return true;
 }
 
-bool wxRegKey::CopyValue(const wxChar *szValue,
+bool wxRegKey::CopyValue(const wxString& szValue,
                          wxRegKey& keyDst,
-                         const wxChar *szValueNew)
+                         const wxString& szValueNew)
 {
-    if ( !szValueNew ) {
+    wxString valueNew(szValueNew);
+    if ( valueNew.empty() ) {
         // by default, use the same name
-        szValueNew = szValue;
+        valueNew = szValue;
     }
 
     switch ( GetValueType(szValue) ) {
@@ -484,7 +507,7 @@ bool wxRegKey::CopyValue(const wxChar *szValue,
             {
                 wxString strVal;
                 return QueryValue(szValue, strVal) &&
-                       keyDst.SetValue(szValueNew, strVal);
+                       keyDst.SetValue(valueNew, strVal);
             }
 
         case Type_Dword:
@@ -492,14 +515,14 @@ bool wxRegKey::CopyValue(const wxChar *szValue,
             {
                 long dwVal;
                 return QueryValue(szValue, &dwVal) &&
-                       keyDst.SetValue(szValueNew, dwVal);
+                       keyDst.SetValue(valueNew, dwVal);
             }
 
         case Type_Binary:
         {
             wxMemoryBuffer buf;
             return QueryValue(szValue,buf) &&
-                   keyDst.SetValue(szValueNew,buf);
+                   keyDst.SetValue(valueNew,buf);
         }
 
         // these types are unsupported because I am not sure about how
@@ -521,9 +544,9 @@ bool wxRegKey::CopyValue(const wxChar *szValue,
     }
 }
 
-bool wxRegKey::Rename(const wxChar *szNewName)
+bool wxRegKey::Rename(const wxString& szNewName)
 {
-    wxCHECK_MSG( !m_strKey.empty(), false, _T("registry hives can't be renamed") );
+    wxCHECK_MSG( !m_strKey.empty(), false, wxT("registry hives can't be renamed") );
 
     if ( !Exists() ) {
         wxLogError(_("Registry key '%s' does not exist, cannot rename it."),
@@ -576,7 +599,7 @@ bool wxRegKey::Rename(const wxChar *szNewName)
     return ok;
 }
 
-bool wxRegKey::Copy(const wxChar *szNewName)
+bool wxRegKey::Copy(const wxString& szNewName)
 {
     // create the new key first
     wxRegKey keyDst(szNewName);
@@ -605,7 +628,7 @@ bool wxRegKey::Copy(wxRegKey& keyDst)
         wxRegKey key(*this, strKey);
         wxString keyName;
         keyName << GetFullName(&keyDst) << REG_SEPARATOR << strKey;
-        ok = key.Copy((const wxChar*) keyName);
+        ok = key.Copy(keyName);
 
         if ( ok )
             bCont = GetNextKey(strKey, lIndex);
@@ -687,7 +710,7 @@ bool wxRegKey::DeleteSelf()
   // now delete this key itself
   Close();
 
-  m_dwLastError = RegDeleteKey((HKEY) m_hRootKey, m_strKey);
+  m_dwLastError = RegDeleteKey((HKEY) m_hRootKey, m_strKey.t_str());
   // deleting a key which doesn't exist is not considered an error
   if ( m_dwLastError != ERROR_SUCCESS &&
           m_dwLastError != ERROR_FILE_NOT_FOUND ) {
@@ -699,7 +722,7 @@ bool wxRegKey::DeleteSelf()
   return true;
 }
 
-bool wxRegKey::DeleteKey(const wxChar *szKey)
+bool wxRegKey::DeleteKey(const wxString& szKey)
 {
   if ( !Open() )
     return false;
@@ -708,12 +731,12 @@ bool wxRegKey::DeleteKey(const wxChar *szKey)
   return key.DeleteSelf();
 }
 
-bool wxRegKey::DeleteValue(const wxChar *szValue)
+bool wxRegKey::DeleteValue(const wxString& szValue)
 {
     if ( !Open() )
         return false;
 
-    m_dwLastError = RegDeleteValue((HKEY) m_hKey, WXSTRINGCAST szValue);
+    m_dwLastError = RegDeleteValue((HKEY) m_hKey, RegValueStr(szValue));
 
     // deleting a value which doesn't exist is not considered an error
     if ( (m_dwLastError != ERROR_SUCCESS) &&
@@ -732,7 +755,7 @@ bool wxRegKey::DeleteValue(const wxChar *szValue)
 // ----------------------------------------------------------------------------
 
 // return true if value exists
-bool wxRegKey::HasValue(const wxChar *szValue) const
+bool wxRegKey::HasValue(const wxString& szValue) const
 {
     // this function should be silent, so suppress possible messages from Open()
     wxLogNull nolog;
@@ -741,7 +764,7 @@ bool wxRegKey::HasValue(const wxChar *szValue) const
         return false;
 
     LONG dwRet = ::RegQueryValueEx((HKEY) m_hKey,
-                                   WXSTRINGCAST szValue,
+                                   RegValueStr(szValue),
                                    RESERVED,
                                    NULL, NULL, NULL);
     return dwRet == ERROR_SUCCESS;
@@ -772,7 +795,7 @@ bool wxRegKey::HasSubkeys() const
 }
 
 // returns true if given subkey exists
-bool wxRegKey::HasSubKey(const wxChar *szKey) const
+bool wxRegKey::HasSubKey(const wxString& szKey) const
 {
   // this function should be silent, so suppress possible messages from Open()
   wxLogNull nolog;
@@ -783,13 +806,13 @@ bool wxRegKey::HasSubKey(const wxChar *szKey) const
   return KeyExists(m_hKey, szKey);
 }
 
-wxRegKey::ValueType wxRegKey::GetValueType(const wxChar *szValue) const
+wxRegKey::ValueType wxRegKey::GetValueType(const wxString& szValue) const
 {
     if ( ! CONST_CAST Open(Read) )
       return Type_None;
 
     DWORD dwType;
-    m_dwLastError = RegQueryValueEx((HKEY) m_hKey, WXSTRINGCAST szValue, RESERVED,
+    m_dwLastError = RegQueryValueEx((HKEY) m_hKey, RegValueStr(szValue), RESERVED,
                                     &dwType, NULL, NULL);
     if ( m_dwLastError != ERROR_SUCCESS ) {
       wxLogSysError(m_dwLastError, _("Can't read value of key '%s'"),
@@ -800,10 +823,11 @@ wxRegKey::ValueType wxRegKey::GetValueType(const wxChar *szValue) const
     return (ValueType)dwType;
 }
 
-bool wxRegKey::SetValue(const wxChar *szValue, long lValue)
+bool wxRegKey::SetValue(const wxString& szValue, long lValue)
 {
   if ( CONST_CAST Open() ) {
-    m_dwLastError = RegSetValueEx((HKEY) m_hKey, szValue, (DWORD) RESERVED, REG_DWORD,
+    m_dwLastError = RegSetValueEx((HKEY) m_hKey, RegValueStr(szValue),
+                                  (DWORD) RESERVED, REG_DWORD,
                                   (RegString)&lValue, sizeof(lValue));
     if ( m_dwLastError == ERROR_SUCCESS )
       return true;
@@ -814,12 +838,13 @@ bool wxRegKey::SetValue(const wxChar *szValue, long lValue)
   return false;
 }
 
-bool wxRegKey::QueryValue(const wxChar *szValue, long *plValue) const
+bool wxRegKey::QueryValue(const wxString& szValue, long *plValue) const
 {
   if ( CONST_CAST Open(Read) ) {
     DWORD dwType, dwSize = sizeof(DWORD);
     RegString pBuf = (RegString)plValue;
-    m_dwLastError = RegQueryValueEx((HKEY) m_hKey, WXSTRINGCAST szValue, RESERVED,
+    m_dwLastError = RegQueryValueEx((HKEY) m_hKey, RegValueStr(szValue),
+                                    RESERVED,
                                     &dwType, pBuf, &dwSize);
     if ( m_dwLastError != ERROR_SUCCESS ) {
       wxLogSysError(m_dwLastError, _("Can't read value of key '%s'"),
@@ -838,14 +863,15 @@ bool wxRegKey::QueryValue(const wxChar *szValue, long *plValue) const
     return false;
 }
 
-bool wxRegKey::SetValue(const wxChar *szValue,const wxMemoryBuffer& buffer)
+bool wxRegKey::SetValue(const wxString& szValue, const wxMemoryBuffer& buffer)
 {
 #ifdef __TWIN32__
   wxFAIL_MSG("RegSetValueEx not implemented by TWIN32");
   return false;
 #else
   if ( CONST_CAST Open() ) {
-    m_dwLastError = RegSetValueEx((HKEY) m_hKey, szValue, (DWORD) RESERVED, REG_BINARY,
+    m_dwLastError = RegSetValueEx((HKEY) m_hKey, RegValueStr(szValue),
+                                  (DWORD) RESERVED, REG_BINARY,
                                   (RegBinary)buffer.GetData(),buffer.GetDataLen());
     if ( m_dwLastError == ERROR_SUCCESS )
       return true;
@@ -857,19 +883,20 @@ bool wxRegKey::SetValue(const wxChar *szValue,const wxMemoryBuffer& buffer)
 #endif
 }
 
-bool wxRegKey::QueryValue(const wxChar *szValue, wxMemoryBuffer& buffer) const
+bool wxRegKey::QueryValue(const wxString& szValue, wxMemoryBuffer& buffer) const
 {
   if ( CONST_CAST Open(Read) ) {
     // first get the type and size of the data
     DWORD dwType, dwSize;
-    m_dwLastError = RegQueryValueEx((HKEY) m_hKey, WXSTRINGCAST szValue, RESERVED,
-                                      &dwType, NULL, &dwSize);
+    m_dwLastError = RegQueryValueEx((HKEY) m_hKey, RegValueStr(szValue),
+                                    RESERVED,
+                                    &dwType, NULL, &dwSize);
 
     if ( m_dwLastError == ERROR_SUCCESS ) {
         if ( dwSize ) {
             const RegBinary pBuf = (RegBinary)buffer.GetWriteBuf(dwSize);
             m_dwLastError = RegQueryValueEx((HKEY) m_hKey,
-                                            WXSTRINGCAST szValue,
+                                            RegValueStr(szValue),
                                             RESERVED,
                                             &dwType,
                                             pBuf,
@@ -893,7 +920,7 @@ bool wxRegKey::QueryValue(const wxChar *szValue, wxMemoryBuffer& buffer) const
 
 
 
-bool wxRegKey::QueryValue(const wxChar *szValue,
+bool wxRegKey::QueryValue(const wxString& szValue,
                           wxString& strValue,
                           bool WXUNUSED_IN_WINCE(raw)) const
 {
@@ -901,8 +928,10 @@ bool wxRegKey::QueryValue(const wxChar *szValue,
     {
 
         // first get the type and size of the data
-        DWORD dwType, dwSize;
-        m_dwLastError = RegQueryValueEx((HKEY) m_hKey, WXSTRINGCAST szValue, RESERVED,
+        DWORD dwType=REG_NONE, dwSize=0;
+        m_dwLastError = RegQueryValueEx((HKEY) m_hKey,
+                                        RegValueStr(szValue),
+                                        RESERVED,
                                         &dwType, NULL, &dwSize);
         if ( m_dwLastError == ERROR_SUCCESS )
         {
@@ -915,7 +944,7 @@ bool wxRegKey::QueryValue(const wxChar *szValue,
             else
             {
                 m_dwLastError = RegQueryValueEx((HKEY) m_hKey,
-                                                WXSTRINGCAST szValue,
+                                                RegValueStr(szValue),
                                                 RESERVED,
                                                 &dwType,
                                                 (RegString)(wxChar*)wxStringBuffer(strValue, dwSize),
@@ -925,12 +954,12 @@ bool wxRegKey::QueryValue(const wxChar *szValue,
 #ifndef __WXWINCE__
                 if ( (dwType == REG_EXPAND_SZ) && !raw )
                 {
-                    DWORD dwExpSize = ::ExpandEnvironmentStrings(strValue, NULL, 0);
+                    DWORD dwExpSize = ::ExpandEnvironmentStrings(strValue.t_str(), NULL, 0);
                     bool ok = dwExpSize != 0;
                     if ( ok )
                     {
                         wxString strExpValue;
-                        ok = ::ExpandEnvironmentStrings(strValue,
+                        ok = ::ExpandEnvironmentStrings(strValue.t_str(),
                                                         wxStringBuffer(strExpValue, dwExpSize),
                                                         dwExpSize
                                                         ) != 0;
@@ -939,7 +968,7 @@ bool wxRegKey::QueryValue(const wxChar *szValue,
 
                     if ( !ok )
                     {
-                        wxLogLastError(_T("ExpandEnvironmentStrings"));
+                        wxLogLastError(wxT("ExpandEnvironmentStrings"));
                     }
                 }
 #endif
@@ -962,11 +991,13 @@ bool wxRegKey::QueryValue(const wxChar *szValue,
     return false;
 }
 
-bool wxRegKey::SetValue(const wxChar *szValue, const wxString& strValue)
+bool wxRegKey::SetValue(const wxString& szValue, const wxString& strValue)
 {
   if ( CONST_CAST Open() ) {
-      m_dwLastError = RegSetValueEx((HKEY) m_hKey, szValue, (DWORD) RESERVED, REG_SZ,
-                                    (RegString)strValue.c_str(),
+      m_dwLastError = RegSetValueEx((HKEY) m_hKey,
+                                    RegValueStr(szValue),
+                                    (DWORD) RESERVED, REG_SZ,
+                                    (RegString)strValue.wx_str(),
                                     (strValue.Len() + 1)*sizeof(wxChar));
       if ( m_dwLastError == ERROR_SUCCESS )
         return true;
@@ -980,7 +1011,7 @@ bool wxRegKey::SetValue(const wxChar *szValue, const wxString& strValue)
 wxString wxRegKey::QueryDefaultValue() const
 {
   wxString str;
-  QueryValue(NULL, str);
+  QueryValue(wxEmptyString, str, false);
   return str;
 }
 
@@ -1080,7 +1111,7 @@ bool wxRegKey::GetNextKey(wxString& strKeyName, long& lIndex) const
 }
 
 // returns true if the value contains a number (else it's some string)
-bool wxRegKey::IsNumericValue(const wxChar *szValue) const
+bool wxRegKey::IsNumericValue(const wxString& szValue) const
 {
     ValueType type = GetValueType(szValue);
     switch ( type ) {
@@ -1124,7 +1155,7 @@ static inline bool WriteAsciiString(wxOutputStream& ostr, const wxString& s)
     wxCharBuffer name(s.mb_str());
     ostr.Write(name, strlen(name));
 #else
-    ostr.Write(s, s.length());
+    ostr.Write(s.mb_str(), s.length());
 #endif
 
     return ostr.IsOk();
@@ -1142,7 +1173,7 @@ bool wxRegKey::Export(const wxString& filename) const
         return false;
     }
 
-    wxFFileOutputStream ostr(filename, _T("w"));
+    wxFFileOutputStream ostr(filename, wxT("w"));
 
     return ostr.Ok() && Export(ostr);
 #else
@@ -1168,26 +1199,26 @@ FormatAsHex(const void *data,
             size_t size,
             wxRegKey::ValueType type = wxRegKey::Type_Binary)
 {
-    wxString value(_T("hex"));
+    wxString value(wxT("hex"));
 
     // binary values use just "hex:" prefix while the other ones must indicate
     // the real type
     if ( type != wxRegKey::Type_Binary )
-        value << _T('(') << type << _T(')');
-    value << _T(':');
+        value << wxT('(') << type << wxT(')');
+    value << wxT(':');
 
     // write all the rest as comma-separated bytes
     value.reserve(3*size + 10);
-    const char * const p = wx_static_cast(const char *, data);
+    const char * const p = static_cast<const char *>(data);
     for ( size_t n = 0; n < size; n++ )
     {
         // TODO: line wrapping: although not required by regedit, this makes
         //       the generated files easier to read and compare with the files
         //       produced by regedit
         if ( n )
-            value << _T(',');
+            value << wxT(',');
 
-        value << wxString::Format(_T("%02x"), (unsigned char)p[n]);
+        value << wxString::Format(wxT("%02x"), (unsigned char)p[n]);
     }
 
     return value;
@@ -1214,23 +1245,24 @@ wxString wxRegKey::FormatValue(const wxString& name) const
                 // quotes and backslashes must be quoted, linefeeds are not
                 // allowed in string values
                 rhs.reserve(value.length() + 2);
-                rhs = _T('"');
+                rhs = wxT('"');
 
                 // there can be no NULs here
                 bool useHex = false;
-                for ( const wxChar *p = value.c_str(); *p && !useHex; p++ )
+                for ( wxString::const_iterator p = value.begin();
+                      p != value.end() && !useHex; ++p )
                 {
-                    switch ( *p )
+                    switch ( (*p).GetValue() )
                     {
-                        case _T('\n'):
+                        case wxT('\n'):
                             // we can only represent this string in hex
                             useHex = true;
                             break;
 
-                        case _T('"'):
-                        case _T('\\'):
+                        case wxT('"'):
+                        case wxT('\\'):
                             // escape special symbol
-                            rhs += _T('\\');
+                            rhs += wxT('\\');
                             // fall through
 
                         default:
@@ -1241,7 +1273,7 @@ wxString wxRegKey::FormatValue(const wxString& name) const
                 if ( useHex )
                     rhs = FormatAsHex(value, Type_String);
                 else
-                    rhs += _T('"');
+                    rhs += wxT('"');
             }
             break;
 
@@ -1252,7 +1284,7 @@ wxString wxRegKey::FormatValue(const wxString& name) const
                 if ( !QueryValue(name, &value) )
                     break;
 
-                rhs.Printf(_T("dword:%08x"), (unsigned int)value);
+                rhs.Printf(wxT("dword:%08x"), (unsigned int)value);
             }
             break;
 
@@ -1344,7 +1376,7 @@ bool wxRegKey::DoExport(wxOutputStream& ostr) const
     // dump all our values
     long dummy;
     wxString name;
-    wxRegKey& self = wx_const_cast(wxRegKey&, *this);
+    wxRegKey& self = const_cast<wxRegKey&>(*this);
     bool cont = self.GetFirstValue(name, dummy);
     while ( cont )
     {
@@ -1378,17 +1410,17 @@ bool wxRegKey::DoExport(wxOutputStream& ostr) const
 // implementation of global private functions
 // ============================================================================
 
-bool KeyExists(WXHKEY hRootKey, const wxChar *szKey)
+bool KeyExists(WXHKEY hRootKey, const wxString& szKey)
 {
     // don't close this key itself for the case of empty szKey!
-    if ( wxIsEmpty(szKey) )
+    if ( szKey.empty() )
         return true;
 
     HKEY hkeyDummy;
     if ( ::RegOpenKeyEx
          (
             (HKEY)hRootKey,
-            szKey,
+            szKey.t_str(),
             RESERVED,
             KEY_READ,        // we might not have enough rights for rw access
             &hkeyDummy
@@ -1402,19 +1434,29 @@ bool KeyExists(WXHKEY hRootKey, const wxChar *szKey)
     return false;
 }
 
-const wxChar *GetFullName(const wxRegKey *pKey, const wxChar *szValue)
+wxString GetFullName(const wxRegKey *pKey, const wxString& szValue)
 {
-  static wxString s_str;
-  s_str = pKey->GetName();
-  if ( !wxIsEmpty(szValue) )
-    s_str << wxT("\\") << szValue;
+  wxString str(pKey->GetName());
+  if ( !szValue.empty() )
+    str << wxT("\\") << szValue;
 
-  return s_str.c_str();
+  return str;
 }
 
-void RemoveTrailingSeparator(wxString& str)
+wxString GetFullName(const wxRegKey *pKey)
+{
+  return pKey->GetName();
+}
+
+inline void RemoveTrailingSeparator(wxString& str)
 {
   if ( !str.empty() && str.Last() == REG_SEPARATOR )
     str.Truncate(str.Len() - 1);
 }
 
+inline const wxChar *RegValueStr(const wxString& szValue)
+{
+    return szValue.empty() ? (const wxChar*)NULL : szValue.t_str();
+}
+
+#endif // wxUSE_REGKEY
