@@ -4,7 +4,7 @@
 // Author:      David Webster
 // Modified by:
 // Created:     10/21/99
-// RCS-ID:      $Id: dataobj.cpp 40345 2006-07-27 12:57:16Z ABX $
+// RCS-ID:      $Id$
 // Copyright:   (c) 1999 David Webster
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -57,11 +57,11 @@ wxString wxDataFormat::GetId() const
 } // end of wxDataFormat::GetId()
 
 void wxDataFormat::SetId (
-  const wxChar*                     zId
+  const wxString&                     zId
 )
 {
     m_uFormat = ::WinAddAtom( ::WinQuerySystemAtomTable()
-                             ,(PSZ)zId
+                             ,zId.char_str()
                             );
 } // end of wxDataFormat::SetId
 
