@@ -4,7 +4,7 @@
 // Author:      David Webster
 // Modified by:
 // Created:     10/14/99
-// RCS-ID:      $Id: dialog.cpp,v 1.31 2006/08/20 13:29:42 SN Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) David Webster
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -25,7 +25,7 @@
 
 #include "wx/os2/private.h"
 #include "wx/evtloop.h"
-#include "wx/ptr_scpd.h"
+#include "wx/scopedptr.h"
 
 #define wxDIALOG_DEFAULT_X 300
 #define wxDIALOG_DEFAULT_Y 300
@@ -72,9 +72,9 @@ wxDEFINE_TIED_SCOPED_PTR_TYPE(wxDialogModalData);
 
 void wxDialog::Init()
 {
-    m_pOldFocus = (wxWindow *)NULL;
+    m_pOldFocus = NULL;
     m_isShown = false;
-    m_pWindowDisabler = (wxWindowDisabler *)NULL;
+    m_pWindowDisabler = NULL;
     m_modalData = NULL;
     SetBackgroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
 } // end of wxDialog::Init
@@ -147,7 +147,7 @@ void wxDialog::SetModal(bool WXUNUSED(bFlag))
 
 wxDialog::~wxDialog()
 {
-    m_isBeingDeleted = true;
+    SendDestroyEvent();
 
     // this will also reenable all the other windows for a modal dialog
     Show(false);
@@ -166,28 +166,6 @@ bool wxDialog::IsModalShowing() const
 
 #endif // WXWIN_COMPATIBILITY_2_6
 
-wxWindow *wxDialog::FindSuitableParent() const
-{
-    // first try to use the currently active window
-    HWND hwndFg = ::WinQueryActiveWindow(HWND_DESKTOP);
-    wxWindow *parent = hwndFg ? wxFindWinFromHandle((WXHWND)hwndFg)
-                              : NULL;
-    if ( !parent )
-    {
-        // next try the main app window
-        parent = wxTheApp->GetTopWindow();
-    }
-
-    // finally, check if the parent we found is really suitable
-    if ( !parent || parent == (wxWindow *)this || !parent->IsShown() )
-    {
-        // don't use this one
-        parent = NULL;
-    }
-
-    return parent;
-}
-
 bool wxDialog::Show( bool bShow )
 {
     if ( bShow == IsShown() )
@@ -202,16 +180,15 @@ bool wxDialog::Show( bool bShow )
         // and we will lose activation
         m_modalData->ExitLoop();
 #if 0
-        if (m_pWindowDisabler)
-        {
-            delete m_pWindowDisabler;
-            m_pWindowDisabler = NULL;
-        }
+        wxDELETE(m_pWindowDisabler);
 #endif
     }
 
     if (bShow)
     {
+        if (CanDoLayoutAdaptation())
+            DoLayoutAdaptation();
+
         // this usually will result in TransferDataToWindow() being called
         // which will change the controls values so do it before showing as
         // otherwise we could have some flicker
@@ -222,7 +199,7 @@ bool wxDialog::Show( bool bShow )
 
     wxString title = GetTitle();
     if (!title.empty())
-        ::WinSetWindowText((HWND)GetHwnd(), (PSZ)title.c_str());
+        ::WinSetWindowText((HWND)GetHwnd(), title.c_str());
 
     if ( bShow )
     {
@@ -243,7 +220,7 @@ bool wxDialog::Show( bool bShow )
 //
 int wxDialog::ShowModal()
 {
-    wxASSERT_MSG( !IsModal(), _T("wxDialog::ShowModal() reentered?") );
+    wxASSERT_MSG( !IsModal(), wxT("wxDialog::ShowModal() reentered?") );
 
     m_endModalCalled = false;
 
@@ -255,11 +232,7 @@ int wxDialog::ShowModal()
     if ( !m_endModalCalled )
     {
         // modal dialog needs a parent window, so try to find one
-        wxWindow *parent = GetParent();
-        if ( !parent )
-        {
-            parent = FindSuitableParent();
-        }
+        wxWindow * const parent = GetParentForModalDialog();
 
         // remember where the focus was
         wxWindow *oldFocus = m_pOldFocus;
@@ -311,7 +284,7 @@ void wxDialog::EndModal(
   int                               nRetCode
 )
 {
-    wxASSERT_MSG( IsModal(), _T("EndModal() called for non modal dialog") );
+    wxASSERT_MSG( IsModal(), wxT("EndModal() called for non modal dialog") );
 
     m_endModalCalled = true;
     SetReturnCode(nRetCode);
