@@ -4,15 +4,11 @@
 // Author:      William Osborne - minimal working wxPalmOS port
 // Modified by: Wlodzimierz ABX Skiba - native implementation
 // Created:     10/13/04
-// RCS-ID:      $Id: slider.cpp,v 1.12 2005/07/01 19:36:57 ABX Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) William Osborne, Wlodzimierz Skiba
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
 
-#if defined(__GNUG__) && !defined(NO_GCC_PRAGMA)
-#pragma implementation "slider.h"
-#endif
-
 // For compilers that support precompilation, includes "wx.h".
 #include "wx/wxprec.h"
 
@@ -22,13 +18,13 @@
 
 #if wxUSE_SLIDER
 
-#ifndef WX_PRECOMP
-#include "wx/utils.h"
-#include "wx/brush.h"
 #include "wx/slider.h"
-#endif
 
-#include "wx/toplevel.h"
+#ifndef WX_PRECOMP
+    #include "wx/utils.h"
+    #include "wx/brush.h"
+    #include "wx/toplevel.h"
+#endif
 
 #include <Form.h>
 #include <Control.h>
@@ -77,7 +73,7 @@ wxBEGIN_FLAGS( wxSliderStyle )
 
 wxEND_FLAGS( wxSliderStyle )
 
-IMPLEMENT_DYNAMIC_CLASS_XTI(wxSlider, wxControl,"wx/scrolbar.h")
+IMPLEMENT_DYNAMIC_CLASS_XTI(wxSlider, wxControl,"wx/slider.h")
 
 wxBEGIN_PROPERTIES_TABLE(wxSlider)
     wxEVENT_RANGE_PROPERTY( Scroll , wxEVT_SCROLL_TOP , wxEVT_SCROLL_ENDSCROLL , wxScrollEvent )
@@ -121,7 +117,7 @@ bool wxSlider::Create(wxWindow *parent, wxWindowID id,
     // wxSL_TOP is ignored - always off
     // wxSL_SELRANGE is ignored - always off
     // wxSL_VERTICAL is impossible in native form
-    wxCHECK_MSG(!(style & wxSL_VERTICAL), false, _T("non vertical slider on PalmOS"));
+    wxCHECK_MSG(!(style & wxSL_VERTICAL), false, wxT("non vertical slider on PalmOS"));
 
     if(!wxControl::Create(parent, id, pos, size, style, validator, name))
         return false;
@@ -139,6 +135,7 @@ bool wxSlider::Create(wxWindow *parent, wxWindowID id,
 
     AdjustForParentClientOrigin(x, y);
 
+#ifdef __WXPALMOS6__
     SliderControlType *slider = CtlNewSliderControl (
                                    (void **)&form,
                                    GetId(),
@@ -155,11 +152,22 @@ bool wxSlider::Create(wxWindow *parent, wxWindowID id,
                                    1,
                                    value
                               );
+#else // __WXPALMOS5__
+    //SliderControlType *CtlNewSliderControl (void **formPP, UInt16 ID, ControlStyleType style, DmResID thumbID,
+    //    DmResID backgroundID, Coord x, Coord y, Coord width, Coord height, UInt16 minValue, UInt16 maxValue,
+    //    UInt16 pageSize, UInt16 value);
+    SliderControlType *slider =  CtlNewSliderControl ((void **)&form,
+             GetId(),
+             feedbackSliderCtl,//style
+             0,//thumbID
+             0,//backgroundid
+             x, y, w, h, minValue, maxValue, 1, value);
+#endif // __WXPALMOS6__/__WXPALMOS5__
 
     if(slider==NULL)
         return false;
 
-    SetInitialBestSize(size);
+    SetInitialSize(size);
     Show();
     return true;
 }
@@ -312,7 +320,7 @@ bool wxSlider::SendUpdatedEvent()
     wxScrollEvent eventWxTrack(wxEVT_SCROLL_THUMBRELEASE, GetId());
     eventWxTrack.SetPosition(m_oldPos);
     eventWxTrack.SetEventObject(this);
-    bool handled = GetEventHandler()->ProcessEvent(eventWxTrack);
+    bool handled = HandleWindowEvent(eventWxTrack);
 
     // then slider event if position changed
     if( m_oldValue != m_oldPos )
@@ -343,7 +351,7 @@ bool wxSlider::SendScrollEvent(WXEVENTPTR event)
     wxScrollEvent eventWx(wxEVT_SCROLL_THUMBTRACK, GetId());
     eventWx.SetPosition(newPos);
     eventWx.SetEventObject(this);
-    return GetEventHandler()->ProcessEvent(eventWx);
+    return HandleWindowEvent(eventWx);
 }
 
 void wxSlider::Command (wxCommandEvent & event)
