@@ -4,7 +4,7 @@
 // Author:      William Osborne - minimal working wxPalmOS port
 // Modified by: Wlodzimierz ABX Skiba - more than minimal functionality
 // Created:     10/13/04
-// RCS-ID:      $Id: window.cpp,v 1.20 2006/08/31 19:31:23 ABX Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) William Osborne, Wlodzimierz Skiba
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -76,16 +76,13 @@
 #include "wx/notebook.h"
 #include "wx/listctrl.h"
 
+#ifndef __WXUNIVERSAL__
 #include <Window.h>
 
 // ---------------------------------------------------------------------------
 // global variables
 // ---------------------------------------------------------------------------
 
-#if wxUSE_MENUS_NATIVE
-wxMenu *wxCurrentPopupMenu = NULL;
-#endif // wxUSE_MENUS_NATIVE
-
 // ---------------------------------------------------------------------------
 // private functions
 // ---------------------------------------------------------------------------
@@ -247,7 +244,7 @@ bool wxGetKeyState(wxKeyCode key)
 
 void wxWindowPalm::Init()
 {
-    m_handle = 0;
+    m_hWnd = 0;
 }
 
 // Destructor
@@ -435,19 +432,12 @@ bool wxWindowPalm::Reparent(wxWindowBase *parent)
     return false;
 }
 
-void wxWindowPalm::Freeze()
-{
-}
-
-void wxWindowPalm::Thaw()
-{
-}
-
 void wxWindowPalm::Refresh(bool eraseBack, const wxRect *rect)
 {
-    WinHandle handle = (WinHandle)GetWinHandle();
+    WinHandle handle = (WinHandle)GetHWND();
     if(handle)
     {
+#ifdef __WXPALMOS6__
         if(rect)
         {
             RectangleType dirtyRect;
@@ -461,6 +451,10 @@ void wxWindowPalm::Refresh(bool eraseBack, const wxRect *rect)
         {
             WinInvalidateWindow(handle);
         }
+#else // __WXPALMOS5__
+        WinSetActiveWindow (handle);
+#endif
+
     }
 }
 
@@ -616,10 +610,11 @@ int wxWindowPalm::GetCharWidth() const
     return 0;
 }
 
-void wxWindowPalm::GetTextExtent(const wxString& string,
-                             int *x, int *y,
-                             int *descent, int *externalLeading,
-                             const wxFont *theFont) const
+void wxWindowPalm::DoGetTextExtent(const wxString& string,
+                                   int *x, int *y,
+                                   int *descent,
+                                   int *externalLeading,
+                                   const wxFont *theFont) const
 {
 }
 
@@ -754,5 +749,5 @@ bool wxWindowPalm::UnregisterHotKey(int hotkeyId)
 {
     return false;
 }
-
+#endif // # __WXUNIVERSAL__
 #endif // wxUSE_HOTKEY
