@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     25.08.00
-// RCS-ID:      $Id: bmpbuttn.cpp 42778 2006-10-30 22:11:26Z VZ $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2000 SciTech Software, Inc. (www.scitechsoft.com)
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -65,7 +65,7 @@ bool wxBitmapButton::Create(wxWindow *parent,
                            pos, size, style | wxBU_EXACTFIT, validator, name) )
         return false;
 
-    m_bmpNormal = bitmap;
+    m_bitmaps[State_Normal] = bitmap;
 
     return true;
 }
@@ -75,27 +75,24 @@ void wxBitmapButton::OnSetBitmap()
     wxBitmap bmp;
     if ( !IsEnabled() )
     {
-        bmp = m_bmpDisabled;
+        bmp = GetBitmapDisabled();
     }
     else if ( IsPressed() )
     {
-        bmp = m_bmpSelected;
+        bmp = GetBitmapPressed();
     }
     else if ( IsFocused() )
     {
-        bmp = m_bmpFocus;
-    }
-    else
-    {
-        bmp = m_bmpNormal;
+        bmp = GetBitmapFocus();
     }
+    //else: just leave it invalid, this means "normal" anyhow in ChangeBitmap()
 
     ChangeBitmap(bmp);
 }
 
 bool wxBitmapButton::ChangeBitmap(const wxBitmap& bmp)
 {
-    wxBitmap bitmap = bmp.Ok() ? bmp : m_bmpNormal;
+    wxBitmap bitmap = bmp.IsOk() ? bmp : GetBitmapLabel();
     if ( bitmap.IsSameAs(m_bitmap) )
         return false;
 
@@ -109,7 +106,7 @@ bool wxBitmapButton::Enable(bool enable)
     if ( !wxButton::Enable(enable) )
         return false;
 
-    if ( !enable && ChangeBitmap(m_bmpDisabled) )
+    if ( !enable && ChangeBitmap(GetBitmapDisabled()) )
         Refresh();
 
     return true;
@@ -117,14 +114,14 @@ bool wxBitmapButton::Enable(bool enable)
 
 bool wxBitmapButton::SetCurrent(bool doit)
 {
-    ChangeBitmap(doit ? m_bmpFocus : m_bmpNormal);
+    ChangeBitmap(doit ? GetBitmapFocus() : GetBitmapLabel());
 
     return wxButton::SetCurrent(doit);
 }
 
 void wxBitmapButton::OnSetFocus(wxFocusEvent& event)
 {
-    if ( ChangeBitmap(m_bmpFocus) )
+    if ( ChangeBitmap(GetBitmapFocus()) )
         Refresh();
 
     event.Skip();
@@ -132,7 +129,7 @@ void wxBitmapButton::OnSetFocus(wxFocusEvent& event)
 
 void wxBitmapButton::OnKillFocus(wxFocusEvent& event)
 {
-    if ( ChangeBitmap(m_bmpNormal) )
+    if ( ChangeBitmap(GetBitmapLabel()) )
         Refresh();
 
     event.Skip();
@@ -140,14 +137,14 @@ void wxBitmapButton::OnKillFocus(wxFocusEvent& event)
 
 void wxBitmapButton::Press()
 {
-    ChangeBitmap(m_bmpSelected);
+    ChangeBitmap(GetBitmapPressed());
 
     wxButton::Press();
 }
 
 void wxBitmapButton::Release()
 {
-    ChangeBitmap(IsFocused() ? m_bmpFocus : m_bmpNormal);
+    ChangeBitmap(IsFocused() ? GetBitmapFocus() : GetBitmapLabel());
 
     wxButton::Release();
 }
