@@ -4,7 +4,7 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     2005-01-18
-// RCS-ID:      $Id: stackwalk.cpp,v 1.12 2006/12/13 13:04:44 VZ Exp $
+// RCS-ID:      $Id$
 // Copyright:   (c) 2005 Vadim Zeitlin <vadim@wxwindows.org>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -67,7 +67,7 @@ public:
 private:
     FILE *m_fp;
 
-    DECLARE_NO_COPY_CLASS(wxStdioPipe)
+    wxDECLARE_NO_COPY_CLASS(wxStdioPipe);
 };
 
 // ============================================================================
@@ -92,13 +92,13 @@ void wxStackFrame::OnGetName()
     // format is: "module(funcname+offset) [address]" but the part in
     // parentheses can be not present
     wxString syminfo = wxString::FromAscii(m_syminfo);
-    const size_t posOpen = syminfo.find(_T('('));
+    const size_t posOpen = syminfo.find(wxT('('));
     if ( posOpen != wxString::npos )
     {
-        const size_t posPlus = syminfo.find(_T('+'), posOpen + 1);
+        const size_t posPlus = syminfo.find(wxT('+'), posOpen + 1);
         if ( posPlus != wxString::npos )
         {
-            const size_t posClose = syminfo.find(_T(')'), posPlus + 1);
+            const size_t posClose = syminfo.find(wxT(')'), posPlus + 1);
             if ( posClose != wxString::npos )
             {
                 if ( m_name.empty() )
@@ -249,12 +249,12 @@ int wxStackWalker::InitFrames(wxStackFrame *arr, size_t n, void **addresses, cha
             name = wxString::FromAscii(g_buf);
             name.RemoveLast(); // trailing newline
 
-            if ( name == _T("??") )
+            if ( name == wxT("??") )
                 name.clear();
         }
         else
         {
-            wxLogDebug(_T("cannot read addr2line output for stack frame #%lu"),
+            wxLogDebug(wxT("cannot read addr2line output for stack frame #%lu"),
                        (unsigned long)i);
             return false;
         }
@@ -265,7 +265,7 @@ int wxStackWalker::InitFrames(wxStackFrame *arr, size_t n, void **addresses, cha
             filename = wxString::FromAscii(g_buf);
             filename.RemoveLast();
 
-            const size_t posColon = filename.find(_T(':'));
+            const size_t posColon = filename.find(wxT(':'));
             if ( posColon != wxString::npos )
             {
                 // parse line number (it's ok if it fails, this will just leave
@@ -274,13 +274,13 @@ int wxStackWalker::InitFrames(wxStackFrame *arr, size_t n, void **addresses, cha
 
                 // remove line number from 'filename'
                 filename.erase(posColon);
-                if ( filename == _T("??") )
+                if ( filename == wxT("??") )
                     filename.clear();
             }
             else
             {
-                wxLogDebug(_T("Unexpected addr2line format: \"%s\" - ")
-                           _T("the semicolon is missing"),
+                wxLogDebug(wxT("Unexpected addr2line format: \"%s\" - ")
+                           wxT("the semicolon is missing"),
                            filename.c_str());
             }
         }
